# shader generator
# generates all combinations of shaders using vsdrgen.pl and psdrgen.pl

@skin = ("--skin", "--noskin");
@light_type = ("--ambient", "--omni", "--spot", "--directional");
@shadows = ("--shadows", "--noshadows");
@fog = ("--fog", "--nofog");
@shading = ("--flat", "--normalmap", "--parallax", "--relief");

foreach (@skin)
{
	$sk = $_;
	foreach (@light_type)
	{
		$lt = $_;
		foreach (@shadows)
		{
			$sdw = $_;
			foreach (@fog)
			{
				$fg = $_;
				foreach (@shading)
				{
					$sdg = $_;
					create_vertex_shader($sk, $lt, $sdw, $fg, $sdg);
				}
			}
		}
	}
}

sub create_vertex_shader
{
	my $filename = "vertex/" . get_name(@_[0], @_[1], @_[2], @_[3], @_[4]) . "_v.cg";
	my $cmd = "vsdrgen.pl " . join(' ', @_);
	print "$cmd:\n";

	# write vsdrgen parameters to a tmp file
	open(TMP, ">tmp");
	print TMP join(' ', @_);
	close(TMP);

	open(OUTFILE, ">" . $filename);
	open(VSG, "vsdrgen.pl < tmp |");
	while (my $ln = <VSG>)
	{
		print OUTFILE $ln;
	}
	close(VSG);
	close(OUTFILE);

	unlink("tmp");
	
	# try to compile the shader
	my $cgc = "cgc -profile vs_2_0 $filename";
	open(CGC, $cgc . "|");
	while ($ln = <CGC>)
	{
		if (index($ln, "errors") >= 0)
		{
			print $ln;
		}
	}
	close(CGC);
}

sub get_name
{
	my $skin = "n";
	my $light_type = "o";
	my $shadows = "n";
	my $fog = "n";
	my $shading = "f";

	if (@_[0] eq "--skin") {$skin = "s";}
	if (@_[1] eq "--ambient") {$light_type = "a";}
	if (@_[1] eq "--spot") {$light_type = "s";}
	if (@_[1] eq "--directional") {$light_type = "d";}
	if (@_[2] eq "--shadows") {$shadows = "s";}
	if (@_[3] eq "--fog") {$fog = "f";}
	if (@_[4] eq "--normalmap") {$shading = "n";}
	if (@_[4] eq "--parallax") {$shading = "p";}
	if (@_[4] eq "--relief") {$shading = "r";}

	return $skin . $light_type . $shadows . $fog . $shading;
}
