# vsdrgen.pl

####################################################################
# settings
####################################################################
$skin = "no";
$light_type = "omni";
$shadows = "no";
$fog = "no";
$shading = "flat";

####################################################################
# script
####################################################################
if (!parse_cmdline(@ARGV))
{
	print_help();
	exit;
}

# print comments
print "// Vertex shader\n";
print "// Automatically generated\n";
print "// Skin: $skin\n";
print "// Light type: $light_type\n";
print "// Shadows: $shadows\n";
print "// Fog: $fog\n";
print "// Shading: $shading\n\n";

# input struct ###################################
print "// input struct\n";
print "struct a2v\n";
print "{\n";
print "\tfloat3 position : POSITION;\n";
print "\tfloat3 normal : NORMAL;\n";
if ($skin eq "yes")
{
	print "\tfloat4 blend_indices : BLENDINDICES;\n";
	print "\tfloat4 blend_weights : BLENDWEIGHT;\n";
}
print "\tfloat2 texcoord : TEXCOORD0;\n";
print "\tfloat4 col : COLOR0;\n";
print "};\n\n";

# output struct ##################################
print "// output struct\n";
print "struct v2f\n";
print "{\n";
print "\tfloat4 position : POSITION;\n";
print "\tfloat4 col : COLOR0;\n";
print "\tfloat2 texcoord : TEXCOORD0;\n";
print "\tfloat3 vpos : TEXCOORD1;\n";
print "\tfloat3 vnor : TEXCOORD2;\n";


if ($shading != "flat")
{
	print "\tfloat3 vtgt : TEXCOORD3;\n";
}
if ($shadows eq "yes")
{
	print "\tfloat3 wpos : TEXCOORD4;\n";
}
print "};\n\n";

# uniforms ######################################
print "// uniforms\n";
print "uniform float4x4 STATE_MODELVIEWPROJECTION_MATRIX;\n";
print "uniform float4x4 STATE_MODEL_MATRIX;\n";
print "uniform float4x4 STATE_VIEW_MATRIX;\n";
if ($skin eq "yes")
{
	print "uniform float3x4 STATE_BONE_MATRIX[66];\n";
}
print "\n";

# main function #################################
print "v2f main(a2v IN)\n";
print "{\n";
print "\tv2f OUT;\n";
print "\tfloat4x4 modelview = mul(STATE_MODEL_MATRIX, STATE_VIEW_MATRIX);\n\n";
if ($skin eq "yes")
{
	print "\tfloat3 net_position = float3(0, 0, 0);\n";
	print "\tfloat3 net_normal = float3(0, 0, 0);\n";
	if ($shading != "flat")
	{
		print "\tfloat3 net_tangent = float3(0, 0, 0);\n";
	}
	print "\n";

	print "\t// loop through all weights\n";
	print "\tfor (int i=0; i<3; i++)\n";
	print "\t{\n";
	print "\t\t// get the current bone matrix\n";
	print "\t\tfloat index =  IN.blend_indices[i];\n";
	print "\t\tfloat3x4 curr_bone_matrix = STATE_BONE_MATRIX[index];\n\n";
	print "\t\t// calculate vertex relative to the current bone\n";
	print "\t\tfloat3 curr_pos = mul(curr_bone_matrix, float4(IN.position, 1.0)).xyz;\n";
	print "\t\tfloat3 curr_normal = mul((float3x3)curr_bone_matrix, IN.normal);\n";
	if ($shading != "flat")
	{
		print "\t\tfloat3 curr_tangent = mul((float3x3)curr_bone_matrix, IN.tangent);\n";
	}
	print "\n";

	print "\t\t// sum the results\n";
	print "\t\tnet_position += IN.blend_weights[i]* curr_pos;\n";
	print "\t\tnet_normal += IN.blend_weights[i] * curr_normal;\n";
	if ($shading != "flat")
	{
		print "\t\tnet_tangent += IN.blend_weights[i] * curr_tangent;\n";
	}
	print "\n";
	
	print "\t}\n";
}
else
{
	print "\tfloat3 net_position = IN.position;\n";
	print "\tfloat3 net_normal = IN.normal;\n";
	if ($shading != "flat")
	{
		print "\tnet_tangent = IN.tangent;\n";
	}
}
print "\n";

# output stuff
print "\tOUT.position = mul(float4(net_position, 1), STATE_MODELVIEWPROJECTION_MATRIX);\n";
print "\tOUT.texcoord = IN.texcoord;\n";
print "\tOUT.vpos = mul(float4(net_position, 1), modelview).xyz;\n";
print "\tOUT.vnor = mul(net_normal, (float3x3)modelview);\n";
print "\tOUT.col = IN.col;\n";
if ($shadows eq "yes")
{
	print "\tOUT.wpos = mul(float4(IN.position, 1), STATE_MODEL_MATRIX).xyz;\n";
}
print "\treturn OUT;\n";
print "}\n";

####################################################################
# subroutines
####################################################################
sub print_help
{
	print "Usage: vsdrgen [parameters]\n";
	print "parameter list:\n\n";
	print "Skinned mesh: --skin, --noskin\n";
	print "Light type: --ambient, --omni, --spot, --directional\n";
	print "Shadows: --shadows, --noshadows\n";
	print "Fog: --fog, --nofog\n";
	print "Shading: --flat, --normalmap, --parallax, --relief\n\n";
	print "Defaults are: --noskin --omni --noshadows --nofog --flat\n";
	print "Print this help message and exit: --help\n";
}

sub parse_cmdline
{
	my $ret = 1;
	my @argv = @_;
	if (!@argv)
	{
		open(TMP, "tmp");
		@argv = split(' ', <TMP>);
		close(TMp);
	}
	for (my $i=0; $i<@argv; $i++)
	{
		my $str = @argv[$i];

		if ($str eq "--skin") {$skin = "yes";}
		if ($str eq "--noskin") {$skin = "no";}

		if ($str eq "--ambient") {$light_type = "ambient";}
		if ($str eq "--omni") {$light_type = "omni";}
		if ($str eq "--spot") {$light_type = "spot";}
		if ($str eq "--directional") {$light_type = "directional";}

		if ($str eq "--shadows") {$shadows = "yes";}
		if ($str eq "--noshadows") {$shadows = "no";}

		if ($str eq "--fog") {$fog = "yes";}
		if ($str eq "--nofog") {$fog = "no";}

		if ($str eq "--flat") {$shading = "flat";}
		if ($str eq "--normalmap") {$shading = "normalmap";}
		if ($str eq "--parallax") {$shading = "parallax";}
		if ($str eq "--relief") {$shading = "relief";}

		if ($str eq "--help")
		{
			$ret = 0;
		}
	}

	return $ret;
}
