// anm.cpp

#include "anm.h"

// KeyframeChunk class implementation
KeyframeChunk::KeyframeChunk()
{
	Resize(sizeof(KeyframeData));
	SetNodeID(0);
	SetKeyframeCount(0);
}
KeyframeChunk::~KeyframeChunk()
{

}

unsigned long KeyframeChunk::GetNodeID() const
{
	return ((KeyframeData*)GetData())->node_id;
}
void KeyframeChunk::SetNodeID(unsigned long node_id)
{
	((KeyframeData*)GetData())->node_id = node_id;
}

unsigned long KeyframeChunk::GetKeyframeCount() const
{
	return ((KeyframeData*)GetData())->keyframe_count;
}
void KeyframeChunk::SetKeyframeCount(unsigned long count)
{
	((KeyframeData*)GetData())->keyframe_count = count;
	Resize(sizeof(KeyframeData) + count * sizeof(PRS));
}

PRS* KeyframeChunk::GetKeyframes()
{
	return (PRS*)(((char*)GetData()) + sizeof(KeyframeData));
}

PRS KeyframeChunk::GetDefaultPRS() const
{
	return ((KeyframeData*) GetData())->default_local_prs;
}
void KeyframeChunk::SetDefaultPRS(const PRS &default_prs)
{
	((KeyframeData*) GetData())->default_local_prs = default_prs;
}
// AnimationFile class implementation
AnimationFile::AnimationFile()
{
	Resize(sizeof(AnimationHeader));
	SetVersion(100);
	SetMessage("Track7 animation format version 1.0");
}
AnimationFile::~AnimationFile()
{

}

unsigned long AnimationFile::GetVersion() const
{
	return ((AnimationHeader*)GetData())->version;
}
void AnimationFile::SetVersion(unsigned long version)
{
	((AnimationHeader*)GetData())->version = version;
}

std::string AnimationFile::GetMessage() const
{
	return GetName();
}
void AnimationFile::SetMessage(const std::string& message)
{
	SetName(message);
}

unsigned long AnimationFile::GetFPS() const
{
	return ((AnimationHeader*)GetData())->fps;
}
void AnimationFile::SetFPS(unsigned long fps)
{
	((AnimationHeader*)GetData())->fps = fps;
}

// high level
unsigned long AnimationFile::GetKeyframeChunkCount() const
{
	return (unsigned long)GetIDChildren(ANM_KEYFRAME_CHUNK).size();
}

KeyframeChunk* AnimationFile::GetKeyframeChunk(unsigned long id) const
{
	ChunkList cl = GetIDChildren(ANM_KEYFRAME_CHUNK);

	for (unsigned long i=0; i<cl.size(); i++)
	{
		if (((KeyframeChunk*)cl[i])->GetNodeID() == id)
		{
			return (KeyframeChunk*)cl[i];
		}
	}

	return 0;
}
