// chunkff.h
// chunk file format handling utility
#ifndef _CHUNKFF_HEADER_
#define _CHUNKFF_HEADER_

#include <libvfs.h>
#include <vector>
#include <string>
#include <byteorder.h>

typedef unsigned int BOOL32;
#ifndef TRUE
#define TRUE	1
#endif

#ifndef FALSE
#define FALSE	0
#endif

// fwd declaration
class Chunk;

typedef std::vector<Chunk*> ChunkList;

#define STRING_CHUNK	0xFFFFFFFF
#define NAME_CHUNK		0xFFFFFFFE
#define FILE_CHUNK		0xFFFFFFFD

class Chunk
{
private:
	unsigned long id;
	
	unsigned long size;
	void* data;

	Chunk* parent;
	ChunkList children;

protected:
	void Resize(unsigned long size);
	void Expand(unsigned long bytes);
	void Shrink(unsigned long bytes);

public:
	//void Resize(unsigned long size);
	Chunk();
	Chunk(VFS_FILE* fp);
	virtual ~Chunk();

	virtual unsigned int GetID() const;
	void SetID(unsigned long id);

	void Destroy();

	bool Load(VFS_FILE *fp);
	bool LoadFile(const char *fname);
	bool Save(VFS_FILE *fp) const;
	bool SaveFile(const char *fname) const;	

	// flips the endianness of the datablock
	void FlipEndian();

	unsigned long GetSize() const;
	void* GetData() const;
	
	// this performs a copy
	void SetData(unsigned long size, void* data);

	Chunk* GetParent() const;
	ChunkList GetChildren() const;
	ChunkList GetIDChildren(unsigned long id) const;
	// returns the first child with specified ID (or zero)
	Chunk* GetIDChild(unsigned long id) const;
	ChunkList GetIDChildrenRec(unsigned long id) const;

	bool Mount(Chunk* child);
	bool Unmount(Chunk* child);
	bool Delete(Chunk* child);

	// info functions
	std::string GetInfo();
	virtual std::string GetDetails() const;
}; // end class Chunk

class stringChunk : public Chunk
{
private:
protected:	
public:
	// constructor
	stringChunk();
	stringChunk(std::string str);
	virtual ~stringChunk();
	
	virtual unsigned int GetID() const
	{
		return STRING_CHUNK;
	}

	std::string Getstring() const;
	void Setstring(const std::string& str);

	// info functions
	virtual std::string GetDetails() const;
}; // end class stringChunk

class NameChunk : public stringChunk
{
private:
protected:
public:
	virtual ~NameChunk() {}
	virtual unsigned int GetID() const
	{
		return NAME_CHUNK;
	}

	// info functions
	virtual std::string GetDetails() const;
};

class NamedChunk : public Chunk
{
private:
protected:
public:
	std::string GetName() const;
	void SetName(const std::string& str);

	Chunk* GetChildByName(const std::string& name) const;
	Chunk* GetChildByNameRecursive(const std::string& name) const;

	// info functions
	virtual std::string GetDetails() const;
}; // end class NamedChunk

class FileChunk : public NamedChunk
{
private:
protected:
public:
	virtual ~FileChunk() {}
	virtual unsigned int GetID() const
	{
		return FILE_CHUNK;
	}
}; // end class FileChunk

#endif // ndef _CHUNKFF_HEADER_
