// file.h

#ifndef _FILE_HEADER_
#define _FILE_HEADER_

#include "libvfs.h"
#include "byteorder.h"
#include <string.h>

template <class T> void FlipEndian(T *p)
{
	char *bytes = new char[sizeof(T)];
	
	char *var = (char*) p;

	for (size_t i=0; i<sizeof(T); i++)
	{
		bytes[i] = var[sizeof(T) - i - 1];
	}
	for (size_t i=0; i<sizeof(T); i++)
	{
		var[i] = bytes[i];
	}

	delete [] bytes;
}


#ifdef LITTLE_ENDIAN

template <class T> bool Read(T* p, unsigned long count, VFS_FILE* fp)
{
	if (!fp || vfs_eof(fp)) return false;
	if (vfs_read(p, sizeof(T), count, fp) != count) return false;
	return true;
}

template <class T> bool Write(T* p, unsigned long count, VFS_FILE* fp)
{
	if (!fp) return false;
	if (vfs_write(p, sizeof(T), count, fp) != count) return false;
	return true;
}

#else

// big endian routines
template <class T> bool Read(T* p, unsigned long count, VFS_FILE* fp)
{
	if (!fp || vfs_eof(fp)) return false;
	if (vfs_read(p, sizeof(T), count, fp) != count) return false;

	for (int i=0; i<count; i++)
	{
		FlipEndian<T>(p + i);
	}

	return true;
}

template <class T> bool Write(T* p, unsigned long count, VFS_FILE* fp)
{
	if (!fp) return false;

	T *tmp = new T(count);
	memcpy(tmp, p, sizeof(T) * count);

	for (unsigned int i=0; i<count; i++)
	{
		FlipEndian<T(tmp + i);
	}

	if (vfs_write(tmp, sizeof(T), count, fp) != count)
	{
		delete [] tmp;
		return false;
	}

	delete [] tmp;
	return true;
}


#endif // LITTLE_ENDIAN

#endif // ndef _FILE_HEADER_
