#ifndef __PACK_H__
#define __PACK_H__

#include <vector>

namespace serialization
{

	template <typename T>
	void pack(void *packed_data, T *unpacked_data);

	template <typename T>
	void unpack(T *unpacked_data, void *packed_data);

	template <typename T>
	int packed_sizeof(const T &);

	template <typename T>
	class Packer;

	template <typename T>
	class Packer
	{
	private:
		struct Segment {
			int size;
			int offset;
		};
		int num_segments;
		int packed_size;
		Segment *segments;
	public:
		static Packer<T> descriptor;

		Packer();
		~Packer();

		friend void pack<T>(void *packed_data, T *unpacked_data);
		friend void unpack<T>(T *unpacked_data, void *packed_data);
		friend int packed_sizeof<T>(const T &);
	};

	template <typename T>
	Packer<T> Packer<T>::descriptor;

	template <typename T>
	Packer<T>::Packer(): segments(0), num_segments(0), packed_size(0)
	{
		std::vector<int> offsets;
		std::vector<int> sizes;

		T obj1, obj2;
		char *obj1_buff = (char*)&obj1;
		char *obj2_buff = (char*)&obj2;
		memset(obj1_buff, 0, sizeof(T));
		memset(obj2_buff, 0, sizeof(T));

		int part_size = 0;
		bool padding = false;
		offsets.push_back(0);
		for (int i=0; i<sizeof(T); i++) {
			obj1_buff[i] = 0xff;
			if (obj1 == obj2) {
				if (!padding) {
					padding = true;
					sizes.push_back(part_size);
					part_size = 0;
				}
			}
			else {
				if (padding) {
					offsets.push_back(i);
					padding = false;
				}
				obj1_buff[i] = 0x00;
				part_size++;
				packed_size++;
			}
		}

		if (!padding) {
			sizes.push_back(part_size);
		}

		segments = (Segment*)malloc(sizeof(Segment)*offsets.size());
		num_segments = offsets.size();
		for (int i=0; i<num_segments; i++) {
			segments[i].offset = offsets[i];
			segments[i].size   = sizes[i];
		}
	}

	template <typename T>
	Packer<T>::~Packer()
	{
		free(segments);
		segments = 0;
	}

	template <typename T>
	void pack(void *packed_data, T *unpacked_data)
	{
		int packed_offset = 0;
		for (int i=0; i<Packer<T>::descriptor.num_segments; i++) {
			memcpy(((char*)packed_data)   + packed_offset,
				((char*)unpacked_data) + Packer<T>::descriptor.segments[i].offset,
				Packer<T>::descriptor.segments[i].size);
			packed_offset += Packer<T>::descriptor.segments[i].size;
		}
	}

	template <typename T>
	void unpack(T *unpacked_data, void *packed_data)
	{
		int packed_offset = 0;
		for (int i=0; i<Packer<T>::descriptor.num_segments; i++) {
			memcpy(((char*)unpacked_data) + Packer<T>::descriptor.segments[i].offset,
				((char*)packed_data)   + packed_offset,
				Packer<T>::descriptor.segments[i].size);
			packed_offset += Packer<T>::descriptor.segments[i].size;
		}
	}

	template <typename T>
	int packed_sizeof(const T &)
	{
		return Packer<T>::descriptor.packed_size;
	}

	class TypeConverter
	{
	public:
		template <typename T>
		operator T() {return T();}
	};

	extern TypeConverter *pap;

	template<typename T>
	T operator * (const T &t, const TypeConverter *pa)
	{
		return T();
	}

}

#define packed_sizeof(a) packed_sizeof((a)*serialization::pap)


#endif
