// sce.cpp

#include "sce.h"

static unsigned long bt_size[] = 
{
	1,		// SCE_BT_INT8
	2,		// SCE_BT_INT16
	4,		// SCE_BT_INT32
	4,		// SCE_BT_FLOAT
	8,		// SCE_BT_DOUBLE
};

// CameraChunk class implementation
CameraChunk::CameraChunk()
{
	Resize(sizeof(CameraData));
}
CameraChunk::~CameraChunk()
{

}

bool CameraChunk::IsTarget() const
{
	return ((CameraData*)GetData())->target == TRUE ? true:false;
}
void CameraChunk::SetTarget(bool target)
{
	((CameraData*)GetData())->target = target ? TRUE : FALSE;
}

float CameraChunk::GetFOV() const
{
	return ((CameraData*)GetData())->fov;
}
void CameraChunk::SetFOV(float fov)
{
	((CameraData*)GetData())->fov = fov;
}

float CameraChunk::GetNearClip() const
{
	return ((CameraData*)GetData())->near_clip;
}
void CameraChunk::SetNearClip(float near_clip)
{
	((CameraData*)GetData())->near_clip = near_clip;
}

float CameraChunk::GetFarClip() const
{
	return ((CameraData*)GetData())->far_clip;
}
void CameraChunk::SetFarClip(float far_clip)
{
	((CameraData*)GetData())->far_clip = far_clip;
}

unsigned long CameraChunk::GetTargetNodeID() const
{
	return ((CameraData*)GetData())->target_node_id;
}
void CameraChunk::SetTargetNodeID(unsigned long target_node_id)
{
	((CameraData*)GetData())->target_node_id = target_node_id;
}

// LightChunk class implementation
LightChunk::LightChunk()
{
	Resize(sizeof(LightData));
}
LightChunk::~LightChunk()
{

}

LightChunkType LightChunk::GetType() const
{
	return ((LightData*)GetData())->type;
}
void LightChunk::SetType(LightChunkType type)
{
	((LightData*)GetData())->type = type;
}

Vector4 LightChunk::GetColor() const
{
	return ((LightData*)GetData())->color;
}
void LightChunk::SetColor(const Vector4& color)
{
	((LightData*)GetData())->color = color;
}

float LightChunk::GetMultiplier() const
{
	return ((LightData*)GetData())->multiplier;
}
void LightChunk::SetMultiplier(float multiplier)
{
	((LightData*)GetData())->multiplier = multiplier;
}

float LightChunk::GetAspectRatio() const
{
	return ((LightData*)GetData())->aspect_ratio;
}
void LightChunk::SetAspectRatio(float aspect_ratio)
{
	((LightData*)GetData())->aspect_ratio = aspect_ratio;
}

float LightChunk::GetFallOff() const
{
	return ((LightData*)GetData())->falloff;
}
void LightChunk::SetFallOff(float falloff)
{
	((LightData*)GetData())->falloff = falloff;
}

float LightChunk::GetHotSpot() const
{
	return ((LightData*)GetData())->hot_spot;
}
void LightChunk::SetHotSpot(float hot_spot)
{
	((LightData*)GetData())->hot_spot = hot_spot;
}

unsigned long LightChunk::GetTargetNodeID() const
{
	return ((LightData*)GetData())->target_node_id;
}
void LightChunk::SetTargetNodeID(unsigned long target_node_id)
{
	((LightData*)GetData())->target_node_id = target_node_id;
}

LightDecay LightChunk::GetLightDecay() const
{
	return ((LightData*)GetData())->decay;
}
void LightChunk::SetLightDecay(LightDecay decay)
{
	((LightData*)GetData())->decay = decay;
}

SpotLightType LightChunk::GetSpotLightType() const
{
	return ((LightData*)GetData())->spot_light_type;
}
void LightChunk::SetSpotLightType(SpotLightType slt)
{
	((LightData*)GetData())->spot_light_type = slt;
}

float LightChunk::GetAttenStart() const
{
	return ((LightData*)GetData())->atten_start;
}
void LightChunk::SetAttenStart(float start)
{
	((LightData*)GetData())->atten_start = start;
}

float LightChunk::GetAttenEnd() const
{
	return ((LightData*)GetData())->atten_end;
}
void LightChunk::SetAttenEnd(float end)
{
	((LightData*)GetData())->atten_end = end;
}

// TextureChunk class implementation
TextureChunk::TextureChunk()
{
	Resize(sizeof(TextureData));
}
TextureChunk::~TextureChunk()
{

}

SceTextureType TextureChunk::GetType() const
{
	return ((TextureData*)GetData())->type;
}
void TextureChunk::SetType(SceTextureType type)
{
	((TextureData*)GetData())->type = type;
}

// MaterialChunk class implementation
MaterialChunk::MaterialChunk()
{
	Resize(sizeof(MaterialData));
	Vector4 c(0.5f, 0.5f, 0.5f, 1.0f);
	SetAmbient(c);
	SetDiffuse(c);
	SetSpecular(c);
	SetEmissive(c);
}

MaterialChunk::~MaterialChunk()
{

}

Vector4 MaterialChunk::GetAmbient() const
{
	return ((MaterialData*)GetData())->ambient;
}

void MaterialChunk::SetAmbient(const Vector4& ambient)
{
	((MaterialData*)GetData())->ambient = ambient;
}

Vector4 MaterialChunk::GetDiffuse() const
{
	return ((MaterialData*)GetData())->diffuse;
}

void MaterialChunk::SetDiffuse(const Vector4& diffuse)
{
	((MaterialData*)GetData())->diffuse = diffuse;
}

Vector4 MaterialChunk::GetEmissive() const
{
	return ((MaterialData*)GetData())->emissive;
}

void MaterialChunk::SetEmissive(const Vector4& emissive)
{
	((MaterialData*)GetData())->emissive = emissive;
}

Vector4 MaterialChunk::GetSpecular() const
{
	return ((MaterialData*)GetData())->specular;
}

void MaterialChunk::SetSpecular(const Vector4& specular)
{
	((MaterialData*)GetData())->specular = specular;
}

// high level
TextureChunk* MaterialChunk::GetTexture(SceTextureType type)
 {
	int i=0;
	if (type == STT_NORMAL)
	{
		i++;
	}

	ChunkList tl = GetIDChildren(SCE_TEXTURE);
	for (size_t i=0; i<tl.size(); i++)
	{
		if (((TextureChunk*)tl[i])->GetType() == type)
		{
			return (TextureChunk*)tl[i];
		}
	}

	// texture not found - add one
	TextureChunk *new_tc = new TextureChunk();
	Mount(new_tc);
	new_tc->SetType(type);
	return new_tc;
}

// GeometryArray class implementation
GeometryArray::GeometryArray(GeometryArrayType type, SceBasicType basic_type,
	unsigned long basic_type_count, unsigned long element_count)
{
	//calculate needed memory
	unsigned long bytes_needed = sizeof(GeometryArrayData) 
		+ bt_size[basic_type] * basic_type_count * element_count;

	// allocate memory
	Resize(bytes_needed);

	// fill data
	((GeometryArrayData*)GetData())->type = type;
	((GeometryArrayData*)GetData())->basic_type = basic_type;
	((GeometryArrayData*)GetData())->basic_type_count = basic_type_count;
	((GeometryArrayData*)GetData())->element_count = element_count;
}

GeometryArray::~GeometryArray()
{

}

GeometryArrayType GeometryArray::GetType() const
{
	return ((GeometryArrayData*)GetData())->type;
}

SceBasicType GeometryArray::GetBasicType() const
{
	return ((GeometryArrayData*)GetData())->basic_type;
}

unsigned long GeometryArray::GetBasicTypeCount() const
{
	return ((GeometryArrayData*)GetData())->basic_type_count;
}
unsigned long GeometryArray::GetElementCount() const
{
	return ((GeometryArrayData*)GetData())->element_count;
}

void* GeometryArray::GetArrayData()
{
	if (!GetData() || !GetElementCount() || !GetBasicTypeCount())
	{
		return 0;
	}
	return (void*)(((char*)GetData()) + sizeof(GeometryArrayData));
}

// MeshChunk class implementation
MeshChunk::MeshChunk()
{
	Resize(sizeof(MeshData));
}

MeshChunk::~MeshChunk()
{

}

bool MeshChunk::IsProxy() const
{
	return ((MeshData*)GetData())->proxy == TRUE ? true:false;
}

void MeshChunk::SetProxy(bool proxy)
{
	((MeshData*)GetData())->proxy = proxy ? TRUE : FALSE;
}

bool MeshChunk::IsPVS() const
{
	return ((MeshData*)GetData())->pvs == TRUE ? true:false;
}
void MeshChunk::SetPVS(bool pvs)
{
	((MeshData*)GetData())->pvs = pvs ? TRUE : FALSE;
}

bool MeshChunk::IsMesh() const
{
	return ((MeshData*)GetData())->mesh == TRUE ? true:false;
}
void MeshChunk::SetMesh(bool mesh)
{
	((MeshData*)GetData())->mesh = mesh ? TRUE : FALSE;
}

bool MeshChunk::IsLink() const
{
	return ((MeshData*)GetData())->link == TRUE ? true:false;
}
void MeshChunk::SetLink(bool link) const
{
	((MeshData*)GetData())->link = link ? TRUE : FALSE;
}

unsigned long MeshChunk::GetMaterialID() const
{
	return ((MeshData*)GetData())->material_id;
}

void MeshChunk::SetMaterialID(unsigned long material_id)
{
	((MeshData*)GetData())->material_id = material_id;
}

bool MeshChunk::IsIndexed() const
{
	if (!GetArray(INDEX_ARRAY))
	{
		return false;
	}

	return true;
}

GeometryArray* MeshChunk::GetArray(GeometryArrayType type) const
{
	ChunkList arrays = GetIDChildren(SCE_GEOMETRY_ARRAY);
	for (unsigned int i=0; i<arrays.size(); i++)
	{
		if (((GeometryArray*)arrays[i])->GetType() == type)
		{
			return (GeometryArray*)arrays[i];
		}
	}

	return 0;
}

bool MeshChunk::CalculateNormals()
{
	// get the vertices
	GeometryArray *vertex_array = GetArray(POSITION_ARRAY);
	if (!vertex_array) return false;
	Vector3 *varr = (Vector3*)vertex_array->GetArrayData();
	unsigned int vcount = vertex_array->GetElementCount();

	// support only indexed mesh for now
	GeometryArray *index_array = GetArray(INDEX_ARRAY);
	if (!index_array) return false;
	unsigned int *iarr = (unsigned int*) index_array->GetArrayData();
	unsigned int icount = index_array->GetElementCount();
	
	// calculate triangle normals and
	// detect triangles that share each vertex
	unsigned int num_tris = icount / 3;
	Vector3 *tri_normals = new Vector3[num_tris];
	std::vector<unsigned int> *v_tris = new std::vector<unsigned int>[vcount];
	for (unsigned int i=0; i<num_tris; i++)
	{
		Vector3 a = varr[iarr[i * 3 + 2]] - varr[iarr[i * 3 + 0]];
		Vector3 b = varr[iarr[i * 3 + 1]] - varr[iarr[i * 3 + 0]];
		tri_normals[i] = CrossProduct(a, b);

		v_tris[iarr[i * 3 + 0]].push_back(i);
		v_tris[iarr[i * 3 + 1]].push_back(i);
		v_tris[iarr[i * 3 + 2]].push_back(i);
	}

	// discard existing normals (if any)
	GeometryArray *normal_array;
	while ((normal_array = GetArray(NORMAL_ARRAY)))
	{
		Unmount(normal_array);
		delete normal_array;
	}

	normal_array = new GeometryArray(NORMAL_ARRAY, SCE_BT_FLOAT, 3, vcount);
	Mount(normal_array);

	Vector3 *narr = (Vector3*) normal_array->GetArrayData();

	// calculate vertex normals
	for (unsigned int i=0; i<vcount; i++)
	{
		Vector3 norm(0, 0, 0);
		for (unsigned int j=0; j<v_tris[i].size(); j++)
		{
			norm += tri_normals[v_tris[i][j]];
		}
		narr[i] = norm.Normalized();
	}

	// cleanup
	delete [] tri_normals;
	delete [] v_tris;
	
	return true;
}

static void CalculateTangent (Vector3 v1, Vector3 v2, Vector3 v3,
	Vector2 tex1, Vector2 tex2, Vector2 tex3,
	Vector3 &uTangent, Vector3 &vTangent)
{
    Vector3 side0 = v1 - v2;
    Vector3 side1 = v3 - v2;

    //Calculate normal
    Vector3 normal = CrossProduct (side1, side0);
    if (fabs (normal.Length()) > 0.000001f)
    {
        normal.Normalize();
    }

    float deltaT0 = tex1.y - tex2.y;
    float deltaT1 = tex3.y - tex2.y;
    uTangent = deltaT1*side0 - deltaT0*side1;
    if (fabs (uTangent.Length()) > 0.000001f)
    {
        uTangent.Normalize();
    }

   //Calculate t tangent
    float deltaS0 = tex1.x - tex2.x;
    float deltaS1 = tex3.x - tex2.x;
    vTangent = deltaS1*side0 - deltaS0*side1;
    if (fabs (vTangent.Length()) > 0.000001f)
    {
        vTangent.Normalize();
    }
    //reverse tangents if necessary

    Vector3 tangentCross = CrossProduct(uTangent, vTangent);
    if(DotProduct(tangentCross, normal) < 0.0f)
    {
        uTangent = -uTangent;
        vTangent = -vTangent;
    }
}

bool MeshChunk::CalculateTangents()
{
	// get the vertices
	GeometryArray *vertex_array = GetArray(POSITION_ARRAY);
	if (!vertex_array) return false;
	Vector3 *varr = (Vector3*)vertex_array->GetArrayData();
	unsigned int vcount = vertex_array->GetElementCount();

	// support only indexed mesh for now
	GeometryArray *index_array = GetArray(INDEX_ARRAY);
	if (!index_array) return false;
	unsigned int *iarr = (unsigned int*) index_array->GetArrayData();
	unsigned int icount = index_array->GetElementCount();

	// get the first available texcoord channel
	GeometryArray *tc_array = 0;
	for (int i=0; i<8; i++)
	{
		tc_array = GetArray((GeometryArrayType) (TEXCOORD0_ARRAY + i));
		if (tc_array) break;
	}
	if (!tc_array) return false;
	Vector2 *tcarr = (Vector2*)tc_array->GetArrayData();

	// calculate triangle tangents and
	// detect triangles that share each vertex
	unsigned int num_tris = icount / 3;
	Vector3 *tri_tangents = new Vector3[num_tris];
	Vector3 *tri_binormals = new Vector3[num_tris];
	std::vector<unsigned int> *v_tris = new std::vector<unsigned int>[vcount];
	for (unsigned int i=0; i<num_tris; i++)
	{
		unsigned int ia,ib,ic;
		ia = iarr[i*3];
		ib = iarr[i*3 + 1];
		ic = iarr[i*3 + 2];

		// calculate triangle tangents
		CalculateTangent(varr[ia], varr[ib], varr[ic],
			tcarr[ia], tcarr[ib], tcarr[ic],
			tri_tangents[i], tri_binormals[i]);

		// cache neighbour tris
		v_tris[ia].push_back(i);
		v_tris[ib].push_back(i);
		v_tris[ic].push_back(i);
	}

	// discard existing yangents (if any)
	GeometryArray *tangent_array;
	while ((tangent_array = GetArray(TANGENT_ARRAY)))
	{
		Unmount(tangent_array);
		delete tangent_array;
	}
	tangent_array = new GeometryArray(TANGENT_ARRAY, SCE_BT_FLOAT, 3, vcount);
	Mount(tangent_array);
	GeometryArray *binormal_array;
	while ((binormal_array = GetArray(BINORMAL_ARRAY)))
	{
		Unmount(binormal_array);
		delete binormal_array;
	}
	binormal_array = new GeometryArray(BINORMAL_ARRAY, SCE_BT_FLOAT, 3, vcount);
	Mount(binormal_array);

	Vector3 *tarr = (Vector3*) tangent_array->GetArrayData();
	Vector3 *barr = (Vector3*) binormal_array->GetArrayData();

	// calculate vertex tangents
	for (unsigned int i=0; i<vcount; i++)
	{
		Vector3 tang(0, 0, 0), binorm(0, 0, 0);
		for (unsigned int j=0; j<v_tris[i].size(); j++)
		{
			tang += tri_tangents[v_tris[i][j]];
			binorm += tri_binormals[v_tris[i][j]];
		}
		if (tang.Length() > 0.0000001) tang.Normalize();
		if (binorm.Length() > 0.0000001) binorm.Normalize();
		
		tarr[i] = tang;
		barr[i] = binorm;
	}

	// cleanup
	delete [] tri_tangents;
	delete [] tri_binormals;
	delete [] v_tris;
	
	return true;
}

// high level
unsigned long MeshChunk::GetTriangleCount() const
{
	if (IsIndexed())
	{
		// triangle count of the index array
		return GetArray(INDEX_ARRAY)->GetElementCount() / 3;
	}
	else
	{
		// triangle count of the position array
		GeometryArray* pa = GetArray(POSITION_ARRAY);
		if (!pa) return 0;
		return pa->GetElementCount() / 3;
	}
}

void MeshChunk::GetTriangle(unsigned int idx, unsigned int &a,
	unsigned int &b, unsigned int &c) const
{
	if (idx >= GetTriangleCount())
	{
		a = b = c = 0;
		return;
	}

	if (IsIndexed())
	{
		unsigned long *inds = (unsigned long*) GetArray(INDEX_ARRAY)->GetArrayData();
		a = inds[3 * idx + 0];
		b = inds[3 * idx + 1];
		c = inds[3 * idx + 2];
	}
	else
	{
		a = 3 * idx + 0;
		b = 3 * idx + 1;
		c = 3 * idx + 2;
	}
}

// XXX: we assume that positions/normals/binormals/etc are Vec3
void MeshChunk::Transform(const Matrix4x4 &mat) {
	GeometryArray *geom = GetArray(POSITION_ARRAY);
	if(geom) {
        Vector3 *verts = (Vector3*)geom->GetArrayData();
		unsigned int vcount = geom->GetElementCount();

		for(unsigned int i=0; i<vcount; i++) {
			verts[i].Transform(mat);
		}
	}

	Matrix3x3 rot_mat = mat;

	geom = GetArray(NORMAL_ARRAY);
	if(geom) {
		Vector3 *norm = (Vector3*)geom->GetArrayData();
		unsigned int ncount = geom->GetElementCount();

		for(unsigned int i=0; i<ncount; i++) {
			norm[i].Transform(rot_mat);
		}
	}

	geom = GetArray(BINORMAL_ARRAY);
	if(geom) {
		Vector3 *binorm = (Vector3*)geom->GetArrayData();
		unsigned int bcount = geom->GetElementCount();

		for(unsigned int i=0; i<bcount; i++) {
			binorm[i].Transform(rot_mat);
		}
	}

	geom = GetArray(TANGENT_ARRAY);
	if(geom) {
		Vector3 *tang = (Vector3*)geom->GetArrayData();
		unsigned int tcount = geom->GetElementCount();

		for(unsigned int i=0; i<tcount; i++) {
			tang[i].Transform(rot_mat);
		}
	}
}

// NodeChunk class implementation
NodeChunk::NodeChunk()
{
	Resize(sizeof(NodeData));
	SetDynamic(false);
	SetNodeID(0);
	SetWorldPRS(PRS());
	SetLocalPRS(PRS());
	SetObjectPRS(PRS());
	SetPivotPRS(PRS());
}

NodeChunk::~NodeChunk()
{
}

Matrix4x4 NodeChunk::GetTransform() const {
	//Matrix4x4 pivot = GetPivotPRS().GetTransform();
	Matrix4x4 mat = GetLocalPRS().GetTransform();
	return mat;// * pivot;
}

PRS NodeChunk::GetWorldPRS() const
{
	return ((NodeData*)GetData())->world;
}

void NodeChunk::SetWorldPRS(const PRS& prs)
{
	((NodeData*)GetData())->world = prs;
}

PRS NodeChunk::GetLocalPRS() const
{
	return ((NodeData*)GetData())->local;
}

void NodeChunk::SetLocalPRS(const PRS& prs)
{
	((NodeData*)GetData())->local = prs;
}

PRS NodeChunk::GetObjectPRS() const
{
	return ((NodeData*)GetData())->object;
}

void NodeChunk::SetObjectPRS(const PRS& prs)
{
	((NodeData*)GetData())->object = prs;
}

PRS NodeChunk::GetPivotPRS() const
{
	return ((NodeData*)GetData())->pivot;
}

void NodeChunk::SetPivotPRS(const PRS &prs)
{
	((NodeData*)GetData())->pivot = prs;
}

bool NodeChunk::GetDynamic() const
{
	return ((NodeData*)GetData())->dynamic == TRUE? true:false;
}

void NodeChunk::SetDynamic(bool dynamic)
{
	((NodeData*)GetData())->dynamic = dynamic ? TRUE : FALSE;
}

unsigned long NodeChunk::GetNodeID() const
{
	return ((NodeData*)GetData())->id;
}
void NodeChunk::SetNodeID(unsigned long node_id)
{
	((NodeData*)GetData())->id = node_id;
}

// high-level
unsigned long NodeChunk::GetSubNodeCount() const
{
	return (unsigned long)GetIDChildren(SCE_NODE).size();
}
NodeChunk* NodeChunk::GetSubNode(unsigned long id) const
{
	if (id >= GetSubNodeCount()) return 0;
	return (NodeChunk*)GetIDChildren(SCE_NODE)[id];
}
unsigned long NodeChunk::GetMeshCount() const
{
	ChunkList l = GetIDChildren(SCE_MESH);
	unsigned long count = 0;
	for (unsigned long i=0; i<l.size(); i++)
	{
		if (((MeshChunk*)l[i])->IsMesh())
		{
			count++;
		}
	}
	return count;
}
MeshChunk* NodeChunk::GetMesh(unsigned long id) const
{
	ChunkList l = GetIDChildren(SCE_MESH);
	unsigned long count = 0;
	for (unsigned long i=0; i<l.size(); i++)
	{
		if (((MeshChunk*)l[i])->IsMesh())
		{
			if (count == id)
			{
				return (MeshChunk*)l[i];
			}
			count++;
		}
	}
	return 0;
}
unsigned long NodeChunk::GetProxyCount() const
{
	ChunkList l = GetIDChildren(SCE_MESH);
	unsigned long count = 0;
	for (unsigned long i=0; i<l.size(); i++)
	{
		if (((MeshChunk*)l[i])->IsProxy())
		{
			count++;
		}
	}
	return count;
}
MeshChunk* NodeChunk::GetProxy(unsigned long id) const
{
	ChunkList l = GetIDChildren(SCE_MESH);
	unsigned long count = 0;
	for (unsigned long i=0; i<l.size(); i++)
	{
		if (((MeshChunk*)l[i])->IsProxy())
		{
			if (count == id)
			{
				return (MeshChunk*)l[i];
			}
			count++;
		}
	}
	return 0;
}

unsigned long NodeChunk::GetPVSCount() const
{
	ChunkList l = GetIDChildren(SCE_MESH);
	unsigned long count = 0;
	for (unsigned long i=0; i<l.size(); i++)
	{
		if (((MeshChunk*)l[i])->IsPVS())
		{
			count++;
		}
	}
	return count;
}
MeshChunk* NodeChunk::GetPVS(unsigned long id) const
{
	ChunkList l = GetIDChildren(SCE_MESH);
	unsigned long count = 0;
	for (unsigned long i=0; i<l.size(); i++)
	{
		if (((MeshChunk*)l[i])->IsPVS())
		{
			if (count == id)
			{
				return (MeshChunk*)l[i];
			}
			count++;
		}
	}
	return 0;
}

unsigned long NodeChunk::GetLinkCount() const
{
	ChunkList l = GetIDChildren(SCE_MESH);
	unsigned long count = 0;
	for (unsigned long i=0; i<l.size(); i++)
	{
		if (((MeshChunk*)l[i])->IsLink())
		{
			count++;
		}
	}
	return count;
}

MeshChunk* NodeChunk::GetLink(unsigned long id) const
{
	ChunkList l = GetIDChildren(SCE_MESH);
	unsigned long count = 0;
	for (unsigned long i=0; i<l.size(); i++)
	{
		if (((MeshChunk*)l[i])->IsLink())
		{
			if (count == id)
			{
				return (MeshChunk*)l[i];
			}
			count++;
		}
	}
	return 0;
}

unsigned long NodeChunk::GetLightCount() const
{
	return (unsigned long) GetIDChildren(SCE_LIGHT).size();
}
LightChunk* NodeChunk::GetLight(unsigned long i) const
{
	ChunkList l = GetIDChildren(SCE_LIGHT);
	if (i >= l.size()) return 0;
	return (LightChunk*)l[i];
}
unsigned long NodeChunk::GetCameraCount() const
{
	return (unsigned long) GetIDChildren(SCE_CAMERA).size();
}
CameraChunk* NodeChunk::GetCamera(unsigned long i) const
{
	ChunkList l = GetIDChildren(SCE_CAMERA);
	if (i >= l.size()) return 0;
	return (CameraChunk*) l[i];
}

void NodeChunk::BatchMeshes()
{
	// collect all meshes of the same material id
	std::map<unsigned long, bool> mat_exists;
	std::map<unsigned long, unsigned long> mat_map;
	std::vector<unsigned long> existing_materials;

	for (unsigned int i=0; i<GetMeshCount(); i++)
	{
		unsigned long mat_id = GetMesh(i)->GetMaterialID();
		if (!mat_exists[mat_id])
		{
			mat_exists[mat_id] = true;
			mat_map[mat_id] = existing_materials.size();
			existing_materials.push_back(mat_id);
		}
	}

	std::vector<MeshChunk*> *meshes_by_mat = new std::vector<MeshChunk*>[existing_materials.size()];
	for (unsigned int i=0; i<GetMeshCount(); i++)
	{
		MeshChunk *m = GetMesh(i);
		unsigned long slot = mat_map[m->GetMaterialID()];
		meshes_by_mat[slot].push_back(m);
	}

	// concatenate meshes by material
	std::vector<MeshChunk*> meshes_to_delete;
	for (unsigned int i=0; i<existing_materials.size(); i++)
	{
		if (meshes_by_mat[i].size() >= 2)
		{
			for (unsigned int j=0; j<meshes_by_mat[i].size(); j++)
			{
				meshes_to_delete.push_back(meshes_by_mat[i][j]);
			}

			MeshChunk *added_mesh = ConcatenateMeshChunks(meshes_by_mat[i]);
			Mount(added_mesh);
		}
	}

	// collect all proxy meshes
	std::vector<MeshChunk*> proxy;
	for (unsigned int i=0; i<GetProxyCount(); i++)
	{
		proxy.push_back(GetProxy(i));
	}
	if (proxy.size() >= 2)
	{
		for (unsigned int i=0; i<proxy.size(); i++)
		{
			meshes_to_delete.push_back(proxy[i]);
		}
		MeshChunk *added_mesh = ConcatenateMeshChunks(proxy);
		Mount(added_mesh);
	}

	// collect all pvs meshes
	std::vector<MeshChunk*> pvs;
	for (unsigned int i=0; i<GetPVSCount(); i++)
	{
		pvs.push_back(GetPVS(i));
	}
	if (pvs.size() >= 2)
	{
		for (unsigned int i=0; i<pvs.size(); i++)
		{
			meshes_to_delete.push_back(pvs[i]);
		}
		MeshChunk *added_mesh = ConcatenateMeshChunks(pvs);
		Mount(added_mesh);
	}

	// collect all link meshes
	std::vector<MeshChunk*> link;
	for (unsigned int i=0; i<GetLinkCount(); i++)
	{
		link.push_back(GetLink(i));
	}
	if (link.size() >= 2)
	{
		for (unsigned int i=0; i<link.size(); i++)
		{
			meshes_to_delete.push_back(link[i]);
		}
		MeshChunk *added_mesh = ConcatenateMeshChunks(link);
		Mount(added_mesh);
	}

	// delete meshes
	for (unsigned int i=0; i<meshes_to_delete.size(); i++)
	{
		Delete(meshes_to_delete[i]);
	}

	// cleanup
	delete [] meshes_by_mat;
}

NodeChunk* NodeChunk::GetNodeByIDRecursive(unsigned long id) const
{
	ChunkList cl = GetChildren();
	for (unsigned long i=0; i<cl.size(); i++)
	{
		if (cl[i]->GetID() == SCE_NODE)
		{
			NodeChunk* tmp = (NodeChunk*)cl[i];
			if (tmp->GetNodeID() == id)
			{
				return tmp;
			}
			else
			{
				tmp = tmp->GetNodeByIDRecursive(id);
				if (tmp) return tmp;
			}
		}
	}

	return 0;
}

NodeChunk* NodeChunk::GetNodeByIDTree(unsigned long id) const
{
	const Chunk* tmp = this;
	while (tmp->GetParent())
	{
		tmp = tmp->GetParent();
	}

	if (tmp->GetID() == SCE_MAIN)
	{
		return ((SceneFile*)tmp)->GetNodeByIDRecursive(id);
	}
	if (tmp->GetID() == SCE_NODE)
	{
		return ((NodeChunk*)tmp)->GetNodeByIDRecursive(id);
	}

	// the node with the specified id is not in this file
	return 0;
}

// SceneFile class implementation
SceneFile::SceneFile()
{
	Resize(sizeof(SceneHeader));
	SetVersion(100);
	SetMessage("");
}

SceneFile::~SceneFile()
{

}

unsigned long SceneFile::GetVersion() const
{
	return ((SceneHeader*)GetData())->version;
}

void SceneFile::SetVersion(unsigned long version)
{
	((SceneHeader*)GetData())->version = version;
}

std::string SceneFile::GetMessage() const
{
	return GetName();
}

void SceneFile::SetMessage(const std::string& message)
{
	SetName(message);
}

// high-level
unsigned long SceneFile::GetMaterialCount() const
{
	return (unsigned long)GetIDChildren(SCE_MATERIAL).size();
}
MaterialChunk* SceneFile::GetMaterial(unsigned long id) const
{
	if (id >= GetMaterialCount()) return 0;
	return (MaterialChunk*)GetIDChildren(SCE_MATERIAL)[id];
}
unsigned long SceneFile::GetRootNodeCount() const
{
	return (unsigned long)GetIDChildren(SCE_NODE).size();
}
NodeChunk* SceneFile::GetRootNode(unsigned long id) const
{
	if (id >= GetRootNodeCount()) return 0;
	return (NodeChunk*)GetIDChildren(SCE_NODE)[id];
}

NodeChunk* SceneFile::GetNodeByIDRecursive(unsigned long id) const
{
	ChunkList cl = GetChildren();
	for (unsigned long i=0; i<cl.size(); i++)
	{
		if (cl[i]->GetID() == SCE_NODE)
		{
			NodeChunk* tmp = (NodeChunk*)cl[i];
			if (tmp->GetNodeID() == id)
			{
				return tmp;
			}
			else
			{
				tmp = tmp->GetNodeByIDRecursive(id);
				if (tmp) return tmp;
			}
		}
	}

	return 0;
}

MeshList SceneFile::GetPVSMeshes() const
{
	MeshList ret;
	ChunkList cl = GetIDChildrenRec(SCE_MESH);
	for (unsigned int i=0; i<cl.size(); i++)
	{
		MeshChunk *m = (MeshChunk*) cl[i];
		if (m->IsPVS())
		{
			ret.push_back(m);
		}
	}
	return ret;
}

MeshList SceneFile::GetLinkMeshes() const
{
	MeshList ret;
	ChunkList cl = GetIDChildrenRec(SCE_MESH);
	for (unsigned int i=0; i<cl.size(); i++)
	{
		MeshChunk *m = (MeshChunk*) cl[i];
		if (m->IsLink())
		{
			ret.push_back(m);
		}
	}
	return ret;
}

// helpers
MeshChunk *ConcatenateMeshChunks(std::vector<MeshChunk*> &chunks)
{
	if (!chunks.size()) return 0;

	// if any mesh has no position array, exclude it
	std::vector<MeshChunk*> meshes;
	for (unsigned int i=0; i<chunks.size(); i++)
	{
		if (chunks[i]->GetArray(POSITION_ARRAY))
		{
			meshes.push_back(chunks[i]);
		}
	}

	if (!meshes.size()) return 0;

	// collect info about the final mesh
	bool proxy=false, pvs=false, mesh=false, link=false;
	unsigned long mat_id = meshes[0]->GetMaterialID();

	for (unsigned int i=0; i<meshes.size(); i++)
	{
		if (meshes[i]->IsProxy()) proxy = true;
		if (meshes[i]->IsPVS()) pvs = true;
		if (meshes[i]->IsMesh()) mesh = true;
		if (meshes[i]->IsLink()) link = true;
	}

	// see which arrays will be available in the final mesh
	bool avail_arrays[MAX_GEOMETRY_ARRAYS];
	for (unsigned int i=0; i<(unsigned int)MAX_GEOMETRY_ARRAYS; i++)
	{
		avail_arrays[i] = true;
	}
	for (unsigned int i=0; i<meshes.size(); i++)
	{
		for (unsigned int j=0; j<(unsigned int) MAX_GEOMETRY_ARRAYS; j++)
		{
			if (!meshes[i]->GetArray((GeometryArrayType) j))
				avail_arrays[j] = false;
		}
	}

	// see how many vertices to copy from each mesh
	unsigned int *mesh_vcount = new unsigned int[meshes.size()];
	for (unsigned int i=0; i<meshes.size(); i++)
	{
		unsigned int vcount = meshes[i]->GetArray(POSITION_ARRAY)->GetElementCount();
		for (unsigned int j=0; j<MAX_GEOMETRY_ARRAYS; j++)
		{
			if (!avail_arrays[j]) continue;
			unsigned int count = meshes[i]->GetArray((GeometryArrayType)j)->GetElementCount();
			if (count < vcount) vcount = count;
		}
		mesh_vcount[i] = vcount;
	}

	// calculate vertex shifts
	unsigned int final_vertex_count = 0;
	unsigned int *begin_vertex = new unsigned int[meshes.size()];
	for (unsigned int i=0; i<meshes.size(); i++)
	{
		begin_vertex[i] = final_vertex_count;
		final_vertex_count += mesh_vcount[i];
	}

	// calculate index shifts
	unsigned int final_index_count = 0;
	unsigned int *begin_index = new unsigned int[meshes.size()];
	for (unsigned int i=0; i<meshes.size(); i++)
	{
		begin_index[i] = final_index_count;
		final_index_count += meshes[i]->GetTriangleCount() * 3;
	}

	// init final mesh
	MeshChunk *final = new MeshChunk();
	final->SetProxy(proxy);
	final->SetPVS(pvs);
	final->SetMesh(mesh);
	final->SetLink(link);
	final->SetMaterialID(mat_id);

	// create geometry arrays
	for (unsigned int i=0; i<MAX_GEOMETRY_ARRAYS; i++)
	{
		if (i == INDEX_ARRAY) continue;
		if (!avail_arrays[i]) continue;

		GeometryArrayType type = (GeometryArrayType)i; 
		SceBasicType basic_type = meshes[0]->GetArray(type)->GetBasicType();
		unsigned long basic_type_count = meshes[0]->GetArray(type)->GetBasicTypeCount();; 
		unsigned long element_count = final_vertex_count;

		unsigned int element_size = bt_size[basic_type] * basic_type_count;

		GeometryArray *arr = new GeometryArray(type, basic_type, 
			basic_type_count, element_count);
		final->Mount(arr);

		unsigned char *data = (unsigned char *)arr->GetArrayData();

		// copy array data from all meshes
		for (unsigned int j=0; j<meshes.size(); j++)
		{
			unsigned char *src = (unsigned char *)meshes[j]->GetArray((GeometryArrayType)i)->GetArrayData();
			unsigned char *dst = data + element_size * begin_vertex[j];
			unsigned int sz = mesh_vcount[j] * element_size;

			memcpy(dst, src, sz);
		}
	}

	// create index array
	GeometryArray *ia = new GeometryArray(INDEX_ARRAY, SCE_BT_INT32, 1, final_index_count);
	final->Mount(ia);

	for (unsigned int i=0; i<meshes.size(); i++)
	{
		unsigned int *dst = (unsigned int*)ia->GetArrayData();
		dst += begin_index[i];
		for (unsigned int j=0; j<meshes[i]->GetTriangleCount(); j++)
		{
			unsigned int a, b, c;
			meshes[i]->GetTriangle(j, a, b, c);
			if (a >= mesh_vcount[i] || b >= mesh_vcount[i] || c >= mesh_vcount[i])
			{
				a = b = c = 0;
			}
			a += begin_vertex[i];
			b += begin_vertex[i];
			c += begin_vertex[i];

			dst[3 * j + 0] = a;
			dst[3 * j + 1] = b;
			dst[3 * j + 2] = c;
		}
	}

	// cleanup
	delete [] begin_index;
	delete [] begin_vertex;
	delete [] mesh_vcount;

	return final;
}







