#include "builddefs.h"
#ifdef USE_OPENMP
#include "bench.h"
#include <sstream>

//Put another threadprivate here, unless you are using Intel's compiler.
int bm_thread_num;
int bm_num_threads;


Benchmark bm;


void bm_init()
{
	bm_num_threads = omp_get_max_threads();
	if (omp_in_parallel()) {
		bm_thread_num  = omp_get_thread_num();
	}
	else {
#		pragma omp parallel
		bm_thread_num  = omp_get_thread_num();
	}
}


void bm_reset()
{
#	pragma omp master
	{
		std::map<std::string, BMSection *>::iterator iter;
		for (iter = bm.smap.begin(); iter != bm.smap.end(); iter++) {
			BMSection *pbms = iter->second;
			if (iter->second->inParallel) {
				for (int i=0; i<bm_num_threads; i++) {
					bm_section_reset(pbms+i);
				}
			}
			else {
				bm_section_reset(pbms);
			}
		}
	}
}


void bm_section_init(BMSection **bms, std::string section_name, std::string file_name, int line)
{
#	pragma omp master
	{
		int num_threads = omp_get_num_threads();
		*bms = new BMSection[num_threads];
		for (int i=0; i<num_threads; i++) {
			bms[i]->time = 0.0;
			bms[i]->count = 0;
			bms[i]->file = file_name;
			bms[i]->line = line;
			bms[i]->name = section_name;
			bms[i]->threadNum = omp_get_thread_num();
			bms[i]->inParallel = omp_in_parallel();
			bms[i]->running = false;
		}
		bm.submit(*bms);
	}
#	pragma omp barrier
}


void bm_section_reset(BMSection *bms)
{
	bms->time = 0;
	bms->temp = 0;
	bms->count = 0;
}


//currently prints
void bm_report(int iter)
{
#	pragma omp master
	{
		printf("report produced for %d iterations:\n", iter);
		std::map<std::string, BMSection *>::iterator iter;
		for (iter = bm.smap.begin(); iter != bm.smap.end(); iter++) {
			BMSection *pbms = iter->second;
			if (iter->second->inParallel) {
				for (int i=0; i<bm_num_threads; i++) {
					printf("Section: %s, thread: %d, time: %f\n", pbms[i].name.c_str(),
						pbms[i].threadNum, pbms[i].time);
					//bm_section_reset(pbms+i);
				}
			}
			else {
				printf("Section: %s, time: %f\n", pbms->name.c_str(), pbms->time);
				//bm_section_reset(pbms);
			}
		}
	}
}


Benchmark::Benchmark()
{
	bm_init();
}


void Benchmark::submit(BMSection *section)
{
	//find whether this section already exists
	if (smap.find(section->name) != smap.end()) {
		BM_ERR_04;
	}
	smap[section->name] = section;
}

BMSection *Benchmark::findSection(std::string name)
{
	static BMSection *rv;
#	pragma omp master
	{
		if (smap.find(name) != smap.end()) {
			bool in_parallel = omp_in_parallel();
			if (smap[name]->inParallel == in_parallel) {
				rv = smap[name];
			}
			else {
				if (in_parallel) {
					BM_ERR_01;
				}
				else {
					BM_ERR_02;
				}
			}
		}
		else {
			BM_ERR_03;
		}
	}
#	pragma omp barrier
	return rv;
}
#endif
