#ifndef COLOR_H
#define COLOR_H

#include <byteorder.h>
#include "color_bits.h"
#include <math3d.h>

#define PACKF_RGBA(r, g, b, a) \
	PACK_COLOR32((int)((r) * 255.0), (int)((g) * 255.0), (int)((b) * 255.0), (int)((a) * 255.0));

#define UNPACKF_RED(p)		((float)UNPACK_RED32(p) / 255.0f)
#define UNPACKF_GREEN(p)	((float)UNPACK_GREEN32(p) / 255.0f)
#define UNPACKF_BLUE(p)		((float)UNPACK_BLUE32(p) / 255.0f)
#define UNPACKF_ALPHA(p)	((float)UNPACK_ALPHA32(p) / 255.0f)

class  Color {
public:
    float r, g, b, a;

	Color(float intensity = 1.0f);
    Color(float r, float g, float b, float a = 1.0f);
	Color(const Vector4 &c);
};

inline Color operator +(const Color &c1, const Color &c2);
inline Color operator -(const Color &c1, const Color &c2);
inline Color operator *(const Color &c1, const Color &c2);
inline Color operator /(const Color &c1, const Color &c2);

inline void operator +=(Color &c1, const Color &c2);
inline void operator -=(Color &c1, const Color &c2);
inline void operator *=(Color &c1, const Color &c2);
inline void operator /=(Color &c1, const Color &c2);

inline unsigned int PackColor(const Color &c);
inline Color UnpackColor(unsigned int p);

#include "color.inl"

#endif	/* COLOR_H */
