/* color bit shift and mask values for color packing/unpacking
 * in a cross-endianess manner.
 */
#ifndef _COLOR_BITS_H_
#define _COLOR_BITS_H_

#include <byteorder.h>
#include "int_types.h"

/* 32bit color shift values */
#ifdef LITTLE_ENDIAN
#define ALPHA_SHIFT32	24
#define RED_SHIFT32		16
#define GREEN_SHIFT32	8
#define BLUE_SHIFT32	0
#else	/* BIG_ENDIAN */
#define ALPHA_SHIFT32	0
#define RED_SHIFT32		8
#define GREEN_SHIFT32	16
#define BLUE_SHIFT32	24
#endif	/* LITTLE_ENDIAN */

/* 32bit color mask values */
#define ALPHA_MASK32	(0xff << ALPHA_SHIFT32)
#define RED_MASK32		(0xff << RED_SHIFT32)
#define GREEN_MASK32	(0xff << GREEN_SHIFT32)
#define BLUE_MASK32		(0xff << BLUE_SHIFT32)

/* 16bit color shift values */
#ifdef LITTLE_ENDIAN
#define RED_SHIFT16		11
#define GREEN_SHIFT16	5
#define BLUE_SHIFT16	0
#else	/* BIG_ENDIAN */
#define RED_SHIFT16		0
#define GREEN_SHIFT16	5
#define BLUE_SHIFT16	11
#endif	/* LITTLE_ENDIAN */

/* 16bit color mask values */
#define RED_MASK16		(0x1f << RED_SHIFT16)
#define GREEN_MASK16	(0x3f << GREEN_SHIFT16)
#define BLUE_MASK16		(0x1f << BLUE_SHIFT16)

/* 15bit color shift values */
#ifdef LITTLE_ENDIAN
#define RED_SHIFT15		10
#define GREEN_SHIFT15	5
#define BLUE_SHIFT15	0
#else	/* BIG_ENDIAN */
#define RED_SHIFT15		0
#define GREEN_SHIFT15	5
#define BLUE_SHIFT15	10
#endif	/* LITTLE_ENDIAN */

/* 15bit color mask values */
#define RED_MASK15		(0x1f << RED_SHIFT15)
#define GREEN_MASK15	(0x1f << GREEN_SHIFT15)
#define BLUE_MASK15		(0x1f << BLUE_SHIFT15)


/* color packing macros */
#define PACK_COLOR32(r, g, b, a) \
	((((unsigned int)(a) << ALPHA_SHIFT32) & ALPHA_MASK32) | \
	(((unsigned int)(r) << RED_SHIFT32) & RED_MASK32) | \
	(((unsigned int)(g) << GREEN_SHIFT32) & GREEN_MASK32) | \
	(((unsigned int)(b) << BLUE_SHIFT32) & BLUE_MASK32))


#define PACK_COLOR24(r, g, b)		PACK_COLOR32(r, g, b, 0xff)

#define UNPACK_ALPHA32(a)	(((a) & ALPHA_MASK32) >> ALPHA_SHIFT32)
#define UNPACK_RED32(r)		(((r) & RED_MASK32) >> RED_SHIFT32)
#define UNPACK_GREEN32(g)	(((g) & GREEN_MASK32) >> GREEN_SHIFT32)
#define UNPACK_BLUE32(b)	(((b) & BLUE_MASK32) >> BLUE_SHIFT32)

#define PACK_COLOR16(r, g, b) \
	((((unsigned short)(r) << RED_SHIFT16) & RED_MASK16) | \
	(((unsigned short)(g) << GREEN_SHIFT16) & GREEN_MASK16) | \
	(((unsigned short)(b) << BLUE_SHIFT16) & BLUE_MASK16))

#define UNPACK_RED16(x) ((x & RED_MASK16) >> RED_SHIFT16)
#define UNPACK_GREEN16(x) ((x & GREEN_MASK16) >> GREEN_SHIFT16)
#define UNPACK_BLUE16(x) ((x & BLUE_MASK16) >> BLUE_SHIFT16)

#define PACK_COLOR15(r, g, b) \
	((((unsigned short)(r) << RED_SHIFT15) & RED_MASK15) | \
	(((unsigned short)(g) << GREEN_SHIFT15) & GREEN_MASK15) | \
	(((unsigned short)(b) << BLUE_SHIFT15) & BLUE_MASK15))

#define UNPACK_RED15(x) ((x & RED_MASK15) >> RED_SHIFT15)
#define UNPACK_GREEN15(x) ((x & GREEN_MASK15) >> GREEN_SHIFT15)
#define UNPACK_BLUE15(x) ((x & BLUE_MASK15) >> BLUE_SHIFT15)


#endif	/* _COLOR_BITS_H_ */
