#include <builddefs.h>
#include <vector>
#include <string>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "console.h"
#include <text.h>
#include "errlog.h"

using namespace std;

static bool visible = true;
static unsigned int con_font;

static vector<char*> con_lines;
static int max_con_rows = 20, max_con_lines = 10;

static const char *font_dir[] = {
	"data/fonts",
	"data",
#ifdef BUILD_WIN32
	"c:/WINDOWS/Fonts",
#endif
#ifdef BUILD_UNIX
	"/usr/share/fonts/truetype",
#endif
	DEV_DATA_PATH "/fonts",
	"c:/theseis/engine/engine_data/fonts",
	0
};

bool ConInit()
{
	static const char *con_font_file = "georgia.ttf";
	char buf[256];

	const char **fdir = font_dir;
	while(*fdir) {
		sprintf(buf, "%s/%s", *fdir, con_font_file);
		if((con_font = CreateFont(buf, 15))) {
			break;
		}
		fdir++;
	}

	if(!*fdir) {
		Error("console init failed to load console font: %s\n", con_font_file);
		return false;
	}
	return true;
}

void ConDestroy()
{
	ConClearBuffer();
	DeleteFont(con_font);
}

void ConShow(bool show)
{
	visible = show;
}

bool ConGetVisible()
{
	return visible;
}

void ConClearBuffer()
{
	for(size_t i=0; i<con_lines.size(); i++) {
		free(con_lines[i]);
	}
	con_lines.clear();
}

#if !defined(GLIBC) && !defined(__GLIBC__)
#define vsnprintf(str, size, format, ap)	vsprintf(str, format, ap)
#endif

void ConPrintf(const char *fmt, ...)
{
	static char linebuf[1024];

	va_list arg_list;
	va_start(arg_list, fmt);
	vsnprintf(linebuf, sizeof linebuf, fmt, arg_list);
	va_end(arg_list);

	ConPrintStr(linebuf);
}

void ConPrintStr(const char *str)
{
	// seperate lines, and add them to the buffer
	char *scrap;
	try {
		scrap = new char[strlen(str) + 1];
	}
	catch(...) {
		Error("ConPrintStr failed to allocate memory\n");
		return;
	}
	strcpy(scrap, str);

	char *tok = 0;
	while((tok = strtok(tok ? 0 : scrap, "\n"))) {
		char *tmp = new char[strlen(tok) + 1];
		strcpy(tmp, tok);
		con_lines.push_back(tmp);
	}
	delete [] scrap;
}

void ConDraw()
{
	if(!visible) return;
	BindFont(con_font);
	SetTextPos(Vector2(0.0f, 0.0f));
	TextLineAdvance();
	SetTextColor(Color(1.0f, 0.7f, 0.3f));

	int lines_num = MIN((int)con_lines.size(), max_con_lines);
	PrintStringLines((const char**)&con_lines[con_lines.size() - lines_num], lines_num);
}
