// character.h

#ifndef _CHARACTER_H_
#define _CHARACTER_H_

#include <map>
#include <objectgroup.h>
#include <scene.h>

using namespace std;

enum CharState
{
	STAY,
	WALK,
	WALK_BACK,
	SIDESTEP_LEFT,
	SIDESTEP_RIGHT,
	RUN,
	SNEAK,

	MAX_CHAR_STATES
};

enum CharNode
{
	ROOT,
	L_HAND,
	R_HAND,
	L_KNEE,
	R_KNEE,
	L_FOOT,
	R_FOOT,
	HEAD,

	MAX_CHAR_NODES
};

struct CharAnimation
{
	Animation *anim;
	Timeline velocity;
	unsigned int ground;
};

struct Control
{
	Vector2 pad1, pad2;
};

struct CollisionResult
{
	bool collision;
	unsigned int triangle_id;
	CollisionMesh *mesh;
	CollisionResult()
	{
		collision = 0;
		triangle_id = 0;
		mesh = 0;
	};
};

class Character
{
public:
	map<int, CharState> state; //active state, TODO: name "state" is not very descriptive
	unsigned int state0_msec, state1_msec, transition_msec;
	
	Matrix4x4 origin2_matrix; // intermediate space

	Scene *parent_scene;

	float t0, t1, dt, tr;

	ObjectGroup *og;
	CharAnimation animations[MAX_CHAR_STATES];
	OGNode *nodes[MAX_CHAR_NODES];
	float node_radius[MAX_CHAR_NODES];
	CollisionResult col_results[MAX_CHAR_NODES];

	// metrics
	float height, leg_length;

	Control ctrl;

	// relative prs
	PRS rel_prs;

	// origin
	OGNode *origin_node;

	Character();
	~Character();

	bool SetOG(ObjectGroup *og);
	bool SetAnimation(CharState which, unsigned int ground, Animation *anim);
	void Update(const Control &control, unsigned int msec);

	Vector3 GetNodePos(CharNode which);
	Sphere GetNodeSphere(CharNode which);

	void ProcessGenericState();
	void ProcessGenericTransition();

	Matrix4x4 GetOriginTransform();
	Matrix4x4 GetGlobalTransform();

	void SetAbsolutePosition(const Vector3 &pos);
	Vector3 GetAbsolutePosition();

	void ProcessCollisions();
};


#endif // ndef _CHARACTER_H_
