#include <stdio.h>
#include <libgc.h>
#include <libgcx.h>
#include <scene.h>
#include <devhelp.h>
#include <libimage.h>
#include "inventory.h"

//// GAME INVENTORY ////

// InventoryItem
InventoryItem::InventoryItem(const std::string &namename, ObjectGroup *og)
{
	this->name = name;
	this->og = og;
	inv = NULL;
}

InventoryItem::~InventoryItem()
{
}

const std::string &InventoryItem::GetName() const
{
	return name;
}

ObjectGroup *InventoryItem::GetObjectGroup()
{
	return og;
}

class Inventory *InventoryItem::getInventory() const
{
	return inv;
}

void InventoryItem::setInventory(class Inventory *inv)
{
	this->inv = inv;
}


// Inventory
void Inventory::Add(InventoryItem *item)
{
	if (item != NULL)
	{
		items.push_back(item);
	}
}

void Inventory::Remove(InventoryItem *item)
{
	if (item != NULL)
	{
		items.remove(item);
	}
}

bool Inventory::Has(InventoryItem *item)
{
	std::list<InventoryItem*>::const_iterator	iter = items.begin();
	for (; iter != items.end(); iter++)
	{
		if (item == *iter)
		{
			return true;
		}
	}

	return false;
}

const std::list<InventoryItem*> &Inventory::GetItemsList() const
{
	return items;
}

// Stuff
static std::list<InventoryItem*>	invobj;

InventoryItem *RegisterInventoryItem(std::string name, ObjectGroup *og)
{
	InventoryItem	*item = new InventoryItem(name, og);
	invobj.push_back(item);
	return item;
}

InventoryItem *InventoryFindItem(std::string name)
{
	std::list<InventoryItem*>::const_iterator	iter = invobj.begin();
	for (; iter != invobj.end(); iter++)
	{
		InventoryItem	*item = *iter;
		if (item->GetName() == name)
		{
			return item;
		}
	}

	return NULL;
}


//// INVENTORY RENDERING ////
static bool				initialized = false;
static Scene			*scItem1, *scItem2;
static ObjectGroup		*ogItem1, *ogItem2;
static unsigned int		sw, sh, tr1, tr2;
static bool				splitview;
static float			rot1X, rot1Y, rot2X, rot2Y;
static float			rot1XI = 0.0f, rot1YI = 0.0f, rot2XI = 0.0f, rot2YI = 0.0f;
static float			panX, panY, panXI, panYI;
static float			win1X1, win1X2, win2X1, win2X2;
static unsigned int		gsgpuprog;
static Matrix4x4		mat1, mat2, cmat1, cmat2;
static PRSCamera		*cam;
static Matrix4x4		refLeft, refRight;
static float			refPanX, refPanY;
static Image			*tinner, *tseparator, *tframe, *titembar, *tactive;

static Image *LoadImage(const char *filename)
{
	Image	*img = new Image();
	if (!img->Load(filename))
	{
		delete img;
		return NULL;
	}
	return img;
}

//// NORMAL VIEW MODE (examining objects)
static void RenderSingleObject(Renderer &rend, unsigned int tr, Scene *sc, ObjectGroup *og, Matrix4x4 &mat, Matrix4x4 &cmat)
{
	Matrix4x4	rot, tmp;
	tmp.SetScaling(Vector4(4, 2, 4, 1));		rot *= tmp;
	if (og == ogItem2)
	{
		tmp.SetTranslation(Vector3(panX, panY, 0.0f));	rot *= tmp;
	}
	rot *= cmat;
	rot	*= mat;

	SetLighting(true);
	SetDepthTest(true);
	SetCulling(CULL_CCW);

	SetRenderTarget(tr, 0);
	sc->SetBGColor(Color(0, 0, 0, 0));
	og->SetModelMatrix(rot);
	rend.Render(sc);
}

static void RenderNormalView(Renderer &rend)
{
	float	halfPixelW = 0.5f/sw;
	float	halfPixelH = 0.5f/sh;

	RenderSingleObject(rend, tr1, scItem1, ogItem1, mat1, cmat1);
	RenderSingleObject(rend, tr2, scItem2, ogItem2, mat2, cmat2);
	SetRenderTarget(0, 0);

	SetLighting(false);
	SetTexTransform(true);

	ImFixedFunction(1);
	SetClearColor(Color(0.6, 0.4, 0.2));
	Clear();

	/*ImMatrixMode(IM_TEXTURE);
	ImLoadIdentity();
	Matrix4x4	tm = GetRenderTargetTexMat(tr1);
	ImMultMatrix(tm);
	ImMatrixMode(IM_MODEL);*/

	SetBlending(true);
	SetBlendFunc(BLEND_SRC_ALPHA, BLEND_ONE_MINUS_SRC_ALPHA);

	ShowImage(tinner, 0, 0, 1, 1);

	// left window
	ImOverlay(Vector2(win1X1 + halfPixelW,  halfPixelH), Vector2(win1X2 + halfPixelW, 1 + halfPixelH), Color(1, 1, 1, 1), tr1, 0);
	//ImFixedFunction(false);
	ImOverlay(Vector2(win1X1 + halfPixelW,  halfPixelH), Vector2(win1X2 + halfPixelW, 1.0f + halfPixelH), Color(1.0, 1.0, 1.0, 0.3), tr2, 0);
	//ImFixedFunction(true);
	// right window
	
	//ImFixedFunction(false);
	ImOverlay(Vector2(win2X1 + halfPixelW, halfPixelH), Vector2(win2X2 + halfPixelW, 1 + halfPixelH), Color(0.7, 0.7, 0.7, 0.3), tr1, gsgpuprog);
	//ImFixedFunction(true);
	ImOverlay(Vector2(win2X1 + halfPixelW, halfPixelH), Vector2(win2X2 + halfPixelW, halfPixelH + 1.0f), Color(1, 1, 1, 1), tr2, 0);

	ShowImage(tseparator, 0.50 - (207.0/2048.0), 0, 0.50 + (207.0/2048.0), (609.0/768.0));

	ShowImage(tframe, 0, 0, 1, 1);
	
	ShowImage(titembar, 0, 1 - (178.0/768.0), 1, 1);
	ShowImage(tactive, 0.5 - (231.0/2048.0), 1 - (199.0/768.0), 0.5 + (231.0/2048), 1);
	
	SetBlending(false);

	SetTexTransform(false);
}

static void ControlNormalView(InvCtlType ctl, bool down)
{
	switch (ctl)
	{
	// rotation
	case ICT_FIRST_ROTATE_LEFT:
		if (!down && (rot1YI > 0.0f))
			rot1YI = 0.0f;
		else if (down && (rot1YI == 0.0f))
			rot1YI = 1.0f;
		break;
	case ICT_FIRST_ROTATE_RIGHT:
		if (!down && (rot1YI < 0.0f))
			rot1YI = 0.0f;
		else if (down && (rot1YI == 0.0f))
			rot1YI = -1.0f;
		break;
	case ICT_SECOND_ROTATE_LEFT:
		if (!down && (rot2YI > 0.0f))
			rot2YI = 0.0f;
		else if (down && (rot2YI == 0.0f))
			rot2YI = 1.0f;
		break;
	case ICT_SECOND_ROTATE_RIGHT:
		if (!down && (rot2YI < 0.0f))
			rot2YI = 0.0f;
		else if (down && (rot2YI == 0.0f))
			rot2YI = -1.0f;
		break;
	case ICT_FIRST_ROTATE_DOWN:
		if (!down && (rot1XI > 0.0f))
			rot1XI = 0.0f;
		else if (down && (rot1XI == 0.0f))
			rot1XI = 1.0f;
		break;
	case ICT_FIRST_ROTATE_UP:
		if (!down && (rot1XI < 0.0f))
			rot1XI = 0.0f;
		else if (down && (rot1XI == 0.0f))
			rot1XI = -1.0f;
		break;
	case ICT_SECOND_ROTATE_DOWN:
		if (!down && (rot2XI > 0.0f))
			rot2XI = 0.0f;
		else if (down && (rot2XI == 0.0f))
			rot2XI = 1.0f;
		break;
	case ICT_SECOND_ROTATE_UP:
		if (!down && (rot2XI < 0.0f))
			rot2XI = 0.0f;
		else if (down && (rot2XI == 0.0f))
			rot2XI = -1.0f;
		break;

	// panning
	case ICT_SECOND_PAN_LEFT:
		if (!down && (panXI < 0.0f))
			panXI = 0.0f;
		else if (down && (panXI == 0.0f))
			panXI = -20.0f;
		break;
	case ICT_SECOND_PAN_RIGHT:
		if (!down && (panXI > 0.0f))
			panXI = 0.0f;
		else if (down && (panXI == 0.0f))
			panXI = 20.0f;
		break;
	case ICT_SECOND_PAN_DOWN:
		if (!down && (panYI < 0.0f))
			panYI = 0.0f;
		else if (down && (panYI == 0.0f))
			panYI = -20.0f;
		break;
	case ICT_SECOND_PAN_UP:
		if (!down && (panYI > 0.0f))
			panYI = 0.0f;
		else if (down && (panYI == 0.0f))
			panYI = 20.0f;
		break;

	// zooming
	case ICT_ZOOM_IN:
		break;
	case ICT_ZOOM_OUT:
		break;

	// misc
	case ICT_RESET:
		panX = panY = panXI = panYI = 0.0f;
		mat1 = cmat1 = Matrix4x4::identity;
		mat2 = cmat2 = Matrix4x4::identity;
		break;
	}
}

static void TimerNormalView(float secs)
{
	rot1Y += rot1YI*secs;
	rot2Y += rot2YI*secs;
	rot1X += rot1XI*secs;
	rot2X += rot2XI*secs;
	panX += panXI*secs;
	panY += panYI*secs;

	Matrix4x4	tmp;

	if (rot1YI != 0.0f || rot1XI != 0.0f)
	{
		cmat1 = Matrix4x4::identity;
		tmp.SetRotation(Vector3(1, 0, 0), rot1X);	cmat1 *= tmp;
		tmp.SetRotation(Vector3(0, 1, 0), rot1Y);	cmat1 *= tmp;
	}
	else
	{
		tmp = cmat1;
		tmp *= mat1;
		mat1 = tmp;
		cmat1 = Matrix4x4::identity;
		rot1X = rot1Y = 0.0f;
	}

	if (rot2YI != 0.0f || rot2XI != 0.0f)
	{
		cmat2 = Matrix4x4::identity;
		tmp.SetRotation(Vector3(1, 0, 0), rot2X);	cmat2 *= tmp;
		tmp.SetRotation(Vector3(0, 1, 0), rot2Y);	cmat2 *= tmp;
	}
	else
	{
		tmp = cmat2;
		tmp *= mat2;
		mat2 = tmp;
		cmat2 = Matrix4x4::identity;
		rot2X = rot2Y = 0.0f;
	}
}


//// SPLIT VIEW MODE (combining objects)
static void RenderSplitView(Renderer &rend)
{
}

static void ControlSplitView(InvCtlType ctl, bool down)
{
}

static void TimerSplitView(float secs)
{
}


//// PUBLIC INTERFACE
static void Init()
{
	GCInitParams	ip = GetInitParams();

	sw = ip.dmode.width;
	sh = ip.dmode.height;


	static Matrix4x4	cmat;
	cmat.SetTranslation(Vector3(0, 0, 200));
	cam = new PRSCamera();
	cam->SetFOV(30.0);
	cam->SetPosition(Vector3(0, 0, -400));
	cam->SetRotation(Quaternion(Vector3(0, 1, 0), 0));

	scItem1 = new Scene();
	scItem2 = new Scene();

	//scItem1->AddObjectGroup("louk.T7SCE");
	//scItem2->AddObjectGroup("teapot.T7SCE");

	scItem1->SetActiveCamera(cam);
	scItem2->SetActiveCamera(cam);

	//ogItem1 = scItem1->GetObjectGroup(0);
	//ogItem2 = scItem2->GetObjectGroup(0);

	scItem1->BuildScene();
	scItem2->BuildScene();

	win1X1 = 0;
	win1X2 = 0.5;
	win2X1 = 0.5;
	win2X2 = 1;

	panX = panY = panXI = panYI = 0.0f;

	mat1 = cmat1 = Matrix4x4::identity;
	mat2 = cmat2 = Matrix4x4::identity;

	initialized = true;
}

void InventoryControl(InvCtlType ctl, bool down)
{
	if (splitview)
		ControlSplitView(ctl, down);
	else
		ControlNormalView(ctl, down);
}

void InventoryTimer(float secs)
{
	if (splitview)
		TimerSplitView(secs);
	else
		TimerNormalView(secs);
}

void InventoryEnter()
{
	if (!initialized)
	{
		Init();
	}

	tinner = LoadImage("inventory/inner.png");
	tseparator = LoadImage("inventory/separator.png");
	tframe = LoadImage("inventory/frame.png");
	titembar = LoadImage("inventory/itembar.png");
	tactive = LoadImage("inventory/active.png");
	
	tr1 = CreateRenderTarget(sw/2, sh, TFMT_32);
	tr2 = CreateRenderTarget(sw/2, sh, TFMT_32);

	rot1X = rot1Y = rot2X = rot2Y = 0.0f;

	/// GPU PROGRAM BEGIN ////
	gsgpuprog = CreateGPUProgram(
// vertex shader
"uniform float4x4 STATE_MODELVIEWPROJECTION_MATRIX; "
"uniform float4x4 STATE_TEXTURE0_MATRIX; "

"struct a2v "
"{ "
"	float4 pos	: POSITION; "
"	float2 tc	: TEXCOORD0; "
"	float4 col	: COLOR; "
"}; "

"a2v main(a2v IN) "
"{ "
"	a2v OUT; "

"	OUT.pos = mul(IN.pos, STATE_MODELVIEWPROJECTION_MATRIX); "
"	float4 tc4; "
"	tc4.xy = IN.tc.xy; "
"	tc4.z = tc4.w = 1; "

"	OUT.tc = (float2) mul(tc4, STATE_TEXTURE0_MATRIX); "

"	return OUT; "
"} ",
// fragment shader
"uniform sampler2D tex : TEXUNIT0; "
"struct v2p "
"{ "
"	float2 tc	: TEXCOORD0; "
"	float4 col	: COLOR; "
"}; "

"float4 main(v2p IN) : COLOR "
"{ "
"	float4 texel = tex2D(tex, IN.tc); "
"	texel.x *= IN.col.x; "	
"	texel.y *= IN.col.y; "
"	texel.z *= IN.col.z; "
"	texel.w *= IN.col.z; "

"	float gs = 0.3 * texel.x + 0.59 * texel.y + 0.11 * texel.z; "
"	texel.x = texel.y = texel.z = gs; "

"	return texel; "
"}");

	printf("%i\n", gsgpuprog);
	
	//// GPU PROGRAM END ////

	splitview = false;
}

void InventoryLeave()
{
	ReleaseGPUProgram(gsgpuprog);

	delete tactive;
	delete titembar;
	delete tframe;
	delete tseparator;
	delete tinner;

	DeleteTexture(tr2);
	DeleteTexture(tr1);

	if (scItem1)
	{
		delete scItem1;
		scItem1 = NULL;
	}
	if (scItem2)
	{
		delete scItem2;
		scItem2 = NULL;
	}
}

void InventoryRender(Renderer &rend)
{
	if (splitview) 
	{
		RenderSplitView(rend);
	}
	else
	{
		RenderNormalView(rend);
	}
}

void InventoryEdit(bool set, Matrix4x4 *left, Matrix4x4 *right, float *panx, float *pany)
{
	if (set)
	{
		mat1 = *left;
		mat2 = *right;
		panX = *panx;
		panY = *pany;
	}
	else
	{
		*left = mat1;
		*right = mat2;
		*panx = panX;
		*pany = panY;
	}
}

void InventoryCorrectSet(Matrix4x4 *left, Matrix4x4 *right, float panx, float pany)
{
	if (left)
	{
		refLeft = *left;
		refRight = *right;
		refPanX = panx;
		refPanY = pany;
	} else {
		refLeft = mat1;
		refRight = mat2;
		refPanX = panX;
		refPanY = panY;
	}
	printf("panx=%f   pany=%f\n", refPanX, refPanY);
}

static bool InvCheckVector(const Vector3 &v)
{
	Vector3	lv1 = v;
	Vector3	lv2 = v;
	Vector3	rv1 = v;
	Vector3	rv2 = v;

	lv1.Transform(mat1);
	lv2.Transform(refLeft);
	rv1.Transform(mat2);
	rv2.Transform(refRight);

	float	dot1 = DotProduct(lv1, lv2);
	float	dot2 = DotProduct(rv1, rv2);

//	printf("%f  %f\n", dot1, dot2);

	return (dot1 > 0.95f) && (dot2 > 0.95f);
}

bool InventoryMatchCheck()
{
	float	dx = fabs(refPanX - panX);
	float	dy = fabs(refPanY - panY);

	//printf("%f  %f\n", dx, dy);

	return InvCheckVector(Vector3(1, 0, 0)) &&
		InvCheckVector(Vector3(0, 1, 0)) &&
		InvCheckVector(Vector3(0, 0, 1)) &&
		(dx < 4.0f) && (dy < 4.0f);
}
