#ifndef _INVENTORY_H_
#define _INVENTORY_H_

#include <list>
#include <renderer.h>

class InventoryItem
{
	std::string	name;
	ObjectGroup	*og;
	class Inventory *inv;
	
public:
	InventoryItem(const std::string &name, ObjectGroup *og);
	~InventoryItem();

	const std::string &GetName() const;
	ObjectGroup *GetObjectGroup();

	class Inventory *getInventory() const;
	void setInventory(class Inventory *inv);
};

class Inventory
{
	std::list<InventoryItem*>	items;
public:
	void Add(InventoryItem* item);
	void Remove(InventoryItem* item);
	bool Has(InventoryItem* item);

	const std::list<InventoryItem*> &GetItemsList() const;
};

InventoryItem *RegisterInventoryItem(std::string name, ObjectGroup *og);
InventoryItem *InventoryFindItem(std::string name);

enum InvCtlType
{
	// first item rotation
	ICT_FIRST_ROTATE_LEFT,
	ICT_FIRST_ROTATE_RIGHT,
	ICT_FIRST_ROTATE_UP,
	ICT_FIRST_ROTATE_DOWN,
	// second item rotation
	ICT_SECOND_ROTATE_LEFT,
	ICT_SECOND_ROTATE_RIGHT,
	ICT_SECOND_ROTATE_UP,
	ICT_SECOND_ROTATE_DOWN,
	// second item panning
	ICT_SECOND_PAN_LEFT,
	ICT_SECOND_PAN_RIGHT,
	ICT_SECOND_PAN_UP,
	ICT_SECOND_PAN_DOWN,
	// misc
	ICT_ZOOM_IN,
	ICT_ZOOM_OUT,
	ICT_ACTIVATE,
	ICT_LEAVE,
	ICT_RESET
};

void InventoryControl(InvCtlType ctl, bool down);
void InventoryTimer(float secs);
void InventoryEnter();
void InventoryLeave();
void InventoryRender(Renderer &rend);
void InventoryEdit(bool set, Matrix4x4 *left, Matrix4x4 *right, float *panx, float *pany);
void InventoryCorrectSet(Matrix4x4 *left, Matrix4x4 *right, float panx, float pany);
bool InventoryMatchCheck();

#endif
