#ifndef GC_H
#define GC_H

#include <builddefs.h>
#include <color.h>
#include <math3d.h>
#include "gctypes.h"

#define MAX_BONES	66

struct DisplayMode {
    unsigned int width, height, bpp, frequency;
};

struct GCInitParams {
	DisplayMode dmode;
	unsigned int depth_bits;
	unsigned int stencil_bits;
	bool fullscreen;
	void *win_ext;
	GCInitParams(int w = 1024, int h = 768, bool fullscr = false);
};

bool InitGC(const GCInitParams &params);
GCInitParams GetInitParams();
void CloseGC();

void BeginScene();
void EndScene();

void Flip();

void SetClearColor(const Color &color);
void SetClearDepth(float depth);
void SetClearStencil(unsigned int s);

void Clear();
void Clear(unsigned int mask);

void SetColorMask(bool red, bool green, bool blue, bool alpha);
void SetColorMask(bool enabled);

void SetDepthMask(bool enabled);
bool GetDepthMask();
void SetDepthTest(bool enabled);
bool GetDepthTest();
void SetDepthFunc(CompareFunc func);

void SetStencil(bool enabled);
bool GetStencil();
void SetStencilFunc(CompareFunc func);
void SetStencilPass(StencilOp op);
void SetStencilFail(StencilOp op);
void SetStencilZFail(StencilOp op);
void SetStencilRef(int ref);

void SetBlending(bool enabled);
bool GetBlending();
void SetBlendFunc(BlendFunc src, BlendFunc dst);
BlendFunc GetBlendSrc();
BlendFunc GetBlendDst();

void SetAlphaTest(bool enabled);
void SetAlphaRef(float ref);
void SetAlphaFunc(CompareFunc func);

void SetCulling(CullFace face);
CullFace GetCulling();

void SetProjectionMatrix(const Matrix4x4 &m);
void SetViewMatrix(const Matrix4x4 &m);
void SetModelMatrix(const Matrix4x4 &m);
void SetTextureMatrix(unsigned int tid, const Matrix4x4 &m);

Matrix4x4 GetModelMatrix();
Matrix4x4 GetViewMatrix();
Matrix4x4 GetInvViewMatrix();
Matrix4x4 GetProjectionMatrix();
Matrix4x4 GetTextureMatrix(int unit = 0);
Matrix4x4 GetModelViewMatrix();

void SetBoneMatrix(unsigned int which, const Matrix4x4 &mat);

void SetViewport(int x, int y, int width, int height);
Vector2 GetViewport(int *x = 0, int *y = 0, int *width = 0, int *height = 0);

void SetLighting(bool enable);
bool GetLighting();

void SetTextureAddressing(int unit, TextureAddress addr);
void SetSamplerState(unsigned int sampler_id, SamplerStateType type, unsigned int value);
void SetTextureFiltering(int unit, TextureFilter minf, TextureFilter magf = TF_NONE, TextureFilter mipf = (TextureFilter)-1);
TextureFilter GetTextureFilteringMin(int unit);
TextureFilter GetTextureFilteringMag(int unit);
TextureFilter GetTextureFilteringMip(int unit);
void SetTexTransform(bool enable);
bool GetTexTransform();

void SetFillMode(FillMode mode);

Box GetFrustum();
// returns a picking ray in world space
// mpos is between [0, 1]
Ray GetPickingRay(Vector2 mpos);

void SetHalfPixelSize(int width, int height);

void SetScissorTest(bool enable);
bool GetScissorTest();
void SetScissorRect(int x1, int y1, int x2, int y2);
void SetScissorRect(const Vector2 &from, const Vector2 &to);

#endif //GC_H
