#define LIBGC_SOURCE_FILE
#include <builddefs.h>

#ifdef BUILD_DX9

#include <assert.h>
#include <libsys.h>
#include <console.h>
#include <errlog.h>
#include "sysgc.h"
#include "gc.h"
#include "gc_impl.h"
#include "statecache.h"
//#include "immed.h"
#include "textures.h"
#include "uniforms.h"
#include "gc_stats.h"

//#define GPU_PROF

extern D3DCAPS9 d3d_caps;

void SetInitParams(GCInitParams iparam);

static unsigned int fb_width, fb_height;

static unsigned int clear_color_value;
static float clear_depth_value;
static unsigned int clear_stencil_value;

static IDirect3D9 *d3d = NULL;
IDirect3DDevice9 *d3d_device = NULL;
//static GCState gc_state;

static bool texture_transforms = false;

static bool scissor_test = false;
static bool stencil_enabled = false;

bool InitGC (const GCInitParams &params)
{
#ifndef NDEBUG
	SetLogStream(stderr, LOG_ALL);
	SetLogFile("t7engine.log", LOG_ALL);
#endif
	if(!SysCreateWindow(params.dmode.width, params.dmode.height, !params.fullscreen, params.win_ext)) {
		Error("failed to create window\n");
		CloseGC ();
		return false;
	}
	
	if(!(d3d = Direct3DCreate9(D3D_SDK_VERSION))) {
		Error("failed to initialize Direct3D\n");
		CloseGC ();
		return false;
	}
	
	D3DPRESENT_PARAMETERS present_params;
	memset(&present_params, 0, sizeof present_params);

	fb_width = params.dmode.width;
	fb_height = params.dmode.height;

	present_params.BackBufferWidth = params.dmode.width;
	present_params.BackBufferHeight = params.dmode.height;
	present_params.BackBufferFormat = D3DFMT_X8R8G8B8;
	present_params.BackBufferCount = 3;
	present_params.MultiSampleType = D3DMULTISAMPLE_NONE;
	present_params.MultiSampleQuality = 0;
	present_params.SwapEffect = D3DSWAPEFFECT_DISCARD;
	present_params.hDeviceWindow = (HWND) SysGetWindow();
	present_params.Windowed = params.fullscreen ? FALSE:TRUE;
	present_params.EnableAutoDepthStencil = TRUE;
	present_params.AutoDepthStencilFormat = D3DFMT_D24S8;
	present_params.Flags = 0;
	present_params.FullScreen_RefreshRateInHz = params.dmode.frequency;
	present_params.PresentationInterval = D3DPRESENT_INTERVAL_IMMEDIATE;//DEFAULT;

	/* TODO: oi parametroi sto present_params den prepei na einai hardcoded... */

	DWORD vertex_processing_flags = (d3d_caps.DevCaps & D3DDEVCAPS_HWTRANSFORMANDLIGHT) ?
		D3DCREATE_HARDWARE_VERTEXPROCESSING : D3DCREATE_SOFTWARE_VERTEXPROCESSING;
	
	// samurai:
	// gia kapoio logo sthn karta mou den yparxei to hw tnl sta caps
	vertex_processing_flags = D3DCREATE_HARDWARE_VERTEXPROCESSING;

#ifndef GPU_PROF
	HRESULT hRes = d3d->CreateDevice(D3DADAPTER_DEFAULT,
			D3DDEVTYPE_HAL,
			(HWND)SysGetWindow(),
			vertex_processing_flags,
			&present_params,
			&d3d_device);
#else
	HRESULT hRes = d3d->CreateDevice(d3d->GetAdapterCount() - 1,
			D3DDEVTYPE_REF,
			(HWND) SysGetWindow(),
			vertex_processing_flags,
			&present_params,
			&d3d_device);
#endif // GPU_PROF

	if((FAILED(hRes)) || (d3d_device == NULL)) {
		Error("failed to create a D3D device context\n");
		CloseGC ();
		return false;
	}

	// XXX: is this needed ? if so, it should be made into a gc interface
	// function that is called from InitSubsys()
	float size = 5.0f;
	d3d_device->SetRenderState(D3DRS_POINTSIZE, *((DWORD*)&size));

	// TODO: move to SetInitParams
	d3d_device->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE );

	SetInitParams(params);
	return InitSubsys();
}

void CloseGC ()
{
	CloseSubsys();

	if(d3d_device) {
		d3d_device->Release();
		d3d_device = NULL;
	}
	if(d3d) {
		d3d->Release();
		d3d = NULL;
	}
	SysCloseWindow();
}


void BeginScene ()
{
	d3d_device->BeginScene ();
}

void EndScene ()
{
	d3d_device->EndScene ();
}


void Flip ()
{
	HRESULT res;
	res = d3d_device->Present (NULL, NULL, NULL, NULL);
	assert (!FAILED(res));
	StatsReset();
}


void SetClearColor (const Color &color)
{
	clear_color_value = PackColor(color);
}

void SetClearDepth (float f)
{
	if(f > 1.0) f = 1.0;
	if(f < 0.0) f = 0.0;
	clear_depth_value = f;
}

void SetClearStencil (unsigned int s)
{
	clear_stencil_value = s;
}

void Clear ()
{
	//TODO : na tsekarei ta caps kai na ftiaxnei to mask swsta
	if(FAILED(d3d_device->Clear (0, NULL, COLOR_BUFFER | DEPTH_BUFFER | STENCIL_BUFFER, clear_color_value, clear_depth_value, clear_stencil_value))) {
		return;
	}
}

void Clear (unsigned int mask)
{
	//TODO : na tsekarei ta caps kai na ftiaxnei to mask swsta
	if (FAILED(d3d_device->Clear (0, NULL, mask, clear_color_value, clear_depth_value, clear_stencil_value)))
	{
		return;
	}
}

void SetSamplerState (unsigned int sampler_id, SamplerStateType type, unsigned int value)
{
	d3d_device->SetSamplerState (sampler_id, (D3DSAMPLERSTATETYPE) type, value);
}


void SetColorMask (bool red, bool green, bool blue, bool alpha)
{
	unsigned int mask = 0;
	mask |= red ? D3DCOLORWRITEENABLE_RED : 0;
	mask |= green ? D3DCOLORWRITEENABLE_GREEN : 0;
	mask |= blue ? D3DCOLORWRITEENABLE_BLUE : 0;
	mask |= alpha ? D3DCOLORWRITEENABLE_ALPHA : 0;

	d3d_device->SetRenderState (D3DRS_COLORWRITEENABLE, mask);
}

void SetColorMask (bool enabled)
{
	SetColorMask (enabled, enabled, enabled, enabled);
}


void SetDepthMask (bool enabled)
{
	d3d_device->SetRenderState (D3DRS_ZWRITEENABLE, enabled ? TRUE : FALSE );
}

bool GetDepthMask()
{
	unsigned long val;
	d3d_device->GetRenderState(D3DRS_ZWRITEENABLE, &val);
	return (bool)val;
}

void SetDepthTest (bool enabled)
{
	d3d_device->SetRenderState (D3DRS_ZENABLE, enabled ? D3DZB_TRUE  : D3DZB_FALSE);
}

bool GetDepthTest()
{
	unsigned long val;
	d3d_device->GetRenderState(D3DRS_ZENABLE, &val);
	return (bool)val;
}

void SetDepthFunc (CompareFunc func)
{
	d3d_device->SetRenderState (D3DRS_ZFUNC, func);
}

void SetStencil(bool enabled)
{
	stencil_enabled = enabled;
	d3d_device->SetRenderState(D3DRS_STENCILENABLE, enabled ? TRUE : FALSE);
}

bool GetStencil()
{
	return stencil_enabled;
}

void SetStencilFunc(CompareFunc func)
{
	d3d_device->SetRenderState(D3DRS_STENCILFUNC, func);
}

void SetStencilPass(StencilOp op)
{
	d3d_device->SetRenderState(D3DRS_STENCILPASS, op);
}

void SetStencilFail(StencilOp op)
{
	d3d_device->SetRenderState(D3DRS_STENCILFAIL, op);
}

void SetStencilZFail(StencilOp op)
{
	d3d_device->SetRenderState(D3DRS_STENCILZFAIL, op);
}

void SetStencilRef(int ref)
{
	d3d_device->SetRenderState(D3DRS_STENCILREF, ref);
}

void SetBlending (bool enabled)
{
	d3d_device->SetRenderState (D3DRS_ALPHABLENDENABLE, enabled ? TRUE : FALSE);
}

bool GetBlending()
{
	unsigned long val;
	d3d_device->GetRenderState(D3DRS_ALPHABLENDENABLE, &val);
	return (bool)val;
}

void SetBlendFunc (BlendFunc src, BlendFunc dst)
{
	d3d_device->SetRenderState(D3DRS_SRCBLEND, src);
	d3d_device->SetRenderState(D3DRS_DESTBLEND, dst);
}

BlendFunc GetBlendSrc()
{
	unsigned long val;
	d3d_device->GetRenderState(D3DRS_SRCBLEND, &val);
	return (BlendFunc)val;
}

BlendFunc GetBlendDst()
{
	unsigned long val;
	d3d_device->GetRenderState(D3DRS_DESTBLEND, &val);
	return (BlendFunc)val;
}

void SetAlphaTest (bool enabled)
{
	d3d_device->SetRenderState (D3DRS_ALPHATESTENABLE, enabled ? TRUE : FALSE);
}

void SetAlphaRef (float ref)
{
	unsigned int rr = (unsigned int) (ref * 255);
	if (rr > 255) 
	{ 
		rr = 255;
	}
	d3d_device->SetRenderState (D3DRS_ALPHAREF, rr);
}

void SetAlphaFunc (CompareFunc func)
{
	d3d_device->SetRenderState (D3DRS_ALPHAFUNC, func);
}


void SetCulling (CullFace face)
{
	d3d_device->SetRenderState (D3DRS_CULLMODE, face);
}

CullFace GetCulling()
{
	unsigned long val;
	d3d_device->GetRenderState(D3DRS_CULLMODE, &val);
	return (CullFace)val;
}

void APISetProjectionMatrix(float *m)
{
	d3d_device->SetTransform(D3DTS_PROJECTION, (D3DMATRIX*)m);
}

void APISetViewMatrix(float *m)
{
	d3d_device->SetTransform(D3DTS_VIEW, (D3DMATRIX*)m);
}

void APISetModelMatrix(float *m)
{
	d3d_device->SetTransform(D3DTS_WORLD, (D3DMATRIX*)m);
}

// this does nothing in the d3d version
void APISetModelViewMatrix(float *m) {}

void APISetTextureMatrix(unsigned int tid, float *m)
{
	// TODO: set this
	d3d_device->SetTransform((D3DTRANSFORMSTATETYPE)(D3DTS_TEXTURE0 + tid), (D3DMATRIX*)m);
}

void SetTexTransform(bool enable)
{
	if (texture_transforms == enable)
		return;

	if (enable)
	{
		d3d_device->SetTextureStageState(0, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_COUNT2);
	}
	else
	{
		d3d_device->SetTextureStageState(0, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_DISABLE);
	}

	texture_transforms = enable;
}

bool GetTexTransform()
{
	return texture_transforms;
}

void SetViewport(int x, int y, int width, int height)
{
	D3DVIEWPORT9 d3dvp;
	d3dvp.X = x;
	d3dvp.Y = y;
	d3dvp.Width = width;
	d3dvp.Height = height;
	d3dvp.MinZ = 0.0f;
	d3dvp.MaxZ = 1.0f;
	d3d_device->SetViewport(&d3dvp);
}
Vector2 GetViewport(int *x, int *y, int *width, int *height)
{
	D3DVIEWPORT9 d3dvp;
	d3d_device->GetViewport(&d3dvp);

	if(x) *x = d3dvp.X;
	if(y) *y = d3dvp.Y;
	if(width) *width = d3dvp.Width;
	if(height) *height = d3dvp.Height;

	return Vector2((float)d3dvp.Width, (float)d3dvp.Height);
}

void SetLighting(bool enable)
{
	d3d_device->SetRenderState(D3DRS_LIGHTING, (unsigned int)enable);
}

bool GetLighting()
{
	unsigned long val;
	d3d_device->GetRenderState(D3DRS_LIGHTING, &val);
	return (bool)val;
}

void SetTextureAddressing(int unit, TextureAddress addr)
{
	d3d_device->SetSamplerState(unit, D3DSAMP_ADDRESSU, addr);
	d3d_device->SetSamplerState(unit, D3DSAMP_ADDRESSV, addr);
	d3d_device->SetSamplerState(unit, D3DSAMP_ADDRESSW, addr);
}

void SetTextureFiltering(int unit, TextureFilter minf, TextureFilter magf, TextureFilter mipf)
{
	if(magf == TF_NONE) magf = minf;
	if(mipf == (TextureFilter)-1) mipf = minf;

	d3d_device->SetSamplerState(unit, D3DSAMP_MINFILTER, minf);
	d3d_device->SetSamplerState(unit, D3DSAMP_MAGFILTER, magf);
	d3d_device->SetSamplerState(unit, D3DSAMP_MIPFILTER, mipf);
}

TextureFilter GetTextureFilteringMin(int unit)
{
	unsigned long val;
	d3d_device->GetSamplerState(unit, D3DSAMP_MINFILTER, &val);
	return (TextureFilter)val;
}

TextureFilter GetTextureFilteringMag(int unit)
{
	unsigned long val;
	d3d_device->GetSamplerState(unit, D3DSAMP_MAGFILTER, &val);
	return (TextureFilter)val;
}

TextureFilter GetTextureFilteringMip(int unit)
{
	unsigned long val;
	d3d_device->GetSamplerState(unit, D3DSAMP_MIPFILTER, &val);
	return (TextureFilter)val;
}

void SetFillMode(FillMode mode)
{
	d3d_device->SetRenderState (D3DRS_FILLMODE, mode);
}

void SetScissorTest(bool enable)
{
	d3d_device->SetRenderState(D3DRS_SCISSORTESTENABLE, enable);
	scissor_test = enable;
}

bool GetScissorTest()
{
	return scissor_test;
}

void SetScissorRect(int x1, int y1, int x2, int y2)
{
	RECT r;
	r.left = x1;
	r.top = y1;
	r.right = x2;
	r.bottom = y2;
	d3d_device->SetScissorRect(&r);
}

void SetScissorRect(const Vector2 &from, const Vector2 &to)
{
	SetScissorRect((int)(fb_width * from.x), 
		(int)(fb_height * from.y), 
		(int)(fb_width * to.x), 
		(int)(fb_height * to.y));
}

#endif	/* D3D build */
