// gc_stats.cpp

#include "gc_stats.h"
#include <map>

struct GcStats
{
	unsigned int batch_count;
	unsigned int triangle_count;

	std::map<unsigned int, unsigned int> col_map;
	unsigned int max_columns;
	unsigned int column_size;
};

static GcStats collecting, showing;
static unsigned int next_column_size = 500;

static void Init()
{
	collecting.batch_count = collecting.triangle_count = 0;
	collecting.max_columns = 0;
	collecting.column_size = 500;
}

void StatsAddBatch(unsigned int tri_count)
{
	static bool first_time = true;
	if (first_time)
	{
		Init();
		first_time = false;
	}

	collecting.batch_count++;
	collecting.triangle_count += tri_count;

	if (collecting.column_size)
	{
		unsigned int col = tri_count / collecting.column_size;
		collecting.col_map[col]++;
		if ((col+1) > collecting.max_columns)
		{
			collecting.max_columns = col + 1;
		}
	}
}

void StatsReset()
{
	showing = collecting;

	collecting.batch_count = collecting.triangle_count = 0;
	collecting.col_map.clear();
	collecting.max_columns = 0;
	collecting.column_size = next_column_size;
}

void StatsSetColumnSize(unsigned int sz)
{
	next_column_size = sz;
}

unsigned int StatsGetColumnSize()
{
	return showing.column_size;
}

unsigned int StatsGetBatchCount()
{
	return showing.batch_count;
}

unsigned int StatsGetBatchColumnCount()
{
	return showing.max_columns;
}

unsigned int StatsGetBatchCount(unsigned int column)
{
	return showing.col_map[column];
}

unsigned int StatsGetTriCount()
{
	return showing.triangle_count;
}

unsigned int StatsGetAverageTrisPerBatch()
{
	if (!showing.batch_count) return 0;
	return showing.triangle_count / showing.batch_count;
}
