#ifndef GCTYPES_H_
#define GCTYPES_H_

#include <builddefs.h>
#include "sysgc.h"

/* -------- DirectX GC type definitions --------- */
#ifdef BUILD_DX9

enum Buffers
{
	COLOR_BUFFER = D3DCLEAR_TARGET,
	DEPTH_BUFFER = D3DCLEAR_ZBUFFER,
	STENCIL_BUFFER = D3DCLEAR_STENCIL
};


enum CompareFunc
{
	CMP_NEVER = D3DCMP_NEVER,
	CMP_LESS = D3DCMP_LESS,
	CMP_LEQUAL = D3DCMP_LESSEQUAL,
	CMP_EQUAL = D3DCMP_EQUAL,
	CMP_GREATER = D3DCMP_GREATER,
	CMP_NOTEQUAL = D3DCMP_NOTEQUAL,
	CMP_GEQUAL = D3DCMP_GREATEREQUAL,
	CMP_ALWAYS = D3DCMP_ALWAYS
};


enum BlendFunc
{
	BLEND_ZERO = D3DBLEND_ZERO,
	BLEND_ONE = D3DBLEND_ONE,
	BLEND_SRC_COLOR = D3DBLEND_SRCCOLOR,
	BLEND_ONE_MINUS_SRC_COLOR = D3DBLEND_INVSRCCOLOR,
	BLEND_SRC_ALPHA = D3DBLEND_SRCALPHA,
	BLEND_ONE_MINUS_SRC_ALPHA = D3DBLEND_INVSRCALPHA,
	BLEND_DST_ALPHA = D3DBLEND_DESTALPHA,
	BLEND_ONE_MINUS_DST_ALPHA = D3DBLEND_INVDESTALPHA,
	BLEND_DST_COLOR = D3DBLEND_DESTCOLOR,
	BLEND_ONE_MINUS_DST_COLOR = D3DBLEND_INVDESTCOLOR,
	BLEND_SRC_ALPHA_SATURATE = D3DBLEND_SRCALPHASAT
};

enum CullFace
{
	CULL_NONE = D3DCULL_NONE,  
	CULL_CW = D3DCULL_CW,
	CULL_CCW = D3DCULL_CCW 
};

enum StencilOp
{
	STENCIL_KEEP = D3DSTENCILOP_KEEP,
	STENCIL_ZERO = D3DSTENCILOP_ZERO,
	STENCIL_REPLACE = D3DSTENCILOP_REPLACE,
	STENCIL_INCRSAT = D3DSTENCILOP_INCRSAT,
	STENCIL_DECRSAT = D3DSTENCILOP_DECRSAT,
	STENCIL_INVERT = D3DSTENCILOP_INVERT,
	STENCIL_INCR = D3DSTENCILOP_INCR,
	STENCIL_DECR = D3DSTENCILOP_DECR
};

enum FillMode
{
	FILL_POINT = D3DFILL_POINT,
    FILL_WIREFRAME = D3DFILL_WIREFRAME,
    FILL_SOLID = D3DFILL_SOLID
};

enum SamplerStateType
{
	ST_ADDRESSU = D3DSAMP_ADDRESSU,
	ST_ADDRESSV = D3DSAMP_ADDRESSV,
	ST_ADDRESSW = D3DSAMP_ADDRESSW,
	ST_BORDERCOLOR = D3DSAMP_BORDERCOLOR,
	ST_MAGFILTER = D3DSAMP_MAGFILTER,
	ST_MINFILTER = D3DSAMP_MINFILTER,
	ST_MIPFILTER = D3DSAMP_MIPFILTER,
	ST_MIPMAPLODBIAS = D3DSAMP_MIPMAPLODBIAS,
	ST_MAXMIPLEVEL = D3DSAMP_MAXMIPLEVEL,
	ST_MAXANISOTROPY = D3DSAMP_MAXANISOTROPY,
	ST_SRGBTEXTURE = D3DSAMP_SRGBTEXTURE,
	ST_ELEMENTINDEX = D3DSAMP_ELEMENTINDEX,
	ST_DMAPOFFSET = D3DSAMP_DMAPOFFSET
};

enum TextureAddress
{
	TA_WRAP = D3DTADDRESS_WRAP,
	TA_MIRROR = D3DTADDRESS_MIRROR,
	TA_CLAMP = D3DTADDRESS_CLAMP,
	TA_BORDER = D3DTADDRESS_BORDER,
	TA_MIRRORONCE = D3DTADDRESS_MIRRORONCE
};

enum TextureFilter
{
	TF_NONE = D3DTEXF_NONE,
	TF_POINT = D3DTEXF_POINT,
	TF_LINEAR = D3DTEXF_LINEAR,
	TF_ANISOTROPIC = D3DTEXF_ANISOTROPIC,
	TF_PYRAMIDALQUAD = D3DTEXF_PYRAMIDALQUAD,
	TF_GAUSSIANQUAD = D3DTEXF_GAUSSIANQUAD
};

enum TextureFormat {
	TFMT_32 = D3DFMT_A8R8G8B8,
	TFMT_24 = D3DFMT_R8G8B8,
	TFMT_16 = D3DFMT_R5G6B5,
	TFMT_8 = D3DFMT_L8,
	TFMT_FLOAT_16 = D3DFMT_R16F,
	TFMT_FLOAT_32 = D3DFMT_R32F,
	TFMT_FLOAT16_RGBA = D3DFMT_A16B16G16R16F,
	TFMT_FLOAT32_RGBA = D3DFMT_A32B32G32R32F,
	TFMT_COMPRESSED_32 = D3DFMT_DXT5,
	TFMT_COMPRESSED_24 = D3DFMT_DXT1
};
#define TFMT_INVALID	((TextureFormat)0)

enum TextureType {
	TEX_2D,
	TEX_CUBE
};
#define TEX_INVALID		((TextureType)0)

enum CubemapFace {
	CUBE_FACE_PX = D3DCUBEMAP_FACE_POSITIVE_X,
	CUBE_FACE_NX = D3DCUBEMAP_FACE_NEGATIVE_X,
	CUBE_FACE_PY = D3DCUBEMAP_FACE_POSITIVE_Y,
	CUBE_FACE_NY = D3DCUBEMAP_FACE_NEGATIVE_Y,
	CUBE_FACE_PZ = D3DCUBEMAP_FACE_POSITIVE_Z,
	CUBE_FACE_NZ = D3DCUBEMAP_FACE_NEGATIVE_Z
};
	

enum PrimitiveType
{
	PT_POINT_LIST = D3DPT_POINTLIST,
	PT_LINE_LIST = D3DPT_LINELIST,
	PT_LINE_STRIP = D3DPT_LINESTRIP,
	PT_TRIANGLE_LIST = D3DPT_TRIANGLELIST,
	PT_TRIANGLE_STRIP = D3DPT_TRIANGLESTRIP,
	PT_TRIANGLE_FAN = D3DPT_TRIANGLEFAN,
	PT_MAKE_DWORD = 0x7fffffff
};

#endif	/* BUILD_DX9 */



/* -------- OpenGL type definitions --------- */
#ifdef BUILD_GL

enum Buffers {
	COLOR_BUFFER	= GL_COLOR_BUFFER_BIT,
	DEPTH_BUFFER	= GL_DEPTH_BUFFER_BIT,
	STENCIL_BUFFER	= GL_STENCIL_BUFFER_BIT
};


enum CompareFunc {
	CMP_NEVER		= GL_NEVER,
	CMP_LESS		= GL_LESS,
	CMP_LEQUAL		= GL_LEQUAL,
	CMP_EQUAL		= GL_EQUAL,
	CMP_GREATER		= GL_GREATER,
	CMP_NOTEQUAL	= GL_NOTEQUAL,
	CMP_GEQUAL		= GL_GEQUAL,
	CMP_ALWAYS		= GL_ALWAYS
};


enum BlendFunc {
	BLEND_ZERO					= GL_ZERO,
	BLEND_ONE					= GL_ONE,
	BLEND_SRC_COLOR				= GL_SRC_COLOR,
	BLEND_ONE_MINUS_SRC_COLOR	= GL_ONE_MINUS_SRC_COLOR,
	BLEND_SRC_ALPHA				= GL_SRC_ALPHA,
	BLEND_ONE_MINUS_SRC_ALPHA	= GL_ONE_MINUS_SRC_ALPHA,
	BLEND_DST_ALPHA				= GL_DST_ALPHA,
	BLEND_ONE_MINUS_DST_ALPHA	= GL_ONE_MINUS_DST_ALPHA,
	BLEND_DST_COLOR				= GL_DST_COLOR,
	BLEND_ONE_MINUS_DST_COLOR	= GL_ONE_MINUS_DST_COLOR,
	BLEND_SRC_ALPHA_SATURATE	= GL_SRC_ALPHA_SATURATE
};

enum CullFace {
	CULL_NONE,
	CULL_CW		= GL_CCW,	/* XXX: this is defined as the opposite, since in */
	CULL_CCW	= GL_CW		/* OpenGL you specify the front face, not the face to be culled. */
};

enum StencilOp
{
	STENCIL_KEEP = GL_KEEP,
	STENCIL_ZERO = GL_ZERO,
	STENCIL_REPLACE = GL_REPLACE,
	STENCIL_INCRSAT = GL_INCR,
	STENCIL_DECRSAT = GL_DECR,
	STENCIL_INVERT = GL_INVERT,
	STENCIL_INCR = GL_INCR,	/* XXX: find appropriate wrapping mode if available */
	STENCIL_DECR = GL_DECR	/* XXX: find appropriate wrapping mode if available */
};

enum FillMode
{
	FILL_POINT		= GL_POINT,
    FILL_WIREFRAME	= GL_LINE,
    FILL_SOLID		= GL_FILL
};

enum SamplerStateType {
	ST_ADDRESSU			= GL_TEXTURE_WRAP_S,
	ST_ADDRESSV			= GL_TEXTURE_WRAP_T,
	ST_ADDRESSW			= GL_TEXTURE_WRAP_R,
	ST_BORDERCOLOR		= GL_TEXTURE_BORDER_COLOR,
	ST_MAGFILTER,	/* handle these three with code, since there is */
	ST_MINFILTER,	/* no special MIP_FILTER state in OpenGL */
	ST_MIPFILTER,
	ST_MIPMAPLODBIAS,	/* TODO */
	ST_MAXMIPLEVEL,		/* TODO */
	ST_MAXANISOTROPY,	/* TODO */
	ST_SRGBTEXTURE,		/* TODO */
	ST_ELEMENTINDEX,	/* TODO */
	ST_DMAPOFFSET		/* TODO */
};

enum TextureAddress {
	TA_WRAP			= GL_REPEAT,
	TA_MIRROR,		/* TODO: is there an OpenGL equivalent? */
	TA_CLAMP		= GL_CLAMP_TO_EDGE,
	TA_BORDER		= GL_CLAMP,
	TA_MIRRORONCE	/* TODO: is there an OpenGL equivalent? */
};

/* these must be handled with code due to differences in mipmap
 * filtering specification with OpenGL
 */
enum TextureFilter {
	TF_NONE,
	TF_POINT,
	TF_LINEAR,
	TF_ANISOTROPIC,
	TF_PYRAMIDALQUAD,
	TF_GAUSSIANQUAD
};

enum TextureFormat {
	TFMT_32				= GL_BGRA,
	TFMT_24				= GL_RGB,
	TFMT_16				= GL_RGB16,
	TFMT_8				= GL_LUMINANCE,
	TFMT_FLOAT_16		= GL_RGBA16F_ARB,
	TFMT_FLOAT_32		= GL_RGBA32F_ARB,
	TFMT_FLOAT16_RGBA	= GL_RGBA16F_ARB,
	TFMT_FLOAT32_RGBA	= GL_RGBA32F_ARB,
	TFMT_COMPRESSED_32	= GL_COMPRESSED_RGBA,
	TFMT_COMPRESSED_24	= GL_COMPRESSED_RGB
};
#define TFMT_INVALID	((TextureFormat)0)

enum TextureType {
	TEX_2D		= GL_TEXTURE_2D,
	TEX_CUBE	= GL_TEXTURE_CUBE_MAP
};
#define TEX_INVALID		((TextureType)0)

enum PrimitiveType
{
	PT_POINT_LIST		= GL_POINTS,
	PT_LINE_LIST		= GL_LINES,
	PT_LINE_STRIP		= GL_LINE_STRIP,
	PT_TRIANGLE_LIST	= GL_TRIANGLES,
	PT_TRIANGLE_STRIP	= GL_TRIANGLE_STRIP,
	PT_TRIANGLE_FAN		= GL_TRIANGLE_FAN,
	PT_QUAD_LIST		= GL_QUADS,
	PT_MAKE_DWORD		= 0x7fffffff
};


#endif	/* BUILD_GL */

#endif	/* GCTYPES_H_ */
