#ifndef IMMED_H
#define IMMED_H

#include <limits.h>
#include "math3d.h"
#include "color.h"

enum MatrixMode {
	IM_MODEL,
	IM_VIEW,
	IM_PROJECTION,
	IM_TEXTURE,

	IM_MATRIX_MODE_COUNT
};

enum ImPrimitiveType {
	IM_POINTS,
	IM_LINES,
	IM_LINE_STRIP,
	IM_TRIANGLES,
	IM_TRIANGLE_STRIP,
	IM_TRIANGLE_FAN,
	IM_QUADS,
	IM_BILLBOARDS
};

bool ImInit();
void ImDestroy();

void ImFixedFunction (bool ff = true);
bool ImFixedFunctionUsed();

void ImBegin(ImPrimitiveType ptype);
void ImEnd();

void ImVertex(float x, float y, float z);
void ImVertex(const Vector3 &v);

void ImNormal(float x, float y, float z);
void ImNormal(const Vector3 &v);

void ImColor(float r, float g, float b, float a = 1.0f);
void ImColor(const Color &c);

void ImTexCoord(unsigned int tunit, float u, float v);
void ImTexCoord(unsigned int tunit, const Vector2 &v);
void ImTexCoord(float u, float v);

void ImTangent(float x, float y, float z);
void ImTangent(const Vector3 &tangent);

void ImBillboardSize(float s);

void ImMatrixMode(MatrixMode mode);
void ImLoadIdentity();
void ImLoadMatrix(const Matrix4x4 &m);
void ImMultMatrix(const Matrix4x4 &m);
void ImPushMatrix();
void ImPopMatrix();
void ImTranslate(float x, float y, float z);
void ImRotate(float angle, float x, float y, float z);
void ImScale(float x, float y, float z);

void ImSetApplyXForm(bool enable);
void ImApplyXForm();

void ImWireBox(const Box &box);
void ImQuad(const Quad &quad);

#define IM_TEX_CUR	UINT_MAX
#define IM_SDR_CUR	UINT_MAX
void ImOverlay(const Vector2 &v1, const Vector2 &v2, const Color &col = 1.0f, unsigned int tex = 0, unsigned int gfxprog = 0);

/* random helpers */

void ImGrid(float spacing, float sz);

#endif	// IMMED_H

