#ifndef LIGHTING_H
#define LIGHTING_H

#include <builddefs.h>
#include <color.h>
#include <math3d.h>
#include <libvfs.h>

void SetLightAmbient (unsigned int lid, const Color &color);
void SetLightDiffuse (unsigned int lid, const Color &color);
void SetLightSpecular (unsigned int lid, const Color &color);
void SetLightPos (unsigned int lid, const Vector3 &pos);
void SetLightDir (unsigned int lid, const Vector3 &dir);
void SetLightInner (unsigned int lid, float inner);
void SetLightOuter (unsigned int lid, float outer);
void SetLightAttenStart (unsigned int lid, float start);
void SetLightAttenEnd (unsigned int lid, float end);

void SetMatAmbient (const Color &color);
void SetMatDiffuse (const Color &color);
void SetMatSpecular (const Color &color);
void SetMatEmissive (const Color &color);
void SetMatPower (float power);


struct Material
{
	Color ambient, diffuse, specular, emissive;
	float power;

	Material ();
	bool SaveToFP (VFS_FILE *fp);
	bool LoadFromFP (VFS_FILE *fp);
};

void SetMaterial (const Material &mat);

enum LightType
{
	LT_DIR_LIGHT,
	LT_POINT_LIGHT,
	LT_SPOT_LIGHT
};

struct Light
{
	LightType lt;
	Color ambient, diffuse, specular;
	Vector3 pos, dir;
	float inner, outer, atten_start, atten_end;

	Light ();
};

void SetLight (unsigned int lid, const Light &light);


#endif //LIGHTING_H

