#ifndef TEXTURES_H
#define TEXTURES_H

#include <builddefs.h>
#include <math3d.h>
#include "gctypes.h"

#define MAX_TEXTURE_UNITS 8

unsigned int AddHWTexture (int w, int h, TextureType type, TextureFormat fmt, bool mip, void *hw_tex);
unsigned int CreateTexture(int w, int h, TextureType type, TextureFormat fmt, bool mip = true, unsigned int id = 0);
bool SetTexImage(unsigned int tid, void *pixels, int w, int h, TextureFormat fmt, int face = 0);
bool GetTexImage(unsigned int tid, void *pixels, TextureFormat fmt = TFMT_32, int face = 0);
unsigned int CreateRenderTarget(int w, int h, TextureFormat fmt);
unsigned int CreateCubeRenderTarget(int w, int h, TextureFormat fmt);
void DeleteTexture(unsigned int tid);
bool BindTexture(unsigned int unit, unsigned int tid);

#define MRT_ALL		-1
bool PushRenderTarget(int mrt_id = 0);
bool PopRenderTarget(int mrt_id = 0);

#define PushRenderTargets()		PushRenderTarget(MRT_ALL)
#define PopRenderTargets()		PopRenderTarget(MRT_ALL)

bool SetRenderTarget(unsigned int tid, unsigned int mrt_id = 0, unsigned int cube_face = 6);
Matrix4x4 GetRenderTargetTexMat(unsigned int tid);

unsigned int GetTextureWidth(unsigned int tid);
unsigned int GetTextureHeight(unsigned int tid);
TextureType GetTextureType(unsigned int tid);
TextureFormat GetTextureFormat(unsigned int tid);
bool IsRenderTarget(unsigned int tid);
unsigned int GetMipLevels(unsigned int tid);

size_t GetTextureMemUsage(unsigned int tid);

#endif	// TEXTURES_H
