#include <string.h>
#include "idman.h"
#include "uniforms.h"


struct UniformDesc
{
	char uniform_name [MAX_UNIFORM_NAME];
	float *uniform_values;
	unsigned int uniform_values_size;
};

static void DeleteHWUniform (void *u);


IdManRef uniform_list(DeleteHWUniform);

void DeleteHWUniform (void *u)
{
	UniformDesc *uniform = (UniformDesc *) u;
	if (uniform->uniform_values)
	{
		delete [] uniform->uniform_values;
	}
	delete uniform;
}


unsigned int AddUniform (const char *uniform_name, unsigned int float_count)
{
	unsigned int uniform = FindUniform (uniform_name);

	if (uniform != 0)
	{
		UniformDesc *uni = (UniformDesc*) uniform_list.Get (uniform);
		if (uni->uniform_values_size == float_count)
		{
			uniform_list.AddRef (uniform);
			return uniform;
		}
		return 0;
	}

	if (uniform_name == 0)
	{
		return 0;
	}
	if (strlen (uniform_name) >= MAX_UNIFORM_NAME)
	{
		return 0;
	}
	if (float_count == 0)
	{
		return 0;
	}

	UniformDesc *u = new UniformDesc;
	memset (u->uniform_name, 0, MAX_UNIFORM_NAME);
	strcpy (u->uniform_name, uniform_name);
	u->uniform_values = new float [float_count];
	u->uniform_values_size = float_count;
	uniform = uniform_list.Add (u);
	if (uniform == 0)
	{
		DeleteHWUniform (u);
	}
	return uniform;
}

void DeleteUniform (unsigned int u)
{
	uniform_list.Remove (u);

}

unsigned int FindUniform (const char *name)
{
	if (name == 0)
	{
		return 0;
	}
	if (strlen (name) == 0)
	{
		return 0;
	}
	for (unsigned int i = 1; i < uniform_list.GetResCount(); i++)
	{
		if (uniform_list.IsHandle (i))
		{
			UniformDesc *uni = (UniformDesc*) uniform_list.Get (i);
			if (strcmp (uni->uniform_name, name) == 0)
			{
				return i;
			}
		}
	}
	return 0;
}

unsigned int FirstUniform ()
{
	return NextUniform (0);
}

unsigned int NextUniform (unsigned int last_uniform)
{
	for (unsigned int i = last_uniform + 1; i < uniform_list.GetResCount (); i++)
	{
		if (uniform_list.IsHandle (i))
		{
			return i;
		}
	}
	return 0;
}

const char *GetUniformName (unsigned int u)
{
	if (uniform_list.IsHandle (u))
	{
		UniformDesc *uniform = (UniformDesc*) uniform_list.Get (u);
		return uniform->uniform_name;
	}
	return 0;
}

unsigned int GetUniformFloats (unsigned int u)
{
	if (uniform_list.IsHandle (u))
	{
		UniformDesc *uniform = (UniformDesc*) uniform_list.Get (u);
		return uniform->uniform_values_size;
	}
	return 0;
}

bool SetUniform (unsigned int u, void *data)
{
	UniformDesc *uniform = (UniformDesc*) uniform_list.Get (u);
	if (uniform)
	{
		memcpy (uniform->uniform_values, data, uniform->uniform_values_size * sizeof (float));
		return true;
	}
	return false;
}

bool GetUniform (unsigned int u, void *data)
{
	UniformDesc *uniform = (UniformDesc*) uniform_list.Get (u);
	if (uniform)
	{
		if (data)
			memcpy (data, uniform->uniform_values, uniform->uniform_values_size * sizeof (float));
		return true;
	}
	return false;
}

void *GetUniformDataPtr (unsigned int u)
{
	UniformDesc *uniform = (UniformDesc*) uniform_list.Get (u);
	if (uniform)
	{
		return uniform->uniform_values;
	}
	return 0;
}
