#ifndef VERTEX_FORMAT_H
#define VERTEX_FORMAT_H

#include <builddefs.h>
#include <libvfs.h>

#define MAX_TEX_COORDS   8


struct VertexFormat
{
    bool vertex;
    bool normal;
    bool tangent;
    bool color;
    bool bone_weight;
    bool bone_index;
    unsigned int tc_count;
    unsigned int tc_stream_count [MAX_TEX_COORDS];

    VertexFormat ();

    bool IsEqual (const VertexFormat &vf) const;
	bool operator==(const VertexFormat &vf) const;
	bool operator!=(const VertexFormat &vf) const;

    bool ContainsFormat (const VertexFormat &vf) const;
    unsigned int GetSize () const;


    unsigned int GetVertexOffset () const;
    unsigned int GetNormalOffset () const;
    unsigned int GetColorOffset () const;
    unsigned int GetBoneWeightOffset () const;
    unsigned int GetBoneIndexOffset () const;
    unsigned int GetTangentOffset () const;
    unsigned int GetTexCoordOffset (unsigned int idx) const;

    unsigned int GetVertexSize () const;
    unsigned int GetNormalSize () const;
    unsigned int GetColorSize () const;
    unsigned int GetBoneWeightSize () const;
    unsigned int GetBoneIndexSize () const;
    unsigned int GetTangentSize () const;
    unsigned int GetTexCoordSize (unsigned int idx) const;

	unsigned int GetOrdinal () const;

	bool WriteToFP (VFS_FILE *fp);
	bool LoadFromFP (VFS_FILE *fp);
};


#endif //VERTEX_FORMAT_H

