#ifndef ELEMENTS_H
#define ELEMENTS_H

#include <libvfs.h>

enum ElementType
{
    ET_VERTEX,
    ET_NORMAL,
    ET_TEXCOORD,
    ET_TANGENT,
    ET_COLOR,
    ET_INDEX,
	ET_BONE_INDEX,
	ET_BONE_WEIGHT
};

enum BasicType
{
    BT_FLOAT,
    BT_CHAR,
    BT_UINT,
    BT_USHORT
};



class ElementArray
{
protected:
    ElementType element_type;
    BasicType basic_type;
    unsigned int basic_type_count;

    unsigned int element_size;

    char *data;
    unsigned int data_size_in_bytes;
    unsigned int elements_count;
public:
    ElementArray (ElementType et, BasicType bt, unsigned int bt_count);
    virtual ~ElementArray ();

    virtual unsigned int GetBasicTypeCount () const;
    virtual unsigned int GetElementsCount () const;
    virtual unsigned int GetSingleElementSize () const;
	virtual void GetElement(void *dest, int idx) const;
	virtual void SetElement(void *src, int idx);

    virtual void *Create (unsigned int element_count);
    virtual void *GetData ();
    virtual void Release ();

	virtual void Remap (unsigned int *remap_index);

    virtual bool MapWithStride (void *dest, unsigned int stride, unsigned int map_element_count = 0);

    virtual bool LoadFromFP (VFS_FILE *fp);
	virtual bool WriteToFP (VFS_FILE *fp);
};

class  VertexArray : public ElementArray
{
public:
    VertexArray ();
    virtual ~VertexArray ();
};

class  NormalArray : public ElementArray
{
public:
    NormalArray ();
    virtual ~NormalArray ();
};

class  TexCoordArray : public ElementArray
{
public:
    TexCoordArray (unsigned int tex_coords = 2);
    virtual ~TexCoordArray ();
};

class  TangentArray : public ElementArray
{
public:
    TangentArray ();
    virtual ~TangentArray ();
};

class  ColorArray : public ElementArray 
{
public:
    ColorArray ();
    virtual ~ColorArray ();
};

class  IndexArray : public ElementArray
{
public:
    IndexArray ();
    virtual ~IndexArray ();
};

class BoneIndexArray : public ElementArray
{
public:
	BoneIndexArray();
	virtual ~BoneIndexArray();
};

class BoneWeightArray : public ElementArray
{
public:
	BoneWeightArray();
	virtual ~BoneWeightArray();
};


#endif //ELEMENTS_H

