#include <stdlib.h>
#include <gc.h>
#include <immed.h>
#include <textures.h>
#include "imgdisp.h"

static bool MakeTexture(Image *img);

void ShowImage(Image *img, float x, float y)
{
	ShowImage(img, x, y, 1.0);
}

void ShowImage(Image *img, float x, float y, float sz)
{
	GCInitParams initp = GetInitParams();
	float xsz = (float)img->GetWidth() / (float)initp.dmode.width;
	float ysz = (float)img->GetHeight() / (float)initp.dmode.height;

	ShowImage(img, x, y, x + xsz, y + ysz);
}

void ShowImage(Image *img, float x1, float y1, float x2, float y2)
{
	if(!img->tex_id) {
		if(!MakeTexture(img)) {
			return;
		}
	}
	ImOverlay(Vector2(x1, y1), Vector2(x2, y2), 1.0f, img->tex_id);
}

static bool MakeTexture(Image *img)
{
	if(img->tex_id) return true;

	int xsz = img->GetWidth();
	int ysz = img->GetHeight();
	unsigned int *pixptr = (unsigned int*)img->GetPixels();
	unsigned int *imgbuf = pixptr;

	// createtexture makes a bigger texture as needed, automatically...
	if(!(img->tex_id = CreateTexture(xsz, ysz, TEX_2D, TFMT_32, false))) {
		return false;
	}

	int tex_xsz = GetTextureWidth(img->tex_id);
	int tex_ysz = GetTextureHeight(img->tex_id);
	bool need_pad = (tex_xsz != xsz || tex_ysz != ysz);

	// if we don't have non-pow2 texture support and the image is not a power of two...
	if(need_pad) {
		// create a pixel buffer that contains the image with the necessary padding
		if(!(imgbuf = (unsigned int*)malloc(tex_xsz * tex_ysz * sizeof *imgbuf))) {
			DeleteTexture(img->tex_id);
			img->tex_id = 0;
			return false;
		}

		unsigned int *dest = imgbuf;
		for(int y=0; y<tex_ysz; y++) {
			if(y < ysz) {
				memcpy(dest, pixptr, xsz * sizeof *dest);
				memset(dest + xsz, 0, (tex_xsz - xsz) * sizeof *dest);
			} else {
				memset(dest, 0, tex_xsz * sizeof *dest);
			}
			dest += tex_xsz;
			pixptr += xsz;
		}
	}

	if(!SetTexImage(img->tex_id, imgbuf, tex_xsz, tex_ysz, TFMT_32)) {
		DeleteTexture(img->tex_id);
		img->tex_id = 0;
		if(need_pad) free(imgbuf);
		return false;
	}

	if(need_pad) {
		free(imgbuf);
	}

	return true;
}
