#include "renderer.h"

Renderer::~Renderer() {}

void Renderer::SetupTransform(const Camera *cam, unsigned int cube_face) const
{
	// set matrices
	if(cam) {
		if (cube_face < 6)
		{
			SetProjectionMatrix(cam->GetProjectionMatrixCube());
			SetViewMatrix(cam->GetViewMatrixCube(cube_face));
		}
		else
		{
			float fov = cam->GetFOV();
			float aspect = cam->GetAspect();
			float nearclip = cam->GetNearClip();
			float farclip = cam->GetFarClip();

			Matrix4x4 proj = PerspectiveProj(fov, aspect, nearclip, farclip);
			SetProjectionMatrix(proj);
			SetViewMatrix(cam->GetViewMatrix());
		}
	} else {
		// default matrix
		Matrix4x4 proj = PerspectiveProj(60.0f, 1.33333f, 1.0f, 10000.0f);
		SetProjectionMatrix(proj);
		Matrix4x4 view;
		view.Translate(Vector3(0, 0, 200));
		SetViewMatrix(view);
	}

	Matrix4x4 model;
	SetModelMatrix(model);
}

void Renderer::SetParam(const char *pname, int val)
{
	map<std::string, RendererParam>::iterator iter = param.find(pname);
	if(iter == param.end()) {
		param[pname].type = RPTYPE_INT;
		param[pname].pint = val;
	} else {
		RendererParam *p = &param[pname];
		switch(p->type) {
		case RPTYPE_INT:
			p->pint = val;
			break;
		case RPTYPE_FLOAT:
			p->pflt = (float)val;
			break;
		case RPTYPE_BOOL:
			p->pbool = val;
			break;
		default:
			break;
		}
	}
}

void Renderer::SetParam(const char *pname, double val)
{
	map<std::string, RendererParam>::iterator iter = param.find(pname);
	if(iter == param.end()) {
		param[pname].type = RPTYPE_FLOAT;
		param[pname].pflt = val;
	} else {
		RendererParam *p = &param[pname];
		switch(p->type) {
		case RPTYPE_INT:
			p->pint = (int)val;
			break;
		case RPTYPE_FLOAT:
			p->pflt = val;
			break;
		case RPTYPE_BOOL:
			p->pbool = (int)val;
			break;
		default:
			break;
		}
	}
}

void Renderer::SetParam(const char *pname, bool val)
{
	map<std::string, RendererParam>::iterator iter = param.find(pname);
	if(iter == param.end()) {
		param[pname].type = RPTYPE_INT;
		param[pname].pbool = val;
	} else {
		RendererParam *p = &param[pname];
		switch(p->type) {
		case RPTYPE_INT:
			p->pint = val;
			break;
		case RPTYPE_FLOAT:
			p->pflt = (float)val;
			break;
		case RPTYPE_BOOL:
			p->pbool = val;
			break;
		default:
			break;
		}
	}
}

int Renderer::GetParamInt(const char *pname) const
{
	map<std::string, RendererParam>::iterator iter = param.find(pname);
	if(iter == param.end()) {
		return 0;
	}

	RendererParam *p = &param[pname];
	switch(p->type) {
	case RPTYPE_INT:
		return p->pint;
	case RPTYPE_FLOAT:
		return (int)p->pflt;
	case RPTYPE_BOOL:
		return p->pbool;
	default:
		break;
	}
	return 0;
}

double Renderer::GetParamFloat(const char *pname) const
{
	map<std::string, RendererParam>::iterator iter = param.find(pname);
	if(iter == param.end()) {
		return 0.0;
	}

	RendererParam *p = &param[pname];
	switch(p->type) {
	case RPTYPE_INT:
		return (double)p->pint;
	case RPTYPE_FLOAT:
		return p->pflt;
	case RPTYPE_BOOL:
		return (double)p->pbool;
	default:
		break;
	}
	return 0.0;
}

bool Renderer::GetParamBool(const char *pname) const
{
	map<std::string, RendererParam>::iterator iter = param.find(pname);
	if(iter == param.end()) {
		return false;
	}

	RendererParam *p = &param[pname];
	switch(p->type) {
	case RPTYPE_INT:
		return p->pint;
	case RPTYPE_FLOAT:
		return (bool)p->pflt;
	case RPTYPE_BOOL:
		return p->pbool;
	default:
		break;
	}
	return false;
}
