#ifndef RENDERER_H
#define RENDERER_H

#include "scene.h"

enum RPType {RPTYPE_INT, RPTYPE_FLOAT, RPTYPE_BOOL};
struct RendererParam {
	enum RPType type;
	union {
		int pint;
		float pflt;
		bool pbool;
	};
};


class Renderer {
private:
	mutable std::map<std::string, RendererParam> param;

protected:
	virtual void SetupTransform(const Camera *cam, unsigned int cube_face=6) const;

public:
	virtual ~Renderer();

	virtual bool Init() = 0;
	virtual void Release() = 0;
	virtual void Render(const Scene *scene) const = 0;

	virtual void SetParam(const char *pname, int val);
	virtual void SetParam(const char *pname, double val);
	virtual void SetParam(const char *pname, bool val);

	virtual int GetParamInt(const char *pname) const;
	virtual double GetParamFloat(const char *pname) const;
	virtual bool GetParamBool(const char *pname) const;
};

#endif //RENDERER_H
