#ifndef RENDERER_MRT_H
#define RENDERER_MRT_H

#include "renderer.h"
#include "smesh.h"

enum {
	MRT_POS,		// rgb: position (TODO: not needed, calc from depth) a: free
	MRT_COLOR,		// rgb: color (ambient + diffuse) a: specular level
	MRT_NORMAL,		// rgb: normal, a: height
	MRT_LIGHTMAP,	// rgb: lightmap

	MRT_ENUM_COUNT
};

#define MAX_LIGHT_SHADERS	16

/* -- MRT renderer parameters --
 * SHOW_LIGHTS: bool	- if enabled, shows the light meshes
 * SHOW_DEBUG_BUF: int  - shows the selected debug per-pixel property
 * 							0 - disable debug buffer output
 *							1 - positions
 *							2 - normals
 *							3 - colors
 *							4 - lightmap texels
 *							5 - heightfields (from normal/height buffer)
 * LRANGE_SCALE: float	- global light range multiplier
 * DEPTH_FACTOR: float	- global depth factor for the parallax shader
 */

class RendererMRTDeferred : public Renderer {
protected:
	SimpleMesh *light_mesh[RL_TYPE_COUNT];
	unsigned int ltmesh_sdr;

	unsigned int rt[MRT_ENUM_COUNT];
	unsigned int amb_sdr;
	unsigned int mrt_sdr, mrt_skin_sdr;
	unsigned int light_sdr[MAX_LIGHT_SHADERS];

	//unsigned int uni_light_rad, uni_light_pos;
	//unsigned int uni_light_dir, uni_light_cone;
	unsigned int uni_depth_fact;

	void RenderDeferredPass(const Scene *scene) const;
	void RenderLights(const Scene *scene) const;
	//void RenderShadows(const Scene *scene) const;

	int GetLightSdrIndex(int ltype) const;

public:
	RendererMRTDeferred();
	virtual ~RendererMRTDeferred();

	virtual bool Init();
	virtual void Release();

	virtual void Render(const Scene *scene) const;
};

#endif //RENDERER_MRT_H
