#ifndef _RENDERER_SIMPLE_H_
#define _RENDERER_SIMPLE_H_

#include "renderer.h"

class SimpleRenderer : public Renderer
{
protected:
	int rigid_shader, rigid_shader_shadow;
	int skin_shader, skin_shader_shadow;
	int color_shader;
	int ambient_shader;
	int ambient_skin_shader;
	int overlay_shader;
	int depthsq_shader;
	int depthsq_skin_shader;

	int zburn_rigid_shader;
	int zburn_skin_shader;

	// DEBUG
	unsigned int cube_tex;

	struct VisResult
	{
		std::vector<RObject*> skinned_obj, rigid_obj;
		std::vector<RLight*> global_lights;
		std::vector<PVS*> active_sets;
		std::map<PVS*, std::vector<RLight*> > pvs_specific_lights;
	};

	mutable VisResult vr[10];
	mutable const Scene *curr_scene;

	mutable unsigned int curr_vr;

	// settings
	bool cast_shadows;
public:
	SimpleRenderer();

	virtual bool Init();
	virtual void Release();
	virtual void Render(const Scene *scene) const;
	void Render(const Scene *scene, const Camera *can, unsigned int cube_face = 6) const;
	void RenderCube(const Scene *scene, const Camera *cam, unsigned int rt) const;
	void RenderProxy(const Scene *scene) const;

	bool hl_prop_bit;
	unsigned int highlight_property;
	CollisionMesh *highlight_proxy;
	bool render_only_highlight;

	void ProcessVisibility(bool meshes_only = false) const;
	void ZBurnPass() const;
	void AmbientPass() const;
	void LightPass(RLight *lt) const;
	void PVSPass(PVS *set) const;
	void RenderAll(unsigned int rigid, unsigned int skinned) const;

	void PumpRObject(const RObject *robj, bool zburn, bool shadow) const;

	// settings
	virtual void SetParam(const char *pname, bool val);
};

#endif // ndef _RENDERER_SIMPLE_H_
