#include "rlight.h"
#include <lighting.h>
#include <gc.h>

RLight::RLight()
{
	type = RL_OMNI;
	position = Vector3(0.0f, 0.0f, 0.0f);
	direction = Vector3(0.0f, 0.0f, 1.0f);
	atten_start = 200.0f;
	atten_end = 250.0f;
	inner = 30.0f;
	outer = 60.0f;
	pvs_specific = true;
	pvs = 0;
	cast_shadows = false;
}

RLight::~RLight() {}

void RLight::SetType(unsigned int type)
{
	this->type = type;
}

unsigned int RLight::GetType() const
{
	return type;
}

void RLight::SetPosition(const Vector3 &pos)
{
	position = pos;
}

Vector3 RLight::GetPosition() const
{
	return position;
}

void RLight::SetDirection(const Vector3 &dir)
{
	direction = dir;
}

Vector3 RLight::GetDirection() const
{
	return direction;
}

void RLight::SetAttenStart(float start)
{
	atten_start = start;
}

float RLight::GetAttenStart() const
{
	return atten_start;
}

void RLight::SetAttenEnd(float end)
{
	atten_end = end;
}

float RLight::GetAttenEnd() const
{
	return atten_end;
}

void RLight::SetInnerCone(float cone)
{
	inner = cone;
}

float RLight::GetInnerCone() const
{
	return inner;
}

void RLight::SetOuterCone(float cone)
{
	outer = cone;
}

float RLight::GetOuterCone() const
{
	return outer;
}

void RLight::SetAmbient(const Color &color)
{
	ambient = color;
}

Color RLight::GetAmbient() const
{
	return ambient;
}

void RLight::SetDiffuse(const Color &color)
{
	diffuse = color;
}

Color RLight::GetDiffuse() const
{
	return diffuse;
}

void RLight::SetSpecular(const Color &color)
{
	specular = color;
}

Color RLight::GetSpecular() const
{
	return specular;
}

void RLight::SetOrientation(const Quaternion &q)
{
	orientation = q;
}

Quaternion RLight::GetOrientation() const
{
	return orientation;
}

void RLight::Bind(unsigned int lid) const
{
	SetLightAmbient(lid, ambient);
	SetLightDiffuse(lid, diffuse);
	SetLightSpecular(lid, specular);
	SetLightPos(lid, position);
	SetLightDir(lid, direction);
	SetLightInner(lid, inner);
	SetLightOuter(lid, outer);
	SetLightAttenStart(lid, atten_start);
	SetLightAttenEnd(lid, atten_end);
}

Sphere RLight::GetBoundingSphere() const
{
	if (type == RL_DIRECTIONAL) return Sphere();
	if (type == RL_SPOT)
	{
		return GetOutSphereOfCone(position, GetConeCenter(), GetConeRange());
	}
	return Sphere(position, atten_end);
}
