#ifndef RLIGHT_H
#define RLIGHT_H


#include <math3d.h>
#include <color.h>
#include <pvs.h>

enum RLightType {
	RL_OMNI,
	RL_SPOT,
	RL_DIRECTIONAL,

	RL_TYPE_COUNT
};

class RLight {
protected:
	Color ambient, diffuse, specular;
	unsigned int type;
	Vector3 position, direction;
	float atten_start, atten_end, inner, outer;
	Quaternion orientation;

	bool pvs_specific;
	PVS *pvs;

	bool cast_shadows;

public:
	RLight();
	~RLight();

	void SetType(unsigned int type);
	unsigned int GetType() const;

	void SetPosition(const Vector3 &pos);
	Vector3 GetPosition() const;

	void SetDirection(const Vector3 &dir);
	Vector3 GetDirection() const;

	void SetAttenStart(float start);
	float GetAttenStart() const;
	void SetAttenEnd(float end);
	float GetAttenEnd() const;

	void SetInnerCone(float cone);
	float GetInnerCone() const;

	void SetOuterCone(float cone);
	float GetOuterCone() const;

	void SetAmbient(const Color &color);
	Color GetAmbient() const;
	void SetDiffuse(const Color &color);
	Color GetDiffuse() const;
	void SetSpecular(const Color &color);
	Color GetSpecular() const;
	void SetOrientation(const Quaternion &q);
	Quaternion GetOrientation() const;

	void Bind(unsigned int lid) const;

	float GetConeRange() const {return atten_end*tanf(outer/2.0f);}
	Vector3 GetConeCenter() const {return position + direction * atten_end;}

	bool IsPVSSpecific() const {return pvs_specific;}
	void SetPVSSpecific(bool enable) {pvs_specific = enable;}

	PVS *GetPVS() {return pvs;}
	void SetPVS(PVS *pvs) {this->pvs = pvs;}

	bool CastShadows() const {return cast_shadows;}
	void SetShadowCasting(bool enable) {cast_shadows = enable;}

	Sphere GetBoundingSphere() const;
};

#endif //RLIGHT_H
