// shadergen_simple.cpp
// shader generator for simple renderer

#include "sdrgen_simple.h"
#include <devhelp.h>
#include <errlog.h>
#include <gpuprog.h>
#include <libvfs.h>
#include <string>
#include <stdlib.h>

#define VSDR_PATH	DEV_DATA_PATH "/shaders/renderer_simple/vertex/"
#define PSDR_PATH	DEV_DATA_PATH "/shaders/renderer_simple/pixel/"

//////////////////////////////////////////////////////////////////////////
// bitmask decomposition
//////////////////////////////////////////////////////////////////////////
static inline bool Skinned(unsigned int mask)
{
	return mask & SGS_SKIN == SGS_SKIN;
}
static inline unsigned int LightType(unsigned int mask)
{
	return mask & SGS_DIRECTIONAL;
}
static inline bool Shadows(unsigned int mask)
{
	return mask & SGS_SHADOWS == SGS_SHADOWS;
}
static inline bool Fog(unsigned int mask)
{
	return mask & SGS_FOG == SGS_FOG;
}
static inline unsigned int Shading(unsigned int mask)
{
	return mask & SGS_RELIEF;
}

//////////////////////////////////////////////////////////////////////////
// filenames
//////////////////////////////////////////////////////////////////////////
static std::string Filename(unsigned int mask)
{
	std::string ret;

	ret += Skinned(mask) ? 's' : 'r';

	switch(LightType(mask)) {
	case SGS_AMBIENT:
		ret += 'a';
		break;
	case SGS_OMNI:
		ret += 'o';
		break;
	case SGS_SPOT:
		ret += 's';
		break;
	case SGS_DIRECTIONAL:
		ret += 'd';
		break;
	default:
		break;
	}

	ret += Shadows(mask) ? 's' : 'n';
	ret += Fog(mask) ? 'f' : 'n';

	switch (Shading(mask)) {
	case SGS_FLAT:
		ret += 'f';
		break;
	case SGS_NORMALMAP:
		ret += 'n';
		break;
	case SGS_PARALLAX:
		ret += 'p';
		break;
	case SGS_RELIEF:
		ret += 'r';
		break;
	default:
		break;
	}

	return ret;
}

static std::string FullName(unsigned int mask, bool vertex)
{
	std::string ret;
	ret += vertex ? VSDR_PATH : PSDR_PATH;
	ret += Filename(mask);
	ret += vertex ? "_v.cg" : "_p.cg";
	return ret;
}

//////////////////////////////////////////////////////////////////////////
// interface
//////////////////////////////////////////////////////////////////////////

/*
unsigned int SGS_GetShader(unsigned int mask)
{
	for(unsigned int i=0; i<SGS_MAX_SHADERS; i++) {
		BasicString sdr, fname;
		sdr += GenVertexShader(i);
		fname += "sdr/";
		fname += Filename(i);
		fname += "_v.cg";

		unsigned int size = sdr.Size();

		// save the shader
		VFS_FILE *fp = vfs_open(fname.c_str(), "wb");
		if(!fp) {
			Error("Could not open %s for writing\n", fname.c_str());
			continue;
		}

		vfs_write((void*)sdr.c_str(), 1, sdr.Size(), fp);

		vfs_close(fp);
	}
	return 0;
}
*/
