#include <vbuffer.h>
#include <errlog.h>
#include "smesh.h"

SimpleMesh::SimpleMesh(SimpleVertex *varr, int vcount, unsigned int *iarr, int icount)
{
	arr.vert = geom.CreateVertexArray();
	arr.norm = geom.CreateNormalArray();
	arr.tex = geom.CreateTexCoordArray(0);
	arr.tang = geom.CreateTangentArray();
	arr.bidx = geom.CreateBoneIndexArray();
	arr.bweight = geom.CreateBoneWeightArray();
	arr.idx = geom.CreateIndexArray();

	this->varr = 0;
	this->vcount = 0;
	vbo = ibo = 0;

	if(varr) {
		SetVertexData(varr, vcount);
	}
	if(iarr) {
		SetIndexData(iarr, icount);
	}
}

SimpleMesh::~SimpleMesh()
{
	geom.ReleaseVertexArray();
	geom.ReleaseNormalArray();
	geom.ReleaseTexCoordArray(0);
	geom.ReleaseTangentArray();
	geom.ReleaseBoneIndexArray();
	geom.ReleaseBoneWeightArray();
	geom.ReleaseIndexArray();
}

bool SimpleMesh::SetVertexData(SimpleVertex *varr, int vcount)
{
	Vector3 *vert, *norm, *tang;
	Vector2 *tex;
	unsigned char *bidx;
	float *bweight;

	if(vcount != (int)arr.vert->GetElementsCount()) {
		vert = (Vector3*)arr.vert->Create(vcount);
		norm = (Vector3*)arr.norm->Create(vcount);
		tex = (Vector2*)arr.tex->Create(vcount);
		tang = (Vector3*)arr.tang->Create(vcount);
		bidx = (unsigned char*)arr.bidx->Create(vcount);
		bweight = (float*)arr.bweight->Create(vcount);

		if(vbo) DeleteVertexBuffer(vbo);
		if(!(vbo = CreateVertexBuffer(geom.GetVertexFormat(), vcount, BUFFER_STATIC))) {
			return false;
		}

	} else {
		vert = (Vector3*)arr.vert->GetData();
		norm = (Vector3*)arr.norm->GetData();
		tex = (Vector2*)arr.tex->GetData();
		tang = (Vector3*)arr.tang->GetData();
		bidx = (unsigned char*)arr.bidx->GetData();
		bweight = (float*)arr.bweight->GetData();
	}

	for(int i=0; i<vcount; i++) {
		*vert++ = varr->pos;
		*norm++ = varr->norm;
		*tang++ = varr->tang;
		*tex++ = varr->tex;
		
		for(int j=0; j<3; j++) {
			*bidx++ = varr->bidx[j];
			*bweight++ = varr->bweight[j];
		}
		*bidx++ = 0;

		varr++;
	}
	return UpdateVertexBuffer(vbo, 0, &geom);
}

bool SimpleMesh::SetIndexData(unsigned int *iarr, int icount)
{
	unsigned int *idx;
	if(icount != (int)arr.idx->GetElementsCount()) {
		idx = (unsigned int*)arr.idx->Create(icount);

		if(ibo) DeleteIndexBuffer(ibo);
		if(!(ibo = CreateIndexBuffer(icount, BUFFER_STATIC))) {
			return false;
		}
	} else {
		idx = (unsigned int*)arr.idx->GetData();
	}

	memcpy(idx, iarr, icount * sizeof *idx);
	return UpdateIndexBuffer(ibo, 0, &geom, 0);
}

SimpleVertex *SimpleMesh::GetVertexData()
{
	return (SimpleVertex*)((const SimpleMesh*)this)->GetVertexData();
}

const SimpleVertex *SimpleMesh::GetVertexData() const
{
	//this is all wrong:
	// You CANNOT cast vertex array data to Vector3, because sizeof(Vector3)
	// is NOT necessarily 12.
	/*
	system("break");
	Vector3 *vert = (Vector3*)arr.vert->GetData();
	Vector3 *norm = (Vector3*)arr.norm->GetData();
	Vector2 *tex = (Vector2*)arr.tex->GetData();
	Vector3 *tang = (Vector3*)arr.tang->GetData();
	unsigned char *bidx = (unsigned char*)arr.bidx->GetData();
	float *bweight = (float*)arr.bweight->GetData();

	if(vcount != (int)arr.vert->GetElementsCount()) {
		vcount = arr.vert->GetElementsCount();
		delete [] varr;
		try {
			varr = new SimpleVertex[vcount];
		}
		catch(...) {
			return 0;
		}
	}

	for(int i=0; i<vcount; i++) {
		varr[i].pos = *vert++;
		varr[i].norm = *norm++;
		varr[i].tang = *tang++;
		varr[i].tex = *tex++;
		
		for(int j=0; j<3; j++) {
			varr[i].bidx[j] = *bidx++;
			varr[i].bweight[j] = *bweight++;
		}
		*bidx++ = 0;
	}

	return varr;
	*/
	Error("you may not enter here\n");
	return 0;
}

unsigned int *SimpleMesh::GetIndexData()
{
	return (unsigned int*)arr.idx->GetData();
}

const unsigned int *SimpleMesh::GetIndexData() const
{
	return (const unsigned int*)arr.idx->GetData();
}

int SimpleMesh::GetIndexCount() const
{
	return arr.idx->GetElementsCount();
}

void SimpleMesh::Draw() const
{
	unsigned int pcount = arr.vert->GetElementsCount() / 3;

	BindVertexBuffer(vbo);
	RenderPrimitives(PT_TRIANGLE_LIST, 0, pcount);
}

void SimpleMesh::DrawIndexed() const
{
	unsigned int pcount = arr.idx->GetElementsCount() / 3;
	unsigned int vcount = arr.vert->GetElementsCount();

	BindVertexBuffer(vbo);
	BindIndexBuffer(ibo);
	RenderIndexedPrimitives(PT_TRIANGLE_LIST, 0, pcount, 0, vcount);
}
