#ifndef _SMESH_H_
#define _SMESH_H_

#include <math3d.h>
#include "geometry.h"

struct SimpleVertex {
	Vector3 pos, norm, tang;
	Vector2 tex;
	unsigned char bidx[4];
	float bweight[3];
};

class SimpleMesh {
private:
	Geometry geom;
	unsigned int vbo, ibo;

	struct {
		VertexArray *vert;
		NormalArray *norm;
		TangentArray *tang;
		TexCoordArray *tex;
		BoneIndexArray *bidx;
		BoneWeightArray *bweight;

		IndexArray *idx;
	} arr;

	mutable SimpleVertex *varr;
	mutable int vcount;

public:
	SimpleMesh(SimpleVertex *varr = 0, int vcount = 0, unsigned int *iarr = 0, int icount = 0);
	~SimpleMesh();

	bool SetVertexData(SimpleVertex *varr, int vcount);
	bool SetIndexData(unsigned int *iarr, int icount);

	SimpleVertex *GetVertexData();
	const SimpleVertex *GetVertexData() const;
	int GetVertexCount() const;

	unsigned int *GetIndexData();
	const unsigned int *GetIndexData() const;
	int GetIndexCount() const;

	void Draw() const;
	void DrawIndexed() const;
};

#endif	/* _SMESH_H_ */
