#ifndef IMAGE_H_
#define IMAGE_H_

#include <builddefs.h>

/* always compile support for image formats that
 * don't create any library dependencies.
 */
#define IMGLIB_USE_TGA
#define IMGLIB_USE_PPM
#define IMGLIB_USE_DDS
/*#define IMGLIB_USE_RGBE*/

/* for jpeg and png, make the default be to compile them in,
 * unless explicitly disabled.
 */
#ifndef IMGLIB_NO_JPEG
#define IMGLIB_USE_JPEG
#endif	/* IMGLIB_NO_JPEG */

#ifndef IMGLIB_NO_PNG
#define IMGLIB_USE_PNG
#endif	/* IMGLIB_NO_PNG */


enum {
	IMG_FMT_PNG,
	IMG_FMT_JPEG,
	IMG_FMT_TGA,
	IMG_FMT_PPM,
	IMG_FMT_DDS,
	IMG_FMT_RGBE,

	IMG_FMT_LAST
};

enum {
	IMG_SAVE_ALPHA		= 1,
	IMG_SAVE_COMPRESS	= 2,
	IMG_SAVE_INVERT		= 4
};

enum {
	IMG_LOAD_FMT_ORIG	= 1
};


#ifdef __cplusplus
extern "C" {
#endif	/* __cplusplus */

/* load_image() loads the specified image from file, returns the pixel data
 * in 32bit mode, and changes xsz and ysz to the size of the image
 */
void *ImgLoadImage(const char *fname, unsigned int *xsz, unsigned int *ysz);

/* deallocate the image data with this function
 * note: provided for consistency, simply calls free()
 */
void ImgFreeImage(void *img);

/* save the supplied image data in a file of the specified format */
int ImgSaveImage(const char *fname, void *pixels, unsigned int xsz, unsigned int ysz, unsigned int fmt);

/* set/get save image options */
void ImgSetImageSaveFlags(unsigned int flags);
unsigned int ImgGetImageSaveFlags(void);

/* set/get load image options */
void ImgSetImageLoadFlags(unsigned int flags);
unsigned int ImgGetImageLoadFlags(void);

/* set mipmap level load */
void ImgSetMipmapLoadLevel(int mip);
int ImgGetMipmapLoadlevel(void);

#ifdef __cplusplus
}
#endif	/* __cplusplus */
	
#endif	/* IMAGE_H_ */
