#include <stdio.h>
#include "image.h"

#ifdef IMGLIB_USE_JPEG

#include <stdlib.h>

#include "jpeglib/jpeglib.h"
#include <byteorder.h>
#include <libvfs.h>
#include <color_bits.h>
#include <errlog.h>

/*#define LAST_SOI_HACK*/

typedef struct { 
	unsigned char r, g, b;
} RGBTriplet;

static void InitSource(j_decompress_ptr jd)
{
}

static int FillInputBuffer(j_decompress_ptr jd)
{
	/* fake an end of image */
	unsigned char buf[2] = {0xff, JPEG_EOI};
	jd->src->next_input_byte = buf;
	jd->src->bytes_in_buffer = 2;
	return 1;
}

static void SkipInputData(j_decompress_ptr jd, long num_bytes)
{
	if(num_bytes > 0) {
		jd->src->next_input_byte += num_bytes;
		jd->src->bytes_in_buffer -= num_bytes;
	}
}

static void TermSource(j_decompress_ptr jd)
{
}

/*jpeg signature*/
int CheckJpeg(VFS_FILE *fp){
	unsigned char sig[10];

	vfs_seek(fp, 0, VFS_SEEK_SET);
	vfs_read(sig, 1, 10, fp);
	
	if(sig[0]!=0xff || sig[1]!=0xd8 || sig[2]!=0xff || sig[3]!=0xe0) {
		return 0;
	}

	if(/*sig[6]!='J' ||*/sig[7]!='F' || sig[8]!='I' || sig[9]!='F') {
		return 0;
	}
	
	return 1;
}

void *LoadJpeg(VFS_FILE *fp, unsigned int *xsz, unsigned int *ysz) {
	unsigned int i;
	RGBTriplet *buffer;
	uint32_t *image;
	JSAMPLE *tmp;
	struct jpeg_decompress_struct cinfo;
	struct jpeg_error_mgr jerr;
	struct jpeg_source_mgr jsrc;
	long fsize;
	unsigned char *file_data;

	cinfo.err = jpeg_std_error(&jerr);
	jpeg_create_decompress(&cinfo);

	/* load the whole file into memory */
	vfs_seek(fp, 0, SEEK_END);
	fsize = vfs_tell(fp);
	vfs_seek(fp, 0, SEEK_SET);

	if(!(file_data = malloc(fsize))) {
		Error("LoadJpeg failed to allocate memory\n");
		return 0;
	}
	
	if(vfs_read(file_data, 1, fsize, fp) != fsize) {
		Error("LoadJpeg failed to read the whole file into memory\n");
		free(file_data);
		return 0;
	}

	/*fseek(vfs_get_real_file(fp), 0, SEEK_SET);
	jpeg_stdio_src(&cinfo, vfs_get_real_file(fp));*/

	jsrc.init_source = InitSource;
	jsrc.fill_input_buffer = FillInputBuffer;
	jsrc.skip_input_data = SkipInputData;
	jsrc.resync_to_restart = jpeg_resync_to_restart;
	jsrc.term_source = TermSource;
	jsrc.next_input_byte = file_data;
	jsrc.bytes_in_buffer = fsize;
	cinfo.src = &jsrc;

	/* ? */
	/*jpeg_save_markers(&cinfo, JPEG_COM, 0xffff);
	jpeg_save_markers(&cinfo, JPEG_APP0 + 1, 0xffff);
	jpeg_save_markers(&cinfo, JPEG_APP0 + 2, 0xffff);*/
	
	jpeg_read_header(&cinfo, TRUE);
	
	/* force output to rgb */
	cinfo.out_color_space = JCS_RGB;
	
	/* allocate space */
	if(!(buffer = malloc(cinfo.image_width * sizeof(RGBTriplet)))) {
		Error("LoadJpeg failed to allocate memory\n");
		free(file_data);
		return 0;
	}
	if(!(image = malloc(cinfo.image_width * (cinfo.image_height) * sizeof(uint32_t)))) {
		Error("LoadJpeg failed to allocate memory\n");
		free(file_data);
		free(buffer);
		return 0;
	}
	
	tmp = (JSAMPLE*) buffer;

	/* return w and h */
	*xsz = cinfo.image_width;
	*ysz = cinfo.image_height;
	
	/* Decompress, pack and store */
	jpeg_start_decompress(&cinfo);
	while(cinfo.output_scanline < cinfo.output_height) {
		jpeg_read_scanlines(&cinfo, &tmp, 1);
		if(cinfo.output_scanline == 0) continue;
		
		for (i=0; i<cinfo.image_width; i++) {
			int offs = i + (cinfo.output_scanline-1) * cinfo.output_width;
			image[offs] = PACK_COLOR24(buffer[i].r, buffer[i].g, buffer[i].b);
		}
	}
	jpeg_finish_decompress(&cinfo);
	free(file_data);
	
	/*Done - cleanup*/
	jpeg_destroy_decompress(&cinfo);
	free(buffer);

	return image;
}

/* TODO: implement this */
int SaveJpeg(FILE *fp, void *pixels, unsigned int xsz, unsigned int ysz) {
	Error("saving jpeg files is not implemented yet\n");
	return -1;
}

#endif	/* IMGLIB_USE_JPEG */
