#include <stdio.h>
#include "image.h"

#ifdef IMGLIB_USE_PPM

#include <stdlib.h>
#include <ctype.h>

#include <libvfs.h>
#include <color_bits.h>
#include <errlog.h>


int CheckPpm(VFS_FILE *fp) {
	vfs_seek(fp, 0, VFS_SEEK_SET);
	if(vfs_getc(fp) == 'P' && vfs_getc(fp) == '6') {
		return 1;
	}
	return 0;
}

static int ReadToWspace(VFS_FILE *fp, char *buf, int bsize) {
	int c, count = 0;
	
	while((c = vfs_getc(fp)) != -1 && !isspace(c) && count < bsize - 1) {
		if(c == '#') {
			while((c = vfs_getc(fp)) != -1 && c != '\n' && c != '\r');
			c = vfs_getc(fp);
			if(c == '\n' || c == '\r') continue;
		}
		*buf++ = c;
		count++;
	}
	*buf = 0;
	
	while((c = vfs_getc(fp)) != -1 && isspace(c));
	vfs_ungetc(c, fp);
	return count;
}

void *LoadPpm(VFS_FILE *fp, unsigned int *xsz, unsigned int *ysz) {
	char buf[64];
	int bytes, raw;
	unsigned int w, h, i, sz;
	uint32_t *pixels;
	
	vfs_seek(fp, 0, VFS_SEEK_SET);
	
	bytes = ReadToWspace(fp, buf, 64);
	raw = buf[1] == '6';

	if((bytes = ReadToWspace(fp, buf, 64)) == 0) {
		return 0;
	}
	if(!isdigit(*buf)) {
		Error("load_ppm: invalid width: %s\n", buf);
		return 0;
	}
	w = atoi(buf);

	if((bytes = ReadToWspace(fp, buf, 64)) == 0) {
		return 0;
	}
	if(!isdigit(*buf)) {
		Error("load_ppm: invalid height: %s\n", buf);
		return 0;
	}
	h = atoi(buf);

	if((bytes = ReadToWspace(fp, buf, 64)) == 0) {
		return 0;
	}
	if(!isdigit(*buf) || atoi(buf) != 255) {
		Error("load_ppm: invalid or unsupported max value: %s\n", buf);
		return 0;
	}

	if(!(pixels = malloc(w * h * sizeof *pixels))) {
		Error("malloc failed\n");
		return 0;
	}

	sz = h * w;
	for(i=0; i<sz; i++) {
		int r = vfs_getc(fp);
		int g = vfs_getc(fp);
		int b = vfs_getc(fp);

		if(r == -1 || g == -1 || b == -1) {
			free(pixels);
			Error("load_ppm: EOF while reading pixel data\n");
			return 0;
		}
		pixels[i] = PACK_COLOR24(r, g, b);
	}

	if(xsz) *xsz = w;
	if(ysz) *ysz = h;
	return pixels;
}

int SavePpm(VFS_FILE *fp, void *pixels, unsigned int xsz, unsigned int ysz) {
	int i, sz = xsz * ysz;
	uint32_t *ptr = pixels;
	
	vfs_printf(fp, "P6\n%lu %lu\n255\n# 3dengfx PPM file writer\n", xsz, ysz);

	for(i=0; i<sz; i++) {
		int r = (*ptr & RED_MASK32) >> RED_SHIFT32;
		int g = (*ptr & GREEN_MASK32) >> GREEN_SHIFT32;
		int b = (*ptr++ & BLUE_MASK32) >> BLUE_SHIFT32;

		if(vfs_putc(r, fp) == EOF || vfs_putc(g, fp) == EOF || vfs_putc(b, fp) == EOF) {
			Error("save_ppm: failed to write to file\n", stderr);
			return -1;
		}
	}
	return 0;
}

#endif	/* IMGLIB_USE_PPM */
