/*
 * Post-Processing Effects
 */
 
#ifndef __LIBPPE_H_INCLUDED__
#define __LIBPPE_H_INCLUDED__

#include <string.h>

struct PFXBuffer
{
	unsigned int	rt;	// render target
	unsigned int	width;
	unsigned int	height;
	bool			ownrt;

	PFXBuffer();
	~PFXBuffer();

	static PFXBuffer *Create(int width, int height);
};

struct PFXArg
{
	char	*name;
	float	value;
	bool	time;	// if true, then the value is time*timescale
};

struct PFXShader
{
	char	*name;
	
	inline virtual ~PFXShader();
	
	virtual void Apply(PFXBuffer *in, PFXBuffer *out, PFXArg *arg, int args, float time)=0;
};

struct PFXPixelShader : public PFXShader
{
	unsigned int	prog;
	char			*code;

	PFXPixelShader(char *progname);
	~PFXPixelShader();

	virtual void Apply(PFXBuffer *in, PFXBuffer *out, PFXArg *arg, int args, float time);
};

struct PFXReference
{
	struct PFXEffect	*fx;
	PFXArg			*arg;
	int			args;
	float			timescale;
	
	PFXReference();
	~PFXReference();
	
	void AddArg(char *name, char *value);
};

struct PFXEffect
{
	char			*name;
	PFXShader		*shader;
	PFXArg			*arg;
	int				args;
	PFXReference	**ref;
	int				refs;
	float			timescale;
	bool			posteffect;
	
	PFXEffect();
	~PFXEffect();
	
	void LoadFromPPFX(void *_node);
	
	void AddArg(char *name, char *value);
	PFXReference *AddRef(PFXEffect *fx);
};

class PFXSet
{
	PFXEffect	*root;
	PFXReference	**ref;
	int		refs;
	PFXBuffer	*tmpb;
	
	void AddRef(PFXReference *rfc);
	void AddEffect(PFXEffect *fx);
	void Flatten();
	
	public:
	PFXSet(PFXEffect *fx);
	~PFXSet();
	
	void Apply(PFXBuffer *in, PFXBuffer *out);
};

void InitPostProcessing();
void ShutdownPostProcessing();
PFXEffect *GetPFXEffect(char *name);
PFXShader *GetPFXShader(char *name);

#endif
