#ifndef LIB_SYS_H
#define LIB_SYS_H

#include <builddefs.h>
#include "keysyms.h"
#include "xpad.h"

bool SysCreateWindow(unsigned int w, unsigned int h, bool border, void *data = 0);
void SysCloseWindow();
void SysSetWinTitle(const char *title);
void SysShowWindow();
void SysSetForegroundWindow();
void *SysGetWindow();

// SysGetWindowWidth/Height functions return the "client" size, or -1 on error
int SysGetWindowWidth();
int SysGetWindowHeight();

// SysShowCursor shows or hides the cursor
void SysShowCursor(bool show);

// SysProcessEvents returns true in general, unless a quit event is received
bool SysProcessEvents(bool block = false);

enum {
	BN_LEFT,
	BN_MIDDLE,
	BN_RIGHT,
	BN_WHEELUP,
	BN_WHEELDOWN
};

typedef void (*SysDisplayHandler)();
typedef void (*SysKeyboardHandler)(int, bool);
typedef void (*SysMotionHandler)(int, int);
typedef void (*SysButtonHandler)(int, int, int, bool);
typedef void (*SysResizeHandler)(int, int);
typedef void (*SysXPadHandler)(const XPadEvent &event);

struct Handlers
{
	SysDisplayHandler display;
	SysKeyboardHandler keyb;
	SysMotionHandler motion;
	SysButtonHandler button;
	SysResizeHandler resize;
	SysXPadHandler xpad;
};

void SysSetDisplayHandler(SysDisplayHandler handler);
void SysSetKeyboardHandler(SysKeyboardHandler handler);
void SysSetMotionHandler(SysMotionHandler handler);
void SysSetButtonHandler(SysButtonHandler handler);
void SysSetResizeHandler(SysResizeHandler handler);
void SysSetXPadHandler(SysXPadHandler handler);

SysDisplayHandler SysGetDisplayHandler();
SysKeyboardHandler SysGetKeyboardHandler();
SysMotionHandler SysGetMotionHandler();
SysButtonHandler SysGetButtonHandler();
SysResizeHandler SysGetResizeHandler();
SysXPadHandler SysGetXPadHandler();

void SysPushHandlers();
bool SysPopHandlers();

#ifdef BUILD_WIN32
typedef int (__stdcall *WIN32PROC)(int hwnd, unsigned int uMsg, int wParam, int lParam);
void SysSetWin32Hook(WIN32PROC hook);
#endif

#endif //LIB_SYS_H
