#include <builddefs.h>
#ifdef WSYS_X11
#include <stack>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <X11/Xlib.h>
#include <GL/glx.h>

#include <errlog.h>
#include "libsys.h"

// XXX: this must be the same as in libgc/gc_ogl.cpp
struct InitData {
	Display *dpy;
	GLXContext ctx;
	XVisualInfo *vis;
	Window win;
	bool fullscreen;
};

static Display *dpy;
static Window win;
static GLXContext ctx;
int width, height;
extern std::stack<Handlers> sys_handlers;

static bool HandleEvent(XEvent *event);

bool SysCreateWindow(unsigned int w, unsigned int h, bool border, void *data)
{
	width = w;
	height = h;
	dpy = ((InitData*)data)->dpy;
	ctx = ((InitData*)data)->ctx;
	
	int scr = DefaultScreen(dpy);
	Window root = RootWindow(dpy, scr);
	
	XVisualInfo *vis = ((InitData*)data)->vis;

	XSetWindowAttributes xattr;
	xattr.background_pixel = xattr.border_pixel = BlackPixel(dpy, scr);
	xattr.colormap = XCreateColormap(dpy, root, vis->visual, AllocNone);	

	unsigned int attr_valid_mask = CWColormap | CWBackPixel | CWBorderPixel;

	if(((InitData*)data)->fullscreen) {
		/* TODO: implement */
		attr_valid_mask |= CWOverrideRedirect;
	}

	if(!(win = XCreateWindow(dpy, root, 0, 0, w, h, 0, vis->depth, InputOutput, vis->visual, attr_valid_mask, &xattr))) {
		Error("Failed to create X window\n");
		return false;
	}

	unsigned int events = ExposureMask | StructureNotifyMask | KeyPressMask
			| ButtonReleaseMask | ButtonPressMask | PointerMotionMask | ButtonMotionMask;
	XSelectInput(dpy, win, events);

	Atom wm_del_atom = XInternAtom(dpy, "WM_DELETE_WINDOW", True);
	XSetWMProtocols(dpy, win, &wm_del_atom, 1);

	SysSetWinTitle("Theseis / X11");

	XClassHint class_hint;
	class_hint.res_name = "theseis";
	class_hint.res_class = "theseis_gfx";
	XSetClassHint(dpy, win, &class_hint);

	SysShowWindow();

	((InitData*)data)->win = win;
	return true;
}

void SysCloseWindow()
{
	if(win) {
		XDestroyWindow(dpy, win);
		win = 0;
	}
}

void SysSetWinTitle(const char *title)
{
	XTextProperty tp_wname;
	XStringListToTextProperty((char**)&title, 1, &tp_wname);
	XSetWMName(dpy, win, &tp_wname);
	XFree(tp_wname.value);
}

void SysShowWindow()
{
	XMapWindow(dpy, win);
	XFlush(dpy);
}

void SysSetForegroundWindow()
{
	// TODO
}

void *SysGetWindow()
{
	return (void*)&win;
}

int SysGetWindowWidth()
{
	return width;
}

int SysGetWindowHeight()
{
	return height;
}

void SysShowCursor(bool show)
{
	// TODO
}

bool SysProcessEvents(bool block)
{
	XEvent event;
	if(block) {
		do {
			XNextEvent(dpy, &event);
			if(!HandleEvent(&event)) {
				return false;
			}
		} while(XPending(dpy));
	} else {
		while(XPending(dpy)) {
			XNextEvent(dpy, &event);
			if(!HandleEvent(&event)) {
				return false;
			}
		}
	}
	return true;
}

static bool HandleEvent(XEvent *event)
{
	static int win_mapped;
	KeySym sym;
	char *atom_name;

	switch(event->type) {
	case MapNotify:
		win_mapped = 1;
		break;

	case UnmapNotify:
		win_mapped = 0;
		break;

	case Expose:
		if(win_mapped && event->xexpose.count == 0) {
			if(sys_handlers.top().display) {
				sys_handlers.top().display();
			}
		}
		break;

	case ClientMessage:
		atom_name = XGetAtomName(dpy, event->xclient.message_type);
		if(!strcmp(atom_name, "WM_PROTOCOLS")) {
			XFree(atom_name);
			return false;
		}
		XFree(atom_name);
		break;

	case KeyPress:
	case KeyRelease:
		sym = XLookupKeysym((XKeyEvent*)&event->xkey, 0);
		if(sys_handlers.top().keyb) {
			sys_handlers.top().keyb(sym & 0xff, event->type == KeyPress);
        }
		break;

	case MotionNotify:
		if(sys_handlers.top().motion) {
			sys_handlers.top().motion(event->xbutton.x, event->xbutton.y);
		}
		break;

	case ButtonPress:
	case ButtonRelease:
		if(sys_handlers.top().button) {
			sys_handlers.top().button(event->xbutton.x, event->xbutton.y, event->xbutton.button, event->type == ButtonPress);
		}
		break;

	default:
		break;
	}

	return true;
}

#else
int silence_stupid_compiler_warning_about_empty_files2 = 0;
#endif
