#ifndef UTK_FILEDIALOG_H_
#define UTK_FILEDIALOG_H_

#include <utk_combobox.h>
#include <utk_dialog.h>

namespace utk {

class FileDialogFilter
{
public:
	virtual ~FileDialogFilter();

	virtual const char *get_name() const=0;
	virtual bool accept(const char *name) const=0;
};

class FileDialogExtFilter : public FileDialogFilter
{
protected:
	std::list<char*> extlist;
	char *name;

public:
	FileDialogExtFilter(const char *name, const char *exts);
	virtual ~FileDialogExtFilter();

	virtual void add_ext(const char *ext);

	virtual const char *get_name() const;
	virtual bool accept(const char *name) const;
};

class FileDialogRegExpFilter : public FileDialogFilter
{
protected:
	void *rxprog;
	char *regexp;
	char *name;
public:
	FileDialogRegExpFilter(const char *name, const char *regexp, bool wildcard);
	virtual ~FileDialogRegExpFilter();

	virtual const char *get_name() const;
	virtual bool accept(const char *name) const;
};

class FileDialog : public Dialog {
	std::list<FileDialogFilter*> filters;
	FileDialogFilter *filter;
	FileDialogRegExpFilter *rxfilter;
	bool show_hidden;

	char *path;
	Entry *pathentry;
	Entry *filename;
	ListBox *listb;
	ComboBox *filterbox;
	CheckBox *use_regexp;
	char *full_filename;
	int click_time;
	IVec2 click_pos;

	bool accept_file(const char *name) const;
	bool fill_filelist();
	void set_filter();
	void custom_regexp();

	static void select_button_handler(Event *event, void *data);
	static void extlist_modify_handler(Event *event, void *data);
	static void filename_notify_handler(Event *event, void *data);
	static void listb_modify_handler(Event *event, void *data);
	static void listb_click_handler(Event *event, void *data);

public:
	FileDialog(const char *title="Select file");
	virtual ~FileDialog();

	virtual void set_path(const char *path);
	virtual const char *get_path() const;
	virtual const char *get_filename();

	virtual void add_filter(FileDialogFilter *filter);

	virtual void refresh();

	friend FileDialog *file_dialog(unsigned int type, const char *fname, FileDialogFilter *filter, const char *start_dir, Callback func, void *cdata);
};

FileDialog *file_dialog(unsigned int type, const char *fname, FileDialogFilter *filter, const char *start_dir, Callback func = 0, void *cdata = 0);

}

#endif
