// tar.cpp
#include "tar.h"
#include <stdio.h>
#include <stdlib.h>

#define RECORDSIZE	512
#define NAMSIZ		100
#define TUNMLEN		32
#define TGNMLEN		32

union tar_record
{
	char	charptr[RECORDSIZE];
	struct header
	{
		char	name[NAMSIZ];
		char	mode[8];
		char	uid[8];
		char	gid[8];
		char	size[12];
		char	mtime[12];
		char	chksum[8];
		char	linkflag;
		char	linkname[NAMSIZ];
		char	magic[8];
		char	uname[TUNMLEN];
		char	gname[TGNMLEN];
		char	devmajor[8];
		char	devminor[8];
	}header;
};

std::vector<tar_file> get_tar_files(const char* tar_filename)
{
	std::vector<tar_file> ret;
	tar_record rec;
	tar_file tf;
	int tar_file_size;

	FILE *fp = fopen(tar_filename, "rb");
	if (!fp)
	{
		return ret;
	}
	tf.tar_fname = tar_filename;

	// get file size
	fseek(fp, 0, SEEK_END);
	tar_file_size = ftell(fp);
	fseek(fp, 0, SEEK_SET);

	while (!feof(fp))
	{
		// read a record
		if (1 != fread(&rec, sizeof(rec), 1, fp))
		{
			fclose(fp);
			return ret;
		}
		tf.filename = rec.header.name;
		tf.offset = ftell(fp);
		tf.size = strtol(rec.header.size, 0, 8);
		if (tf.filename == "././@LongLink")
		{
			// read filename block
			if (1 != fread(&rec, sizeof(rec), 1, fp))
			{
				fclose(fp);
				return ret;
			}
			tf.filename = rec.charptr;
			tf.offset += 1024;

			// read original tar record
			if (1 != fread(&rec, sizeof(rec), 1, fp))
			{
				fclose(fp);
				return ret;
			}
			tf.size = strtol(rec.header.size, 0, 8);
		}
		if (tf.filename == "")
		{
			continue;
		}

		// seek after file data records
		unsigned int data_recs = (tf.size + 511) / 512;
		fseek(fp, data_recs * 512, SEEK_CUR);
		if (ftell(fp) > tar_file_size)
		{
			fclose(fp);
			return ret;
		}

		// get file type
		tf.type = TFT_FILE;
		if (tf.filename[tf.filename.size() - 1] == '/')
		{
			tf.type = TFT_DIR;
		}

		// add file to return list
		ret.push_back(tf);
	}

	fclose(fp);
	return ret;
}
