// cullable.h

#ifndef _CULLABLE_H_
#define _CULLABLE_H_

#include <vector>
#include <map>
#include <math3d.h>

// cullable types
enum CullableType
{
	CULLABLE_PVS,
	CULLABLE_MESH,
	CULLABLE_PROXY,
	CULLABLE_LIGHT
};

class PVS;

class Cullable
{
protected:
	std::map<PVS*, bool> owners;
public:

	// used to determine visibility in the current frame
	unsigned int frame;

	Cullable();
	virtual ~Cullable();
	
	// pure virtuals
	virtual Box GetSpacetimeBoundingBox() const = 0;
	virtual unsigned int GetCullableType() const = 0;

	// overload these if you want per triangle checking
	// for outdoor areas
	virtual unsigned int GetCullableTriangleCount() const;
	virtual Triangle GetCullableTriangle(unsigned int i) const;

	bool OwnedBy(PVS *set);

	void SetOwner(PVS *owner, bool enable=true);
};

// an easier to use cullable class
class CachedCullable : public Cullable
{
protected:
	Box cullable_spacetime_box;
	CullableType cullable_type;

public:
	
	CachedCullable();

	Box GetSpacetimeBoundingBox() const;
	unsigned int GetCullableType() const;
};

typedef std::vector<Cullable*> CullableList;

// helper
CullableList ConcatCullLists(const CullableList &a, const CullableList &b);

#endif // ndef _CULLABLE_H_
