#include "portals.h"
#include <immed.h>
#include <libgc.h>


enum PortalRes
{
	PORTAL_OK,
	PORTAL_FAILED,
	NO_PORTAL
};


std::vector<Vector3> debug_points;

int CalculatePortal (ConvexArea *src_area, ConvexArea *dst_area, Quad **src_quad, Quad **dst_quad)
{
	std::vector<Vector3> portal_points;

	for (unsigned int i = 0; i < src_area->GetTriangleCount(); i++)
	{
		if (dst_area->TriangleIntersects (src_area->GetTriangle (i)))
		{
			Line line;

			line.SetVertex (0, src_area->GetTriangle(i)->GetVertex(0));
			line.SetVertex (1, src_area->GetTriangle(i)->GetVertex(1));
			if (dst_area->LineIntersects (line))
			{
				Vector3 cut, inv;
				dst_area->IntersectLine (line, &cut, &inv);
				portal_points.push_back (cut);
				debug_points.push_back (cut);
			}

			line.SetVertex (0, src_area->GetTriangle(i)->GetVertex(1));
			line.SetVertex (1, src_area->GetTriangle(i)->GetVertex(2));
			if (dst_area->LineIntersects (line))
			{
				Vector3 cut, inv;
				dst_area->IntersectLine (line, &cut, &inv);
				portal_points.push_back (cut);
				debug_points.push_back (cut);
			}

			line.SetVertex (0, src_area->GetTriangle(i)->GetVertex(2));
			line.SetVertex (1, src_area->GetTriangle(i)->GetVertex(0));
			if (dst_area->LineIntersects (line))
			{
				Vector3 cut, inv;
				dst_area->IntersectLine (line, &cut, &inv);
				portal_points.push_back (cut);
				debug_points.push_back (cut);
			}
		}
	}

	for (unsigned int i = 0; i < dst_area->GetTriangleCount(); i++)
	{
		if (src_area->TriangleIntersects (dst_area->GetTriangle (i)))
		{
			Line line;

			line.SetVertex (0, dst_area->GetTriangle(i)->GetVertex(0));
			line.SetVertex (1, dst_area->GetTriangle(i)->GetVertex(1));
			if (src_area->LineIntersects (line))
			{
				Vector3 cut, inv;
				src_area->IntersectLine (line, &cut, &inv);
				portal_points.push_back (cut);
				debug_points.push_back (cut);
			}
			line.SetVertex (0, dst_area->GetTriangle(i)->GetVertex(1));
			line.SetVertex (1, dst_area->GetTriangle(i)->GetVertex(2));
			if (src_area->LineIntersects (line))
			{
				Vector3 cut, inv;
				src_area->IntersectLine (line, &cut, &inv);
				portal_points.push_back (cut);
				debug_points.push_back (cut);
			}
			line.SetVertex (0, dst_area->GetTriangle(i)->GetVertex(2));
			line.SetVertex (1, dst_area->GetTriangle(i)->GetVertex(0));
			if (src_area->LineIntersects (line))
			{
				Vector3 cut, inv;
				src_area->IntersectLine (line, &cut, &inv);
				portal_points.push_back (cut);
				debug_points.push_back (cut);
			}
		}
	}

	if (portal_points.size () < 3)
	{
		return PORTAL_FAILED;
	}


	bool complete = false;
	unsigned int plane_idx [3];

	for (unsigned int i = 0; i < portal_points.size(); i ++)
	{
		if (complete == true)
		{
			break;
		}
		plane_idx [0] = i;
		Line line;
		line.SetVertex (0, portal_points[i]);

		for (unsigned int j = i+1; j < portal_points.size(); i++)
		{
			if (complete == true)
			{
				break;
			}
			plane_idx[1] = j;
			line.SetVertex (1, portal_points[j]);

			for (unsigned int k = j + 1; k < portal_points.size(); k++)
			{
				plane_idx[2] = k;

				if (Distance(portal_points[k], line.GetRay()) >0.01f)
				{
					complete = true;
					break;
				}
			}
		}
	}


	Triangle triangle;
	triangle.SetVertex (0, portal_points[plane_idx[0]]);
	triangle.SetVertex (1, portal_points[plane_idx[1]]);
	triangle.SetVertex (2, portal_points[plane_idx[2]]);

	Plane plane = triangle.GetPlane ();

	for (unsigned int i = 0; i < portal_points.size(); i++)
	{
		float dist = DistanceFromPlane (portal_points[i], plane) ;
		if (dist > 0.001f)
		{
			return PORTAL_FAILED;
		}
	}

	// calculate the actual portal inside 

	Vector3 keep_points[4];

	Vector3 actual_points[4];
	float current_angle = 0.0f;

	bool have_last_dim = false;
	float last_dim = 0.0f;

	for (int i = 0; i < 1; i++)
	{
		current_angle = (float) i;
		Vector3 look_vector;
		Vector3 right_vector;
		look_vector = triangle.GetVertex (0) - triangle.GetVertex (1);
		look_vector.Normalize ();

		Matrix4x4 rot_matrix;
		rot_matrix.SetRotation (plane.GetNormal(), current_angle);
		look_vector.Transform (rot_matrix);
		right_vector = CrossProduct (look_vector, plane.GetNormal());
		look_vector.Normalize();
		right_vector.Normalize();

		/*

		1 (0)  0
		(1)    (3)
		2 (2)  3

		*/

		const float mult = 1000.0f;

		actual_points[0] = triangle.GetVertex (0) + look_vector * mult;
		actual_points[0] += right_vector * mult;

		actual_points[1] =  triangle.GetVertex (0) + look_vector * mult;
		actual_points[1] -= right_vector * mult;

		actual_points[2] = triangle.GetVertex (0) -  look_vector * mult;
		actual_points[2] -= right_vector * mult;

		actual_points[3] = triangle.GetVertex (0) -  look_vector * mult;
		actual_points[3] += right_vector * mult;

		float min_distance[4] = {10000.0f, 10000.0f, 10000.0f, 10000.0f};
		Line check_lines [4];

		check_lines[0].SetVertex (0, actual_points[0]);
		check_lines[0].SetVertex (1, actual_points[1]);

		check_lines[1].SetVertex (0, actual_points[1]);
		check_lines[1].SetVertex (1, actual_points[2]);

		check_lines[2].SetVertex (0, actual_points[2]);
		check_lines[2].SetVertex (1, actual_points[3]);

		check_lines[3].SetVertex (0, actual_points[3]);
		check_lines[3].SetVertex (1, actual_points[0]);

		for (unsigned int j = 0; j < portal_points.size(); j++)
		{
			if (min_distance [0] > Distance(portal_points[j], check_lines[0].GetRay()))
			{
				min_distance[0] = Distance(portal_points[j], check_lines[0].GetRay());
			}
			if (min_distance [1] > Distance(portal_points[j], check_lines[1].GetRay()))
			{
				min_distance[1] = Distance(portal_points[j], check_lines[1].GetRay());
			}
			if (min_distance [2] > Distance(portal_points[j], check_lines[2].GetRay()))
			{
				min_distance[2] = Distance(portal_points[j], check_lines[2].GetRay());
			}
			if (min_distance [3] > Distance(portal_points[j], check_lines[3].GetRay()))
			{
				min_distance[3] = Distance(portal_points[j], check_lines[3].GetRay());
			}
		}

		check_lines[0].SetVertex (0, check_lines[0].GetVertex (0) - look_vector*min_distance[0]);
		check_lines[0].SetVertex (1, check_lines[0].GetVertex (1) - look_vector*min_distance[0]);

		check_lines[2].SetVertex (0, check_lines[2].GetVertex (0) + look_vector*min_distance[2]);
		check_lines[2].SetVertex (1, check_lines[2].GetVertex (1) + look_vector*min_distance[2]);

		check_lines[0].SetVertex (0, check_lines[0].GetVertex (0) - right_vector*min_distance [3]); 
		check_lines[0].SetVertex (1, check_lines[0].GetVertex (1) + right_vector*min_distance [1]); 

		check_lines[2].SetVertex (0, check_lines[2].GetVertex (0) + right_vector*min_distance [1]); 
		check_lines[2].SetVertex (1, check_lines[2].GetVertex (1) - right_vector*min_distance [3]); 


		if (have_last_dim == false)
		{
			keep_points [0] = check_lines[0].GetVertex(0);
			keep_points [1] = check_lines[0].GetVertex(1);
			keep_points [2] = check_lines[2].GetVertex(0);
			keep_points [3] = check_lines[2].GetVertex(1);
			last_dim = (keep_points[0] - keep_points[1]).Length() * (keep_points[1] - keep_points[2]).Length();
			have_last_dim = true;
		}
		else
		{
			float dim = (check_lines[0].GetVertex(0) - check_lines[0].GetVertex(1)).Length() * 
				(check_lines[0].GetVertex(1) - check_lines[2].GetVertex(1)).Length();

			if (dim < last_dim)
			{
				last_dim = dim;
				keep_points [0] = check_lines[0].GetVertex(0);
				keep_points [1] = check_lines[0].GetVertex(1);
				keep_points [2] = check_lines[2].GetVertex(0);
				keep_points [3] = check_lines[2].GetVertex(1);
			}
		}
	}

	Quad *quad1 = new Quad (keep_points[0], keep_points[1], keep_points[2], keep_points[3]);
	Quad *quad2 = new Quad (keep_points[3], keep_points[2], keep_points[1], keep_points[0]);

	/*
	// find a point from source quad that is not inside dest quad
	Vector3  vertex_not_inside;
	for (unsigned int i = 0; i < src_area->GetTriangleCount (); i++)
	{
		Triangle *tri = src_area->GetTriangle (i);
		if ( ! dst_area->IsPointInside (tri->GetVertex(0)))
		{
			vertex_not_inside = tri->GetVertex (0);
			break;
		}
		if ( ! dst_area->IsPointInside (tri->GetVertex(1)))
		{
			vertex_not_inside = tri->GetVertex (1);
			break;
		}
		if ( ! dst_area->IsPointInside (tri->GetVertex(2)))
		{
			vertex_not_inside = tri->GetVertex (2);
			break;
		}
	}

	Plane p_plane = quad1->GetPlane();
	if (UnderPlane (vertex_not_inside, p_plane))
	{
		*src_quad = quad2;
		*dst_quad = quad1;
	}
	else
	{
		*src_quad = quad1;
		*dst_quad = quad2;
	}*/

	if (!UnderPlane(src_area->GetCenter(), quad1->GetPlane()))
	{
		*src_quad = quad2;
		*dst_quad = quad1;
	}
	else
	{
		*src_quad = quad1;
		*dst_quad = quad2;
	}


	return PORTAL_OK;
}





//----------------------- CONVEX AREA implementation -------------------------------------------





ConvexArea::ConvexArea ()
{
	memset (name, 0, 128);
	strcpy (name, "noname");
	is_link = false;
}

ConvexArea::~ConvexArea ()
{
}

Box ConvexArea::GetBoundingBox() const
{
	bool first = true;
	Vector3 minv, maxv;
	for (unsigned int i=0; i<triangle_list.size(); i++)
	{
		for (unsigned int j=0; j<3; j++)
		{
			Vector3 v = triangle_list[i]->GetVertex(j);
			if (first)
			{
				minv = maxv = v;
				first = false;
			}

			if (v.x < minv.x) minv.x = v.x;
			if (v.y < minv.y) minv.y = v.y;
			if (v.z < minv.z) minv.z = v.z;

			if (v.x > maxv.x) maxv.x = v.x;
			if (v.y > maxv.y) maxv.y = v.y;
			if (v.z > maxv.z) maxv.z = v.z;
		}
	}

	return Box(minv, maxv);
}

void ConvexArea::SetName (const char *node_name)
{
	memset (name, 0, 128);
	strcpy (name, node_name);
}


void ConvexArea::AddTriangle (const Vector3 &v1, const Vector3 &v2, const Vector3 &v3)
{
    Triangle *triangle = new Triangle (v1, v2, v3);
    triangle_list.push_back (triangle);
}

unsigned int ConvexArea::GetTriangleCount ()
{
    return (unsigned int) triangle_list.size();
}

Triangle *ConvexArea::GetTriangle (unsigned int idx)
{
    if (idx >= GetTriangleCount ())
    {
        return NULL;
    }
    return triangle_list[idx];
}

          
bool ConvexArea::IsConvex ()
{
    for (unsigned int i = 0; i < triangle_list.size(); i++)
    {
        Triangle *current_triangle = triangle_list[i];
        Plane current_tri_plane = current_triangle->GetPlane();

        for (unsigned int j = i+1; j < triangle_list.size(); j++)
        {
            Triangle *check_tri = triangle_list[j];
            if (current_tri_plane.ClassifyPoint(check_tri->GetVertex (0)) == CP_BACK)
            {
                return false;
            }
            if (current_tri_plane.ClassifyPoint(check_tri->GetVertex (1)) == CP_BACK)
            {
                return false;
            }
            if (current_tri_plane.ClassifyPoint(check_tri->GetVertex (2)) == CP_BACK)
            {
                return false;
            }
        }
    }
    return true;
}

bool ConvexArea::TriangleIntersects (Triangle *triangle)
{
    for (unsigned int i = 0; i < triangle_list.size(); i++)
    {
        Triangle *check = triangle_list[i];
        if (Intersect(*check, *triangle))
        {
            return true;
        }
    }
    return false;
}


bool ConvexArea::ConvexIntersects (ConvexArea *area)
{
    for (unsigned int i = 0; i < area->GetTriangleCount(); i++)
    {
        if (TriangleIntersects (area->GetTriangle (i)))
        {
            return true;
        }
    }
    return false;
}

bool ConvexArea::Encloses(ConvexArea *area)
{
    for (unsigned int i = 0; i < triangle_list.size(); i++)
    {
        Plane current_plane = triangle_list[i]->GetPlane();

        for (unsigned int j = 0; j < area->GetTriangleCount(); j++)
        {
            if (UnderPlane (*area->GetTriangle (j), current_plane))
            {
                return false;
            }
        }
    }
    return true;
}

bool ConvexArea::Encloses(Triangle *tri)
{
    for (unsigned int i = 0; i < triangle_list.size(); i++)
    {
        Plane current_plane = triangle_list[i]->GetPlane();

        if (UnderPlane (*tri, current_plane))
        {
            return false;
        }

    }
    return true;
}

bool ConvexArea::IsEnclosed (ConvexArea *area)
{
    return area->Encloses (this);
}


bool ConvexArea::IsPointInside (const Vector3 &vec)
{

    for (unsigned int i = 0; i < triangle_list.size(); i++)
    {
        if (UnderPlane (vec, triangle_list[i]->GetPlane()))
        {
            return false;
        }
    }
    return true;
}

Vector3 ConvexArea::GetCenter()
{
	if (!triangle_list.size()) return Vector3(0, 0, 0);
	Vector3 netv = Vector3(0, 0, 0);
	for (unsigned int i=0; i<triangle_list.size(); i++)
	{
		for (unsigned int j=0; j<3; j++)
		{
			netv += triangle_list[i]->GetVertex(j);
		}
	}

	netv /= (3 * triangle_list.size());
	return netv;
}

bool ConvexArea::LineIntersects (Line line)
{
	if (IsPointInside (line.GetVertex(0)) != IsPointInside (line.GetVertex(1)))
	{
		return true;
	}
	return false;
}

void ConvexArea::IntersectLine (Line line, Vector3 *cut, Vector3 *inp)
{
	for (unsigned int i = 0; i < triangle_list.size(); i++)
	{
		if (Intersect(line,*triangle_list[i]))
		{
			*cut = PointOfIntersection (line.GetRay(), triangle_list[i]->GetPlane());

			if (UnderPlane(line.GetVertex(0), triangle_list[i]->GetPlane()))
			{
				*inp = line.GetVertex(1);
			}
			else
			{
				*inp = line.GetVertex (0);
			}
		}
	}
}


void ConvexArea::Render ()
{
	SetFillMode (FILL_WIREFRAME);
	ImSetApplyXForm(false);
	ImBegin (IM_TRIANGLES);
	for (unsigned int i = 0; i < triangle_list.size(); i++)
	{
		ImColor (0.5f, 0.5f, 0.5f, 0.3f);
		ImVertex (triangle_list[i]->GetVertex(0));
		//ImVertex (triangle_list[i]->GetVertex(1));
		ImVertex (triangle_list[i]->GetVertex(1));
		ImVertex (triangle_list[i]->GetVertex(2));
		//ImVertex (triangle_list[i]->GetVertex(2));
		//ImVertex (triangle_list[i]->GetVertex(0));
	}
	ImEnd ();
	ImSetApplyXForm(true);
	SetFillMode (FILL_SOLID);
	RenderNormals ();
}

void ConvexArea::RenderNormals ()
{
	ImSetApplyXForm(false);
	ImBegin (IM_LINES);
	for (unsigned int i = 0; i < triangle_list.size(); i++)
	{
		Vector3 v, vn, n;

		n = triangle_list[i]->GetNormal ();
		n*=5.0f;

		v = triangle_list[i]->GetVertex(0);
		vn = v + n;
		ImVertex (v);
		ImVertex (vn);

		v = triangle_list[i]->GetVertex(1);
		vn = v + n;
		ImVertex (v);
		ImVertex (vn);

		v = triangle_list[i]->GetVertex(2);
		vn = v + n;
		ImVertex (v);
		ImVertex (vn);
	}
	ImEnd ();
	ImSetApplyXForm(true);
}




//----------------------- PORTAL generator implementation -------------------------------------------



PortalBuilder::PortalBuilder ()
{
}


PortalBuilder::~PortalBuilder ()
{
}

void PortalBuilder::AddConvexArea (ConvexArea *area)
{
	area_list.push_back (area);
}

unsigned int PortalBuilder::GetAreasCount ()
{
	return (unsigned int) area_list.size();
}

void PortalBuilder::Build ()
{
	for (unsigned int i = 0; i < area_list.size(); i++)
	{
		ConvexArea *current = area_list[i];

		for (unsigned int j = i+1; j < area_list.size(); j ++)
		{
			ConvexArea *check = area_list[j];

			//TODO: check if portal "check" is completely inside portal "current"

			Quad *src_quad = NULL;
			Quad *dst_quad = NULL;
			if (CalculatePortal (current, check, &src_quad, &dst_quad) == PORTAL_OK)
			{
				PortalConnection conn;
				conn.src_quad = src_quad;
				conn.dst_quad = dst_quad;
				conn.src_convex = current;
				conn.dst_convex = check;

				printf("src :%p, dst: %p\n", (void*)current, (void*)check);

				connection_list.push_back(conn);
			}
		}
	}

}

void PortalBuilder::Render ()
{
	for (unsigned int i = 0; i < area_list.size(); i++)
	{
		area_list[i]->Render ();
	}

	ImSetApplyXForm(false);
	ImBegin(IM_QUADS);
	for (unsigned int i=0; i<connection_list.size(); i++)
	{
		Quad s = *connection_list[i].src_quad;
		ImColor(0.4, 0, 0);
		for (unsigned int j=0; j<4; j++)
		{
			ImVertex(s.GetVertex(j));
		}

		Quad d = *connection_list[i].dst_quad;
		ImColor(0, 0, 0.4);
		for (unsigned int j=0; j<4; j++)
		{
			ImVertex(d.GetVertex(j));
		}
	}
	ImEnd();
	ImSetApplyXForm(true);
}

void PortalBuilder::RenderPoints ()
{
	SetBlending (false);
	SetBlendFunc (BLEND_SRC_ALPHA, BLEND_ONE_MINUS_SRC_ALPHA);
	ImBegin (IM_QUADS);
	ImColor (0.0f, 1.0f, 0.0f, 0.5f);
	for (unsigned int i = 0; i < connection_list.size(); i++)
	{
		PortalConnection conn = connection_list[i];

		ImVertex (conn.dst_quad->GetVertex(0));
		ImVertex (conn.dst_quad->GetVertex(1));
		ImVertex (conn.dst_quad->GetVertex(2));
		ImVertex (conn.dst_quad->GetVertex(3));
		
		ImVertex (conn.src_quad->GetVertex(0));
		ImVertex (conn.src_quad->GetVertex(1));
		ImVertex (conn.src_quad->GetVertex(2));
		ImVertex (conn.src_quad->GetVertex(3));
	}
	ImEnd ();
	SetBlending (false);
	///*ImBegin (IM_QUADS);
	//for (unsigned int i = 0; i < quad_list.size();i++)
	//{
	//	ImVertex (quad_list[i]->GetVertex(0));
	//	ImVertex (quad_list[i]->GetVertex(1));
	//	ImVertex (quad_list[i]->GetVertex(2));
	//	ImVertex (quad_list[i]->GetVertex(3));
	//}
	//ImEnd ();*/

	//ImBegin (IM_QUADS);
	//ImColor (0.0f, 1.0f, 0.0f, 0.5f);
	//for (int i = 0; i < actual_portal.size(); i++)
	//{
	//	ImVertex (actual_portal[i]);
	//}
	//ImEnd ();

	//ImBegin (IM_TRIANGLES);
	//ImColor (1.0f, 0.0f, 0.0f);
	//ImVertex (fuck_tri.GetVertex (0));
	//ImVertex (fuck_tri.GetVertex (1));
	//ImVertex (fuck_tri.GetVertex (2));
	//ImEnd ();

	//SetBlending (true);
	//SetBlendFunc (BLEND_ONE, BLEND_ONE);
	//ImBegin (IM_QUADS);
	//ImColor (1.0f, 1.0f, 0.0f);
	//ImVertex (debug_start_quad[0]);
	//ImVertex (debug_start_quad[1]);
	//ImVertex (debug_start_quad[2]);
	//ImVertex (debug_start_quad[3]);
	//ImEnd ();
	//SetBlending (false);

	ImBegin (IM_POINTS);
	for (unsigned int i = 0; i < debug_points.size(); i++)
	{
		ImColor (1.0f, 0.0f, 1.0f);
		ImVertex (debug_points[i]);
	}
	ImEnd ();

	//ImBegin (IM_POINTS);
	//ImColor (1.0f, 0.0f, 1.0f);
	//ImVertex (arxidia);
	//ImEnd ();

}

unsigned int PortalBuilder::GetConnectionCount() const
{
	return (unsigned int) connection_list.size();
}

PortalConnection PortalBuilder::GetConnection(unsigned int idx) const
{
	return connection_list[idx];
}

