#ifndef PORTALS_H
#define PORTALS_H

#include "math3d.h"
#include "geom.h"



class ConvexArea 
{
protected:
    std::vector<Triangle*> triangle_list;
	char name[128];
	bool is_link;
public:
    ConvexArea ();
    ~ConvexArea ();

	Box GetBoundingBox() const;

	bool IsLink() const {return is_link;}
	void SetLink(bool l) {is_link = l;}

	void SetName (const char *node_name);
	const char *GetName() {return name;}

    unsigned int GetTriangleCount ();
    Triangle *GetTriangle (unsigned int idx);

    void AddTriangle (const Vector3 &v1, const Vector3 &v2, const Vector3 &v3);
    bool IsConvex ();

    bool TriangleIntersects (Triangle *triangle);
    bool ConvexIntersects (ConvexArea *area);
    bool Encloses (ConvexArea *area);
	bool Encloses(Triangle *tri);
    bool IsEnclosed (ConvexArea *area);
	bool LineIntersects (Line line);
	void IntersectLine (Line line, Vector3 *cut, Vector3 *inp);

    bool IsPointInside (const Vector3 &vec);

	Vector3 GetCenter();

	void Render ();
	void RenderNormals ();
};

struct PortalConnection
{
	Quad *src_quad;
	Quad *dst_quad;
	ConvexArea *src_convex;
	ConvexArea *dst_convex;
};

class PortalBuilder
{
protected:
	std::vector<ConvexArea *> area_list;
	std::vector<PortalConnection> connection_list;
public:
    PortalBuilder();
    virtual ~PortalBuilder();

    void AddConvexArea (ConvexArea *area);
    unsigned int GetAreasCount ();

	unsigned int GetConnectionCount() const;
	PortalConnection GetConnection(unsigned int idx) const;

	void Build ();
	virtual void Render ();
	void RenderPoints ();
};

void DrawCutPoints ();
void RenderPoints ();




#endif //PORTALS_H

