#ifndef _MATH3D_H_
#define _MATH3D_H_

#include <math.h>
#include <stdlib.h>
#include "vector.h"
#include "matrix.h"
#include "quaternion.h"
#include "sphvec.h"
#include "rndval.h"
#include "geom.h"
#include "prs.h"
#include "approx.h"
#include "stat.h"

#define SQ(x)	((x) * (x))
#ifndef DEG_TO_RAD
#define DEG_TO_RAD(a) (Pi / 180.0f * (a))
#endif //DEG_TO_RAD

#define MIN(a, b)		((a) < (b) ? (a) : (b))
#define MAX(a, b)		((b) < (a) ? (a) : (b))
#define CLAMP(x, a, b)	MIN(b, MAX(a, x))

extern const float SmallNumber, XSmallNumber;
extern const float Pi, HalfPi, QuarterPi, TwoPi, ThreeHalfPi;

inline bool IsPow2(int x);
inline int NextPow2(int x);
#if !defined(GLIBC) && !defined(__GLIBC__)
float log2(float x);
#endif
inline float frand(float range = 1.0f);

inline float Lerp(float a, float b, float t);
inline Vector3 Lerp(const Vector3 &a, const Vector3 &b, float t);
inline Quaternion Lerp(const Quaternion &a, const Quaternion &b, float t);

inline float Spline(float a, float b, float c, float d, float t);
inline float BSpline(float a, float b, float c, float d, float t);

#include "math3d.inl"

#endif	// _MATH3D_H_
