// x is a power of two if it only has one bit set
inline bool IsPow2(int x)
{
	int bit_num = sizeof x * CHAR_BIT;
	int bcount = 0;

	for(int i=0; i<bit_num; i++) {
		if(x & 1) bcount++;
		x >>= 1;
	}
	return bcount == 1;
}

/* next_pow2, returns the next power of 2 (...[256->256], [257->512]...)
 * NOTE: log2 is an ISO C99 function, GCC supports it, but msvc doesn't, so
 *       I also include a quick log2 hack for MSVC in math3d.cpp.
 */
inline int NextPow2(int x) {
	float lg2 = log2((float)x);
	return (int)pow(2.0f, (int)ceil(lg2));
}

inline float frand(float range) {
	return range * ((float)rand() / (float)RAND_MAX);
}

inline float Lerp(float a, float b, float t) {
	return a + (b - a) * t;
}

inline Vector3 Lerp(const Vector3 &a, const Vector3 &b, float t) {
	return a + (b - a) * t;
}

inline Quaternion Lerp(const Quaternion &a, const Quaternion &b, float t) {
	return Slerp(a, b, t);
}

inline float Spline(float a, float b, float c, float d, float t) {
	static const Matrix4x4 catmul_rom_mat(-1, 3, -3, 1, 2, -5, 4, 1, -1, 0, 1, 0, 0, 2, 0, 0);

	float t_sq = t * t;
	float t_cube = t_sq * t;
	Vector4 cpvec(a, b, c, d);
	Vector4 params(t_cube, t_sq, t, 1.0);

	return DotProduct(params, cpvec.Transformed(catmul_rom_mat) / 2.0);
}

inline float BSpline(float a, float b, float c, float d, float t) {
	static const Matrix4x4 bspline_mat(-1, 3, -3, 1, 3, -6, 3, 0, -3, 0, 3, 0, 1, 4, 1, 0);

	float t_sq = t * t;
	float t_cube = t_sq * t;
	Vector4 cpvec(a, b, c, d);
	Vector4 params(t_cube, t_sq, t, 1.0);

	return DotProduct(params, cpvec.Transformed(bspline_mat) / 6.0);
}
