// prs.cpp
#include "prs.h"
#include <stdio.h>

void PrintPRS(const PRS &prs)
{
	Vector3 p, s;
	Quaternion r;
	p = prs.GetPosition();
	r = prs.GetRotation();
	s = prs.GetScaling();

	printf("[PRS]:\n");
	printf("\tp:[%f, %f, %f]\n", p.x, p.y, p.z);
	printf("\tr:[%f, [%f, %f, %f]]\n", r.s, r.v.x, r.v.y, r.v.z);
	printf("\ts:[%f, %f, %f]\n", s.x, s.y, s.z);
}

// PRS class implementation
PRS::PRS()
{
	p = Vector3();
	r = Quaternion();
	s = Vector3(1, 1, 1);
}
PRS::PRS(const Vector3& p, const Quaternion& r, const Vector3& s)
{
	this->p = p;
	this->r = r;
	this->s = s;
}

PRS::PRS(const Matrix4x4 &composite)
{
	p = composite.ExtractTranslation();
	r = Quaternion(composite.ExtractRotation());
	s = composite.ExtractScaling();
}

PRS::~PRS()
{

}

PRS PRS::operator + (const PRS &prs) const
{
	// TODO: think about scaling!!!
	return PRS(this->p + prs.p, this->r * prs.r, this->s);
}

PRS PRS::operator - (const PRS &prs) const
{
	// TODO: think about scaling!!!
	return PRS(this->p - prs.p, this->r * prs.r.Inverse(), this->s);
}

Matrix4x4 PRS::GetTransform() const
{
	Matrix4x4 ret;

	ret.Translate(p);
	Matrix4x4 rot_mat = (Matrix4x4) r.GetRotationMatrix();
	ret *= rot_mat;
	ret.Scale(s);
	
	return ret;
}

// set / get
void PRS::SetPosition(const Vector3& p)
{
	this->p = p;
}
Vector3 PRS::GetPosition() const
{
	return p;
}
void PRS::SetRotation(const Quaternion& r)
{
	this->r = r;
}
Quaternion PRS::GetRotation() const
{
	return r;
}
void PRS::SetScaling(const Vector3& s)
{
	this->s = s;
}
Vector3 PRS::GetScaling() const
{
	return s;
}


// Interpolation
PRS Slerp(const PRS& a, const PRS& b, const float& t)
{
	PRS ret;
	ret.p = Lerp(a.p, b.p, t);
	ret.r = Slerp(a.r, b.r, t);
	ret.s = Lerp(a.s, b.s, t);
	return ret;
}
