// prs.h

#include "math3d.h"

#ifndef _PRS_HEADER_
#define _PRS_HEADER_

class PRS
{
private:
	Vector3 p;
	Quaternion r;
	Vector3 s;

public:
	PRS();
	PRS(const Vector3& p, const Quaternion& r, const Vector3& s);
	PRS(const Matrix4x4 &composite);
	~PRS();

	// Interpolation
	friend PRS Slerp(const PRS& a, const PRS& b, const float& t);

	PRS operator + (const PRS &prs) const;
	PRS operator - (const PRS &prs) const;

	Matrix4x4 GetTransform() const;

	// set / get
	void SetPosition(const Vector3& p);
	Vector3 GetPosition() const;
	void SetRotation(const Quaternion& r);
	Quaternion GetRotation() const;
	void SetScaling(const Vector3& s);
	Vector3 GetScaling() const;
};

// Interpolation
PRS Slerp(const PRS& a, const PRS& b, const float& t);
void PrintPRS(const PRS &prs);

#endif // ndef _PRS_HEADER_
