// stat.cpp

#include "stat.h"
#include "math3d.h"

float Average(const std::vector<float> &ra)
{
	if (!ra.size()) return 0;
	float sum = 0;
	for (unsigned int i=0; i<ra.size(); i++)
	{
		sum += ra[i];
	}
	return sum / ra.size();
}

std::vector<float> ExcludeDistant(const std::vector<float> &ra)
{
	const float dist_tol = 4.0f;

	float avg = Average(ra);
	std::vector<float> dist;
	for (unsigned int i=0; i<ra.size(); i++)
	{
		dist.push_back(fabs(ra[i] - avg));
	}
	float dist_avg = Average(dist);
	std::vector<float> ret;
	for (unsigned int i=0; i<ra.size(); i++)
	{
		if (dist[i] < dist_tol * dist_avg)
		{
			ret.push_back(ra[i]);
		}
	}

	return ret;
}

float LocalAverage(const std::vector<float> &ra)
{
	return Average(ExcludeDistant(ra));
}

Vector2 Average(const std::vector<Vector2> &ra)
{
	if (!ra.size()) return Vector2(0, 0);
	Vector2 sum = Vector2(0, 0);
	for (unsigned int i=0; i<ra.size(); i++)
	{
		sum += ra[i];
	}
	return sum / (float)ra.size();
}

Vector3 Average(const std::vector<Vector3> &ra)
{
	if (!ra.size()) return Vector3(0, 0, 0);
	Vector3 sum = Vector3(0, 0, 0);
	for (unsigned int i=0; i<ra.size(); i++)
	{
		sum += ra[i];
	}
	return sum / (float)ra.size();
}

void Split(const std::vector<float> &ra, float val, std::vector<float> &l,
		   std::vector<float> &ge)
{
	l.erase(l.begin(), l.end());
	ge.erase(ge.begin(), ge.end());
	for (unsigned int i=0; i<ra.size(); i++)
	{
		if (ra[i] < val) 
		{
			l.push_back(ra[i]);
		}
		else
		{
			ge.push_back(ra[i]);
		}
	}
}

// simple least squares
// x component is the independent variable and y is its function
// return value is a Vector2(a, b) , where a and b are the parameters
// of a linear function : y = a*x + b
Vector2 LeastSq(const std::vector<Vector2> &ra)
{
	Vector2 avg = Average(ra);

	float sum_xsq = 0, sum_xy = 0;
	for (unsigned int i=0; i<ra.size(); i++)
	{
		float dist_x = ra[i].x - avg.x;
		float dist_y = ra[i].y - avg.y;
		sum_xsq += SQ(dist_x);
		sum_xy += dist_x * dist_y;
	}

	if (sum_xsq < XSmallNumber)
	{
		// not a function. all y's correspond to the same x
		// return a line with a large slope
		return Vector2(tan(HalfPi - SmallNumber), 0.0f);
	}

	float b = sum_xy / sum_xsq;
	return Vector2(avg.y - b * avg.x, b);
}
