// dialog.cpp
#include "dialog.h"
#include "resource.h"
#include <windows.h>
#include <math.h>
#include <commctrl.h>
#include <string>
#include "CImg.h"

extern HINSTANCE hinstance;
static DialogData data = {true, true, true, true, true, true};
bool ok = false;

static HWND dlg_handle = 0;
static HWND prog_dlg_handle = 0;

unsigned long thread_id = 0;
HANDLE thread_handle;

float progress;
std::string progress_msg;
bool show_progress;

void CloseProgressDialog()
{
	show_progress = false;
	WaitForSingleObject(thread_handle, 0);
	//TerminateThread(thread_handle, 0);
}

void SetProgress(float p)
{
	progress = p;
	HWND prog_handle = GetDlgItem(prog_dlg_handle, IDC_PROGRESS1);
	SendMessage(prog_handle, SBM_SETPOS, (int) p * 100.0f, 0);
	RedrawWindow(prog_dlg_handle, 0, 0, RDW_UPDATENOW);
}

void SetProgressMsg(const char  *msg)
{
	progress_msg = msg;
}

static void PositionWindow(HWND w, int x, int y)
{
	RECT wr;
	GetWindowRect(w, &wr);
	int ww = wr.right - wr.left;
	int wh = wr.bottom - wr.top;
	MoveWindow(w, x - ww / 2, y - wh / 2, ww, wh, true); 
}

static void CenterWindow(HWND w)
{
	unsigned long dw, dh;
	dw = GetSystemMetrics(SM_CXSCREEN);
	dh = GetSystemMetrics(SM_CYSCREEN);
	
	PositionWindow(w, dw / 2, dh / 2);
}

static LRESULT CALLBACK ProgDlgProc(HWND hWndDlg, 
							  UINT Msg, 
							  WPARAM wParam, 
							  LPARAM lParam)
{
	//MessageBox(0, "foo", "bar", 0);
	switch(Msg)
	{
	case WM_INITDIALOG:
		CenterWindow(hWndDlg);
		prog_dlg_handle = hWndDlg;
		return TRUE;
	case WM_PAINT:
		break;
	}

	return DefWindowProc(hWndDlg, Msg, wParam, lParam);
}

using namespace cimg_library;
unsigned long WINAPI progress_thread(void *dat)
{
	/*
	DialogBox(hinstance, MAKEINTRESOURCE(IDD_DIALOG2), 0, 
		reinterpret_cast<DLGPROC>(ProgDlgProc));
	*/

	show_progress = true;

	CImg<unsigned char> image(400, 80, 1, 3);
	image.fill(0, 0, 0);
	for (unsigned int j=20; j<50; j++)
	{
		for (unsigned int i=0; i<400; i++)
		{
			if (j > 36 && j < 40)
			{
				image(i, j, 0) = 50;
				image(i, j, 1) = 50;
				image(i, j, 2) = 100;
			}
			else
			{	
				image(i, j, 0) = 30;
				image(i, j, 1) = 30;
				image(i, j, 2) = 80;
			}
		}
	}
	CImgDisplay disp(image, "Track7 Progress Saloon (TM)");
	while (show_progress)
	{
		for (unsigned int j=20; j<50; j++)
		{
			for (unsigned int i=0; i<400 * progress; i++)
			{
				if (j > 36 && j < 40)
				{
					image(i, j, 0) = 50;
					image(i, j, 1) = 50;
					image(i, j, 2) = 100;
				}
				else
				{	
					image(i, j, 0) = 30;
					image(i, j, 1) = 30;
					image(i, j, 2) = 80;
				}
			}
		}
		image.blur(2.5);

		// draw text
		CImg<unsigned char> img2 = image;
		const unsigned char clr[3] = {190 + 64 * sin(GetTickCount() / 1000.0f), 0, 0};
		img2.draw_text(progress_msg.c_str(), 20, 5, clr);

		img2.display(disp);
		Sleep(10);
	}
	disp.close();

	return 0;
}

void StartProgressDialog()
{
	thread_handle = CreateThread(0, 0, progress_thread, 0,0, &thread_id);
}

static LRESULT CALLBACK DialogProc(HWND hWndDlg, 
							  UINT Msg, 
							  WPARAM wParam, 
							  LPARAM lParam)
{
	switch(Msg)
	{
	case WM_INITDIALOG:
		dlg_handle = hWndDlg;
		CheckDlgButton(hWndDlg,ID_NORMALS_BOX, data.normals);
		CheckDlgButton(hWndDlg,ID_TANGENTS_BOX, data.tangents);
		CheckDlgButton(hWndDlg,ID_TC0_BOX, data.tc0);
		CheckDlgButton(hWndDlg,ID_TC1_BOX, data.tc1);
		CheckDlgButton(hWndDlg,ID_TC2_BOX, data.tc2);
		CheckDlgButton(hWndDlg,ID_TC3_BOX, data.tc3);
		CheckDlgButton(hWndDlg,IDC_RADIO4, true);
		CenterWindow(hWndDlg);
		ok = false;
		return TRUE;

	case WM_COMMAND:
		switch(wParam)
		{
		case IDOK:
			if (IsDlgButtonChecked(hWndDlg, ID_NORMALS_BOX) == BST_CHECKED)
			{
				data.normals = true;
			}
			else
			{
				data.normals = false;
			}

			if (IsDlgButtonChecked(hWndDlg, ID_TANGENTS_BOX) == BST_CHECKED)
			{
				data.tangents = true;
			}
			else
			{
				data.tangents = false;
			}

			if (IsDlgButtonChecked(hWndDlg, ID_TC0_BOX) == BST_CHECKED)
			{
				data.tc0 = true;
			}
			else
			{
				data.tc0 = false;
			}

			if (IsDlgButtonChecked(hWndDlg, ID_TC1_BOX) == BST_CHECKED)
			{
				data.tc1 = true;
			}
			else
			{
				data.tc1 = false;
			}

			if (IsDlgButtonChecked(hWndDlg, ID_TC2_BOX) == BST_CHECKED)
			{
				data.tc2 = true;
			}
			else
			{
				data.tc2 = false;
			}

			if (IsDlgButtonChecked(hWndDlg, ID_TC3_BOX) == BST_CHECKED)
			{
				data.tc3 = true;
			}
			else
			{
				data.tc3 = false;
			}

			if (IsDlgButtonChecked(hWndDlg, IDC_RADIO4) == BST_CHECKED)
			{
				data.tex_func = 0;
			}
			if (IsDlgButtonChecked(hWndDlg, IDC_RADIO5) == BST_CHECKED)
			{
				data.tex_func = 1;
			}
			if (IsDlgButtonChecked(hWndDlg, IDC_RADIO2) == BST_CHECKED)
			{
				data.tex_func = 2;
			}
			if (IsDlgButtonChecked(hWndDlg, IDC_CHECK1) == BST_CHECKED)
			{
				data.preview = true;
			}
			else
			{
				data.preview = false;
			}

			EndDialog(hWndDlg, 0);
			ok = true;
			return TRUE;

		case IDCANCEL:
			EndDialog(hWndDlg, 0);
			ok = false;
			return TRUE;
		}
		break;


	}

	return FALSE;
}

bool DisplayDialog()
{
	DialogBox(hinstance, MAKEINTRESOURCE(IDD_DIALOG1), 0, 
		reinterpret_cast<DLGPROC>(DialogProc));

	return ok;
}

DialogData GetDialogData()
{
	return data;
}

