// plugin.h
// Max plugin class

#include <windows.h>
#include "maxinc.h"
#include "sce.h"
//#include "types.h"
#include "dialog.h"
#include "anm.h"

#ifndef _PLUGIN_HEADER_
#define _PLUGIN_HEADER_

// unique identifier for our new plugin

class ExportPlugin : public SceneExport
{
private:
	IGameScene *scene;

	Chunk* curr_file_node;

	IntHash mat_map;
	unsigned long mat_count;

	DialogData dialog_data;

	bool InitExport();
	void CloseExport();

	void ProcessSingleNode(IGameNode* node);
	void ProcessNode(IGameNode* node);

	void ProcessCamera(IGameCamera* camera, IGameNode* node);
	void ProcessLight(IGameLight* light, IGameNode* node);
	void ProcessMesh(IGameMesh* mesh, IGameNode* node);

	void ProcessMaterial(IGameMaterial* mat);
	
public:
	
	// constructor
	ExportPlugin();
	~ExportPlugin();

	// Methods required for the plugin
	int	ExtCount();						// Number of extemsions supported
	const TCHAR * Ext(int n);			// Extension #n (i.e. "3DS")
	const TCHAR * LongDesc();			// Long ASCII description (i.e. "Autodesk 3D Studio File")
	const TCHAR * ShortDesc();			// Short ASCII description (i.e. "3D Studio")
	const TCHAR * AuthorName();			// ASCII Author name
	const TCHAR * CopyrightMessage();	// ASCII Copyright message
	const TCHAR * OtherMessage1();		// Other message #1
	const TCHAR * OtherMessage2();		// Other message #2
	unsigned int Version();				// Version number * 100 (i.e. v3.01 = 301)
	void ShowAbout(HWND hWnd);			// Show DLL's "About..." box
	int DoExport(const TCHAR *name, ExpInterface *ei,
		Interface *i, BOOL suppressPrompts = FALSE,
		DWORD options = 0);				// Export file
	BOOL SupportsOptions(int ext, DWORD options); 
										// Returns TRUE if all option bits set are supported for this extension
}; // end class ExportPlugin


// animation exporter
// -------------------
class AnimationPlugin : public SceneExport
{
private:
	IGameScene *scene;

	AnimationFile* animation;

	void ProcessSingleNode(IGameNode* node);
	void ProcessNode(IGameNode* node);

	void Cleanup();

public:
	
	// constructor
	AnimationPlugin();
	~AnimationPlugin();

	// Methods required for the plugin
	int	ExtCount();						// Number of extemsions supported
	const TCHAR * Ext(int n);			// Extension #n (i.e. "3DS")
	const TCHAR * LongDesc();			// Long ASCII description (i.e. "Autodesk 3D Studio File")
	const TCHAR * ShortDesc();			// Short ASCII description (i.e. "3D Studio")
	const TCHAR * AuthorName();			// ASCII Author name
	const TCHAR * CopyrightMessage();	// ASCII Copyright message
	const TCHAR * OtherMessage1();		// Other message #1
	const TCHAR * OtherMessage2();		// Other message #2
	unsigned int Version();				// Version number * 100 (i.e. v3.01 = 301)
	void ShowAbout(HWND hWnd);			// Show DLL's "About..." box
	int DoExport(const TCHAR *name, ExpInterface *ei,
		Interface *i, BOOL suppressPrompts = FALSE,
		DWORD options = 0);				// Export file
	BOOL SupportsOptions(int ext, DWORD options); 
										// Returns TRUE if all option bits set are supported for this extension
}; // end class AnimationPlugin

// returns the class descriptor
ClassDesc2* GetExportPluginDesc();
ClassDesc2* GetAnimationPluginDesc();

#endif
