// calendar.cpp

#include "calendar.h"

using namespace boost::gregorian;

Date::Date()
{
	// Initialize to today
	d = date(day_clock::local_day());
	Refresh();
}

Date::Date(unsigned int year, unsigned int month, unsigned int day)
{
	d = date(year, month, day);
	Refresh();
}

Date::Date(const std::string &str)
{
	std::string y,m,d;
	unsigned int mode = 0;
	for (unsigned int i=0; i<str.size(); i++)
	{
		if (str[i] == '/')
		{
			mode ++;
			if (mode == 3) break;
			continue;
		}
		switch (mode)
		{
		case 0:
			d += str[i];
			break;
		case 1:
			m += str[i];
			break;
		case 2:
			y += str[i];
		}
	}

	year = atoi(y.c_str());
	if (year < 2000) year += 2000;
	month = atoi(m.c_str());
	day = atoi(d.c_str());
	*this = Date(year, month, day);
}

void Date::Refresh()
{
	std::string str = to_iso_string(d);
	std::string y,m,d;
	y += str[0];
	y += str[1];
	y += str[2];
	y += str[3];

	m += str[4];
	m += str[5];

	d += str[6];
	d += str[7];

	year = atoi(y.c_str());
	month = atoi(m.c_str());
	day = atoi(d.c_str());
}

Date Date::operator + (int days) const
{
	Date ret = *this;
	int days_to_add = days;
	bool subtr = false;
	if (days < 0)
	{
		subtr = true;
		days_to_add = -days;
	}
	date_duration dd(days_to_add);
	if (!subtr)
	{
		ret.d = ret.d + dd;
	}
	else
	{
		ret.d = ret.d - dd;
	}

	ret.Refresh();
	
	return ret;
}

Date Date::operator - (int days) const
{
	return *this + (-days);
}

void Date::operator += (int days)
{
	*this = *this + days;
}

void Date::operator -= (int days)
{
	*this = *this - days;
}

bool Date::operator > (const Date &date) const
{
	int dd = *this - date;
	if (dd > 0) return true;
	return false;
}

bool Date::operator < (const Date &date) const
{
	int dd = *this - date;
	if (dd < 0) return true;
	return false;
}

bool Date::operator == (const Date &date) const
{
	int dd = *this - date;
	if (dd == 0) return true;
	return false;
}

bool Date::operator >= (const Date &date) const
{
	return !(*this < date);
}

bool Date::operator <= (const Date &date) const
{
	return !(*this > date);
}

int Date::operator - (const Date &date) const
{
	date_duration dd = d - date.d;
	return dd.days();
}

std::string Date::GetWeekDay() const
{
	return d.day_of_week().as_long_string();
}

std::string Date::GetString() const
{
	std::string m = d.month().as_long_string();
	char buf[256];
	sprintf(buf, "%d %s %d", day, m.c_str(), year);
	return buf;
}

unsigned int Date::GetYear()
{
	return year;
}

unsigned int Date::GetMonth()
{
	return month;
}

unsigned int Date::GetDay()
{
	return day;
}

unsigned int WorkingDays(const Date &from, const Date &to)
{
	if (to - from < 0) return 0;
	unsigned int ret = 0;
	for (Date i = from; i<to; i+=1)
	{
		if (i.GetWeekDay() != "Saturday" && i.GetWeekDay() != "Sunday")
		{
			ret ++;
		}
	}
	return ret;
}

Date WorkingDaysAdd(const Date &from, unsigned int work)
{
	Date ret = from;
	for (unsigned int i=0; i<work; i++)
	{
		while (ret.GetWeekDay() == "Saturday" ||
			ret.GetWeekDay() == "Sunday")
		{
			ret += 1;
		}
		ret += 1;
	}
	return ret;
}
