// plan.cpp

#include "plan.h"

static std::string TextToLink(const std::string &str)
{
	std::string ret;
	for (unsigned int i=0; i<str.size() && i < 30; i++)
	{
		if (str[i] >= 'a' && str[i] <= 'z')
			ret += str[i];
	}
	return ret;
}

Project::Project()
{

}

Project::~Project()
{

}

bool Project::Load(const char *filename)
{
	TxtNode *root = LoadTxt(filename);
	if (!root) return false;
	if (!root->GetChildrenByType("project").size())
	{
		delete root;
		return false;
	}

	Destroy();

	TxtNode *project = root->GetChildrenByType("project")[0];
	name = project->GetValue("name");
	from = Date(project->GetValue("start"));

	// load coders
	TxtNodeList coder_list = project->GetChildrenByType("coder");
	for (unsigned int i=0; i<coder_list.size(); i++)
	{
		Coder new_coder;
		new_coder.name = coder_list[i]->GetValue("name");
		new_coder.initials = coder_list[i]->GetValue("initials");
		new_coder.email = coder_list[i]->GetValue("email");
		coders.push_back(new_coder);
	}

	// load tasks
	TxtNodeList task_list = project->GetChildrenByType("task");
	for (unsigned int i=0; i<task_list.size(); i++)
	{
		Task new_task;
		new_task.name = task_list[i]->GetValue("name");
		new_task.duration = atoi(task_list[i]->GetValue("duration").c_str());
		std::string from = task_list[i]->GetValue("from");
		if (from == "")
		{
			new_task.assigned = false;
		}
		else
		{
			new_task.from = Date(from);
			new_task.assigned = true;
		}
		std::string to = task_list[i]->GetValue("to");
		if (to == "")
		{
			new_task.to = Date();
			new_task.complete = false;
		}
		else
		{
			new_task.to = Date(to);
			new_task.complete = true;
		}
		new_task.coder = task_list[i]->GetValue("coder");

		tasks.push_back(new_task);
	}

	// load notes
	TxtNodeList note_list = project->GetChildrenByType("note");
	for (unsigned int i=0; i<note_list.size(); i++)
	{
		Note new_note;
		new_note.from = note_list[i]->GetValue("from");
		new_note.message = note_list[i]->GetValue("message");
		new_note.about = note_list[i]->GetValue("about");
		new_note.date = Date(note_list[i]->GetValue("date"));
		notes.push_back(new_note);
	}

	delete root;
	return true;
}

static std::vector<Task> SortByFinishDate(const std::vector<Task> &list)
{
	std::vector<Task> ret = list;
	for (unsigned int i=0; i<ret.size(); i++)
	{
		for (unsigned int j = (unsigned int)ret.size() - 1; j>i; j--)
		{
			if (ret[j].to < ret[j-1].to)
			{
				Task tmp = ret[j];
				ret[j] = ret[j-1];
				ret[j-1] = tmp;
			}
		}
	}

	return ret;
}

static std::vector<Task> SortByAssignDate(const std::vector<Task> &list)
{
	std::vector<Task> ret = list;
	for (unsigned int i=0; i<ret.size(); i++)
	{
		for (unsigned int j = (unsigned int)ret.size() - 1; j>i; j--)
		{
			if (ret[j].from < ret[j-1].from)
			{
				Task tmp = ret[j];
				ret[j] = ret[j-1];
				ret[j-1] = tmp;
			}
		}
	}

	return ret;
}

static std::vector<Note> SortByDate(const std::vector<Note> &list)
{
	std::vector<Note> ret = list;
	for (unsigned int i=0; i<ret.size(); i++)
	{
		for (unsigned int j = (unsigned int)ret.size() - 1; j>i; j--)
		{
			if (ret[j].date < ret[j-1].date)
			{
				Note tmp = ret[j];
				ret[j] = ret[j-1];
				ret[j-1] = tmp;
			}
		}
	}

	return ret;
}

bool Project::SaveReport(const char *filename)
{
	FILE *fp = fopen(filename, "w+");
	if (!fp) return false;

	std::string title = "Programming team report for project \"";
	title += name;
	title += "\"";

	// html stuff
	fprintf(fp, "<html><head><title>\n");
	fprintf(fp, title.c_str());
	fprintf(fp, "</title></head><body>\n");
	
	// header
	fprintf(fp, "<h1 align=\"center\">%s</h1>\n", title.c_str());
	fprintf(fp, "<i><h2 align=\"center\">Date: %s</h2></i><hr>", 
		Date().GetString().c_str());

	// table of contents
	fprintf(fp, "<h2><u><b>Table of Contents</b></u></h2>\n");
	fprintf(fp, "<ul>\n");
	fprintf(fp, "<li><a href=\"#assigned\">Assigned tasks</li>\n");
	fprintf(fp, "<li><a href=\"#pool\">Task pool</li>\n");
	fprintf(fp, "<li><a href=\"#completed\">All completed tasks</li>\n");
	for (unsigned int i=0; i<coders.size(); i++)
	{
		std::string section = "completedby" + coders[i].initials;
		fprintf(fp, "<li><a href=\"#%s\">Tasks completed by %s</li>\n",
			section.c_str(), coders[i].name.c_str());
	}
	fprintf(fp, "<li><a href=\"#summary\">Summary</li>\n");
	fprintf(fp, "<li><a href=\"#notes\">Notes</li>\n");
	fprintf(fp, "</ul><hr>");

	// assigned tasks
	int assigned_days_remaining = 0;
	fprintf(fp, "<p><b><u><a name=\"assigned\">Assigned tasks</a></u></b></p>\n");
	fprintf(fp, "<table align=\"center\" border=\"1\" width=\"80%%\">");
	std::vector<Task> assn = GetAssignedTasks();
	fprintf(fp, "<tr>\n");
	fprintf(fp, "<td><b>Task name</b></td>\n");
	fprintf(fp, "<td><b>Duration</b></td>\n");
	fprintf(fp, "<td><b>Assigned to</b></td>\n");
	fprintf(fp, "<td><b>From</b></td>\n");
	fprintf(fp, "<td><b>Days working</b></td>\n");
	fprintf(fp, "<td><b>Notes</b></td>\n");
	fprintf(fp, "</tr>\n");
	for (unsigned int i=0; i<assn.size(); i++)
	{
		unsigned int dur = WorkingDays(assn[i].from, Date());
		std::string color = "FFFFFF";
		if (dur > assn[i].duration)
		{
			color = "FF4444";
		}
		else
		{
			assigned_days_remaining += assn[i].duration - dur;
		}
		fprintf(fp, "<tr bgcolor=\"%s\">\n", color.c_str());
		fprintf(fp, "<td>%s</td>\n", assn[i].name.c_str());
		fprintf(fp, "<td>%d</td>\n", assn[i].duration);
		fprintf(fp, "<td>%s</td>\n", assn[i].coder.c_str());
		fprintf(fp, "<td>%s</td>\n", assn[i].from.GetString().c_str());
		fprintf(fp, "<td>%d</td>\n", dur);
		
		// provide links to relevant notes
		fprintf(fp, "<td>");
		fprintf(fp, "%s\n", GetNotesAboutLink(assn[i].name).c_str());
		fprintf(fp, "</td>");
		
		fprintf(fp, "</tr>\n");
	}
	fprintf(fp, "</table>");
	fprintf(fp, "<table width=\"40%%\" align=\"center\" bgcolor=\"AAAAAA\">\n");
	fprintf(fp, "<tr><td align=\"center\">ProgrammerDays remaining: %d</td></tr></table>",
		assigned_days_remaining);


	// task pool
	unsigned int total_coder_days = 0;
	fprintf(fp, "<p><b><u><a name=\"pool\">Task pool</a></u></b></p>\n");
	fprintf(fp, "<table align=\"center\" border=\"1\" width=\"80%%\">");
	std::vector<Task> tp = GetTaskPool();
	fprintf(fp, "<tr>\n");
	fprintf(fp, "<td><b>Task name</b></td>\n");
	fprintf(fp, "<td><b>Duration</b></td>\n");
	fprintf(fp, "<td><b>Notes</b></td>\n");
	fprintf(fp, "</tr>\n");
	for (unsigned int i=0; i<tp.size(); i++)
	{
		total_coder_days += tp[i].duration;
		fprintf(fp, "<tr>\n");
		fprintf(fp, "<td>%s</td>\n", tp[i].name.c_str());
		fprintf(fp, "<td>%d</td>\n", tp[i].duration);
		
		// provide links to relevant notes
		fprintf(fp, "<td>");
		fprintf(fp, "%s\n", GetNotesAboutLink(tp[i].name).c_str());
		fprintf(fp, "</td>");
		fprintf(fp, "</tr>\n");
	}
	fprintf(fp, "</table>");
	fprintf(fp, "<table width=\"40%%\" align=\"center\" bgcolor=\"AAAAAA\">\n");
	fprintf(fp, "<tr><td align=\"center\">ProgrammerDays remaining: %d</td></tr></table>",
		total_coder_days);

	// completed tasks
	fprintf(fp, "<p><b><u><a name=\"completed\">Completed tasks - sorted by completion date</a></u></b></p>\n");
	PrintCompletedTasksTable(SortByFinishDate(GetCompletedTasks()), fp);

	// completed tasks by coder
	for (unsigned int i=0; i<coders.size(); i++)
	{
		std::string section = "completedby" + coders[i].initials;
		fprintf(fp, "<p><b><u><a name=\"%s\">Tasks completed by %s</a></u></b></p>\n", 
			section.c_str(),
			coders[i].name.c_str());
		PrintCompletedTasksTable(SortByFinishDate(GetCompletedTasksByCoder(coders[i].initials)), fp);
	}

	// summary
	unsigned int coder_day_sum = assigned_days_remaining + 
		total_coder_days;
	Date completion_date = WorkingDaysAdd(Date(), 
		coder_day_sum / coders.size());
	fprintf(fp, "<p><b><u><a name=\"summary\">Summary</a></u></b></p>\n");
	fprintf(fp, "<p>Project \"%s\" start date: %s</p>\n", 
		name.c_str(), from.GetString().c_str());
	fprintf(fp, "<p>Total ProgrammerDays remaining: %d</p>\n", 
		coder_day_sum);
	fprintf(fp, "<p>Total Working Days remaining: %d</p>\n", 
		coder_day_sum / coders.size());
	fprintf(fp, "<p>Estimated completion date: %s</p>\n", 
		completion_date.GetString().c_str());

	// notes
	fprintf(fp, "<p><b><u><a name=\"notes\">Notes</a></u></b></p>\n");
	std::vector<Note> snotes = SortByDate(notes);
	for (unsigned int i=0; i<snotes.size(); i++)
	{
		fprintf(fp, "<table border=\"1\" width=\"80%%\" align=\"center\">\n");
		fprintf(fp, "<a name=\"%s\"></a>", TextToLink(snotes[i].message).c_str());
		fprintf(fp, "<tr>\n");
		fprintf(fp, "<td width=\"25%%\">%s</td>\n", GetCoderLink(snotes[i].from).c_str());
		fprintf(fp, "<td width=\"15%%\">%s</td>\n", snotes[i].date.GetString().c_str());
		fprintf(fp, "<td>%s</td>\n", snotes[i].message.c_str());
		fprintf(fp, "</tr>\n");
		fprintf(fp, "</table><br>");
	}

	// leave a lot of blank space to make links work correctly
	for (unsigned int i=0; i<80; i++)
	{
		fprintf(fp, "<br>");
	}

	// footer
	fprintf(fp, "<hr><i>%s<br>%s</i></body></html>", title.c_str(), 
		Date().GetString().c_str());

	fclose(fp);

	return true;
}

void Project::Destroy()
{
	coders.erase(coders.begin(), coders.end());
	tasks.erase(tasks.begin(), tasks.end());
	notes.erase(notes.begin(), notes.end());
}

std::vector<Task> Project::GetCompletedTasks() const
{
	std::vector<Task> ret;
	for (unsigned int i=0; i<tasks.size(); i++)
	{
		if (tasks[i].complete)
		{
			ret.push_back(tasks[i]);
		}
	}
	return ret;
}

std::vector<Task> Project::GetAssignedTasks() const
{
	std::vector<Task> ret;
	for (unsigned int i=0; i<tasks.size(); i++)
	{
		if (!tasks[i].complete && tasks[i].assigned)
		{
			ret.push_back(tasks[i]);
		}
	}
	return ret;
}

std::vector<Task> Project::GetTaskPool() const
{
	std::vector<Task> ret;
	for (unsigned int i=0; i<tasks.size(); i++)
	{
		if (!tasks[i].complete && !tasks[i].assigned)
		{
			ret.push_back(tasks[i]);
		}
	}
	return ret;
}

std::vector<Task> Project::GetCompletedTasksByCoder(const std::string &initials) const
{
	std::vector<Task> ret;
	for (unsigned int i=0; i<tasks.size(); i++)
	{
		if (tasks[i].complete && tasks[i].coder == initials)
		{
			ret.push_back(tasks[i]);
		}
	}
	return ret;
}

Coder Project::GetCoder(const std::string &initials) const
{
	for (unsigned int i=0; i<coders.size(); i++)
	{
		if (coders[i].initials == initials) return coders[i];
	}

	// error
	Coder err;
	err.name = "Error: No coder specified with initials ";
	err.name += initials;
	err.initials = "Error";
	return err;
}

void Project::PrintCompletedTasksTable(const std::vector<Task> &list, FILE *fp)
{
	if (!fp) return;
	int total_delay = 0;
	unsigned int total_days_worked = 0;
	fprintf(fp, "<table align=\"center\" border=\"1\" width=\"80%%\">");
	fprintf(fp, "<tr>\n");
	fprintf(fp, "<td><b>Task name</b></td>\n");
	fprintf(fp, "<td><b>Duration</b></td>\n");
	fprintf(fp, "<td><b>Completed by</b></td>\n");
	fprintf(fp, "<td><b>From</b></td>\n");
	fprintf(fp, "<td><b>To</b></td>\n");
	fprintf(fp, "<td><b>Days working</b></td>\n");
	fprintf(fp, "<td><b>Delay</b></td>\n");
	fprintf(fp, "<td><b>Notes</b></td>\n");
	fprintf(fp, "</tr>\n");
	for (unsigned int i=0; i<list.size(); i++)
	{
		unsigned int dur = WorkingDays(list[i].from, list[i].to);
		total_days_worked += dur;
		std::string color = "FFFFFF";
		int delay = dur - list[i].duration;
		total_delay += delay;
		if (dur > list[i].duration) color = "FF4444";
		if (dur < list[i].duration) color = "44FF44";
		fprintf(fp, "<tr bgcolor=\"%s\">\n", color.c_str());
		fprintf(fp, "<td>%s</td>\n", list[i].name.c_str());
		fprintf(fp, "<td>%d</td>\n", list[i].duration);
		fprintf(fp, "<td>%s</td>\n", GetCoderLink(list[i].coder, true).c_str());
		fprintf(fp, "<td>%s</td>\n", list[i].from.GetString().c_str());
		fprintf(fp, "<td>%s</td>\n", list[i].to.GetString().c_str());
		fprintf(fp, "<td>%d</td>\n", dur);
		fprintf(fp, "<td>%d</td>\n", delay);

		// provide links to relevant notes
		fprintf(fp, "<td>");
		fprintf(fp, "%s\n", GetNotesAboutLink(list[i].name).c_str());
		fprintf(fp, "</td>");

		fprintf(fp, "</tr>\n");
	}
	fprintf(fp, "</table>");

	std::string color = "AAAAAA";
	if (total_delay > 0) color = "FF4444";
	if (total_delay < 0) color = "44FF44";
	fprintf(fp, "<table width=\"40%%\" align=\"center\" bgcolor=\"%s\">\n", 
		color.c_str());
	fprintf(fp, "<tr><td align=\"center\">Total Delay: %d</td></tr>", total_delay);
	fprintf(fp, "<tr><td align=\"center\">Total ProgrammerDays: %d</td></tr></table>", total_days_worked);
}

std::string Project::GetCoderLink(std::string initials, bool sh)
{
	Coder c = GetCoder(initials);
	std::string display_name = c.name;
	if (sh) display_name = c.initials;
	if (c.email == "") return display_name;
	return std::string("<a href=\"mailto:") + c.email +
		std::string("\">") + display_name + std::string("</a>");
}


std::vector<Note> Project::GetNotesAbout(const std::string &about)
{
	std::vector<Note> ret;
	for (unsigned int i=0; i<notes.size(); i++)
	{
		if (notes[i].about == about) 
		{
			ret.push_back(notes[i]);
		}
	}
	return ret;
}

std::string Project::GetNotesAboutLink(const std::string &about)
{
		std::vector<Note> relevant = GetNotesAbout(about);
		std::string link;
		for (unsigned int i=0; i<relevant.size(); i++)
		{
			char buffer[1024];
			sprintf(buffer, "<a href=\"#%s\">%d, </a>",
				TextToLink(relevant[i].message).c_str(), 
				i+1);
			link += buffer;
		}

		return link;
}
