// plan.h

#ifndef _PLAN_H_
#define _PLAN_H_

#include "txtfile.h"
#include "calendar.h"

struct Coder
{
	std::string name;
	std::string initials;
	std::string email;
};

struct Task
{
	std::string name, coder;
	Date from, to;
	unsigned int duration;
	bool assigned, complete;
};

struct Note
{
	std::string from;
	std::string message;
	std::string about;
	Date date;
};

class Project
{
protected:
	std::vector<Coder> coders;
	std::vector<Task> tasks;
	std::vector<Note> notes;
	std::string name;
	Date from;

public:
	Project();
	~Project();

	bool Load(const char *filename);
	bool SaveReport(const char *filename);
	void Destroy();

	std::vector<Task> GetCompletedTasks() const;
	std::vector<Task> GetAssignedTasks() const;
	std::vector<Task> GetTaskPool() const;
	std::vector<Task> GetCompletedTasksByCoder(const std::string &initials) const;
	Coder GetCoder(const std::string &initials) const;

	void PrintCompletedTasksTable(const std::vector<Task> &list, FILE *fp);

	std::string GetCoderLink(std::string initials, bool sh=false);

	std::vector<Note> GetNotesAbout(const std::string &about);
	std::string GetNotesAboutLink(const std::string &about);
};

#endif // ndef _PLAN_H_
