// txtfile.cpp
#include "txtfile.h"
#include <stdio.h>

TxtNode::TxtNode()
{
	parent = 0;
}

TxtNode::~TxtNode()
{
	for (unsigned int i=0; i<children.size(); i++)
	{
		if (children[i])
		{
			delete children[i];
			children[i] = 0;
		}
	}
}

void TxtNode::SetValue(const std::string &which, const std::string &val)
{
	values[which] = val;
}

std::string TxtNode::GetValue(const std::string &which) 
{
	return values[which];
}

std::string TxtNode::GetType() const
{
	return type;
}

void TxtNode::SetType(const std::string &type)

{	this->type = type;
}

void TxtNode::Mount(TxtNode *child)
{
	for (unsigned int i=0; i<children.size(); i++)
	{
		if (children[i] == child) return;
	}

	child->parent = this;
	children.push_back(child);
}

void TxtNode::Unmount(TxtNode *child)
{
	for (unsigned int i=0; i<children.size(); i++)
	{
		if (children[i] == child)
		{
			children[i]->parent = 0;
			children.erase(children.begin() + i, children.begin() + i + 1);
			return;
		}
	}
}

TxtNode *TxtNode::GetParent()
{
	return parent;
}

TxtNodeList TxtNode::GetChildren() const
{
	return children;
}

TxtNodeList TxtNode::GetChildrenByType(const std::string &type) const
{
	TxtNodeList ret;
	for (unsigned int i=0; i<children.size(); i++)
	{
		if (children[i]->type == type) ret.push_back(children[i]);
	}
	return ret;
}

static std::string RemoveBlanks(const std::string &str)
{
	std::string ret;
	for (unsigned int i=0; i<str.size(); i++)
	{
		if (str[i] != ' ' && str[i] != '\t')
			ret += str[i];
	}
	return ret;
}

std::string RemoveBeforeAfterBlanks(const std::string &str)
{
	int start = -1;
	for (unsigned int i=0; i<str.size(); i++)
	{
		if (str[i] != ' ' && str[i] != '\t')
		{
			start = i;
			break;
		}
	}
	if (start == -1) return "";

	int end = start;
	for (int i=str.size()-1; i>=start; i--)
	{
		if (str[i] != ' ' && str[i] != '\t')
		{
			end = i+1;
			break;
		}
	}

	std::string ret;
	for (int i=start; i<end; i++)
	{
		ret += str[i];
	}
	return ret;
}

static bool IsNewNode(const std::string &line)
{
	if (line[line.size() - 1] == '{') return true;
	return false;
}

static std::string GetType(const std::string &line)
{
	if (!IsNewNode(line)) return "";
	std::string ret = RemoveBlanks(line);
	ret.erase(ret.begin() + ret.size() - 1, ret.begin() + ret.size());
	return ret;
}

static bool IsEnding(const std::string &line)
{
	std::string clean = RemoveBlanks(line);
	if (!clean.size()) return false;
	if (clean[0] == '}') return true;
	return false;
}

static bool IsEquation(const std::string &line)
{
	for (unsigned int i=0; i<line.size(); i++)
	{
		if (line[i] == '=') return true;
	}

	return false;
}

static std::string LeftHand(const std::string &line)
{
	if (!IsEquation(line)) return "";
	std::string ret;
	for (unsigned int i=0; i<line.size(); i++)
	{
		if (line[i] == '=') break;
		ret += line[i];
	}
	return RemoveBlanks(ret);
}

static std::string RightHand(const std::string &line)
{
	if (!IsEquation(line)) return "";
	unsigned int start = 0;
	for (unsigned int i=0; i<line.size(); i++)
	{
		if (line[i] == '=')
		{
			start = i + 1;
		}
	}

	std::string ret;
	for (unsigned int i=start; i<line.size(); i++)
	{
		ret += line[i];
	}

	return RemoveBeforeAfterBlanks(ret);
}

TxtNode* LoadText(const std::vector<std::string> &lines)
{
	TxtNode *curr = new TxtNode();
	curr->SetType("loader");
	for (unsigned int i=0; i<lines.size(); i++)
	{
		if (IsNewNode(lines[i]))
		{
			TxtNode *new_node = new TxtNode();
			new_node->SetType(GetType(lines[i]));
			curr->Mount(new_node);
			curr = new_node;
			continue;
		}
		if (IsEnding(lines[i]))
		{
			if (curr->GetParent())
			{
				curr = curr->GetParent();
			}
			else
			{
				return curr;
			}
			continue;
		}
		if (IsEquation(lines[i]))
		{
			curr->SetValue(LeftHand(lines[i]), RightHand(lines[i]));
		}
	}

	while (curr->GetParent())
	{
		curr = curr->GetParent();
	}

	return curr;
}

static std::vector<std::string> BreakLines(const std::string &str)
{
	std::vector<std::string> ret;
	std::string curr;
	for (unsigned int i=0; i<str.size(); i++)
	{
		if (str[i] == 0x0A || str[i] == 0x0D)
		{
			if (curr.size()) ret.push_back(curr);
			curr = "";
		}
		else
		{
			curr += str[i];
		}
	}

	if (curr.size()) ret.push_back(curr);

	return ret;
}

TxtNode* LoadTxt(const char *filename)
{
	// Load the file
	FILE *fp = fopen(filename, "rb");
	if (!fp) return 0;
	fseek(fp, 0, SEEK_END);
	unsigned int file_size = ftell(fp);
	fseek(fp, 0, SEEK_SET);

	std::string whole_text;
	for (unsigned int i=0; i<file_size; i++)
	{
		whole_text += getc(fp);
	}

	return LoadText(BreakLines(whole_text));
}
